@@ -30,6 +30,7 @@ public final class ConfigKeys {
 		public static final String ICFG_BACKUP_AUTO_RETAIN = "br.backup.auto.retain";
 		public static final String BCFG_BACKUP_NOTIFY = "br.backup.notify";
 		public static final String BCFG_RESTORE_AUTOPAUSE = "br.restore.autopause";
+		public static final String BCFG_BACKUP_PLUGINS = "br.restore.doplugins";
 	}
 
 	public static class Connection {
@@ -45,6 +46,7 @@ public final class ConfigKeys {
 		public static final String BCFG_PEERCONTROL_TCP_PUBLIC_ENABLE = "peercontrol.tcp.public.enable";
 		public static final String BCFG_PEERCONTROL_UDP_PUBLIC_ENABLE = "peercontrol.udp.public.enable";
 		public static final String BCFG_PEERCONTROL_PREFER_UDP = "peercontrol.prefer.udp";
+		public static final String BCFG_PEERCONTROL_PREFER_IPV6_CONNECTIONS = "peercontrol.prefer.ipv6";
 		public static final String BCFG_HTTP_DATA_LISTEN_PORT_ENABLE = "HTTP.Data.Listen.Port.Enable";
 		public static final String ICFG_HTTP_DATA_LISTEN_PORT = "HTTP.Data.Listen.Port";
 		public static final String ICFG_HTTP_DATA_LISTEN_PORT_OVERRIDE = "HTTP.Data.Listen.Port.Override";
@@ -131,12 +133,14 @@ public final class ConfigKeys {
 		public static final String BCFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART = "Insufficient Space Download Restart Enable";
 		public static final String ICFG_INSUFFICIENT_SPACE_DOWNLOAD_RESTART_MINS = "Insufficient Space Download Restart Period";
 		public static final String BCFG_MISSING_FILE_DOWNLOAD_RESTART = "Missing File Download Restart Enable";
+		public static final String BCFG_SKIP_COMP_DL_FILE_CHECKS = "Skip Complete Download File Checks";
 		public static final String ICFG_MISSING_FILE_DOWNLOAD_RESTART_MINS = "Missing File Download Restart Period";
 		public static final String BCFG_USE_RESUME = "Use Resume";
 		public static final String ICFG_SAVE_RESUME_INTERVAL = "Save Resume Interval";
 		public static final String BCFG_ON_RESUME_RECHECK_ALL = "On Resume Recheck All";
 		public static final String BCFG_FILE_SAVE_PEERS_ENABLE = "File.save.peers.enable";
 		public static final String ICFG_FILE_SAVE_PEERS_MAX = "File.save.peers.max";
+		public static final String BCFG_DISABLE_SAVE_INTERIM_DOWNLOAD_STATE = "Disable Interim Download State Save";
 		public static final String SCFG_PRIORITY_EXTENSIONS = "priorityExtensions";
 		public static final String BCFG_PRIORITY_EXTENSIONS_IGNORE_CASE = "priorityExtensionsIgnoreCase";
 		public static final String SCFG_QUICK_VIEW_EXTS = "quick.view.exts";
@@ -145,6 +149,7 @@ public final class ConfigKeys {
 		public static final String SCFG_RENAME_INCOMPLETE_FILES_EXTENSION = "Rename Incomplete Files Extension";
 		public static final String BCFG_ENABLE_SUBFOLDER_FOR_DND_FILES = "Enable Subfolder for DND Files";
 		public static final String SCFG_SUBFOLDER_FOR_DND_FILES = "Subfolder for DND Files";
+		/** Despite name, this is the DND file prefix when SCFG_SUBFOLDER_FOR_DND_FILES is true */
 		public static final String BCFG_USE_INCOMPLETE_FILE_PREFIX = "Use Incomplete File Prefix";
 		public static final String BCFG_DOWNLOAD_HISTORY_ENABLED = "Download History Enabled";
 		public static final String BCFG_FILES_AUTO_TAG_ENABLE = "Files Auto Tag Enable";
@@ -228,7 +233,10 @@ public final class ConfigKeys {
 		public static final String FCFG_IP_FILTER_BAN_DISCARD_RATIO = "Ip Filter Ban Discard Ratio";
 		public static final String ICFG_IP_FILTER_BAN_DISCARD_MIN_KB = "Ip Filter Ban Discard Min KB";
 		public static final String ICFG_IP_FILTER_BAN_BLOCK_LIMIT = "Ip Filter Ban Block Limit";
+		public static final String ICFG_IP_FILTER_AUTOLOAD_DAYS = "Ip Filter Autoload Days";
+		public static final String ICFG_IP_FILTER_AUTOLOAD_LAST = "Ip Filter Autoload Last Date";
 		public static final String SCFG_IP_FILTER_AUTOLOAD_FILE = "Ip Filter Autoload File";
+		public static final String SCFG_IP_FILTER_V6_AUTOLOAD_FILE = "Ip Filter V6 Autoload File";
 		public static final String BCFG_IP_FILTER_CLEAR_ON_RELOAD = "Ip Filter Clear On Reload";
 		public static final String BCFG_IP_FILTER_ENABLE_DESCRIPTION_CACHE = "Ip Filter Enable Description Cache";
 	}
@@ -395,10 +403,12 @@ public final class ConfigKeys {
 		public static final String BCFG_UP_RATE_LIMITS_INCLUDE_PROTOCOL = "Up Rate Limits Include Protocol";
 		public static final String BCFG_DOWN_RATE_LIMITS_INCLUDE_PROTOCOL = "Down Rate Limits Include Protocol";
 		public static final String BCFG_ALLOW_SAME_IP_PEERS = "Allow Same IP Peers";
+		public static final String ICFG_IPv4_IPv6_CONN_ACTION = "Dual IPV4 IPV6 Connection Action";
 		public static final String BCFG_USE_LAZY_BITFIELD = "Use Lazy Bitfield";
 		public static final String BCFG_PEERCONTROL_HIDE_PIECE = "peercontrol.hide.piece";
 		public static final String BCFG_PEERCONTROL_HIDE_PIECE_DS = "peercontrol.hide.piece.ds";
 		public static final String BCFG_PRIORITIZE_FIRST_PIECE = "Prioritize First Piece";
+		public static final String BCFG_PRIORITIZE_FIRST_PIECE_FORCE = "Prioritize First Piece Force";
 		public static final String BCFG_PRIORITIZE_MOST_COMPLETED_FILES = "Prioritize Most Completed Files";
 		public static final String SCFG_IGNORE_PEER_PORTS = "Ignore.peer.ports";
 		public static final String BCFG_LAN_SPEED_ENABLED = "LAN Speed Enabled";
@@ -427,4 +437,4 @@ public final class ConfigKeys {
 	public static class Tag {
 		public static final String BCFG_TRACKER_AUTO_TAG_INTERESTING_TRACKERS = "Auto Tag Interesting Trackers";
 	}
-}
\ No newline at end of file
+}
