@@ -229,6 +229,7 @@ public class ConfigurationDefaults {
     def.put("Merge Same Size Files Extended", FALSE );
     def.put("Merge Same Size Files Tolerance", ZERO );
     def.put(ConfigKeys.File.ICFG_MERGE_SAME_SIZE_FILES_MIN_PIECES, 5 );
+    def.put(ConfigKeys.File.BCFG_SKIP_COMP_DL_FILE_CHECKS, FALSE );
     def.put("Stop Ratio", new Float(0));
     def.put("Stop Peers Ratio", ZERO);
     def.put("Disconnect Seed", TRUE);
@@ -255,10 +256,14 @@ public class ConfigurationDefaults {
     def.put("Ip Filter Ban Discard Min KB", new Long(128));
     def.put("Ip Filter Banning Persistent", TRUE);
     def.put("Ip Filter Enable Description Cache", TRUE);
-    def.put("Ip Filter Autoload File", "");
+    def.put(ConfigKeys.IPFilter.SCFG_IP_FILTER_AUTOLOAD_FILE, "" );
+    def.put(ConfigKeys.IPFilter.SCFG_IP_FILTER_V6_AUTOLOAD_FILE, "");
+    def.put(ConfigKeys.IPFilter.ICFG_IP_FILTER_AUTOLOAD_LAST, ZERO );
+    def.put(ConfigKeys.IPFilter.ICFG_IP_FILTER_AUTOLOAD_DAYS, 7L );
     def.put("Ip Filter Clear On Reload", TRUE );
 
     def.put("Allow Same IP Peers",FALSE);
+    def.put(ConfigKeys.Transfer.ICFG_IPv4_IPv6_CONN_ACTION, ZERO );
     def.put("Use Super Seeding",FALSE);
     def.put("StartStopManager_bAddForDownloadingSR1", TRUE);
     
@@ -298,15 +303,15 @@ public class ConfigurationDefaults {
     try {
       File dirDownloads = PlatformManagerFactory.getPlatformManager().getLocation(PlatformManager.LOC_DOWNLOADS);
       if (dirDownloads != null) {
-        f = new File(dirDownloads, Constants.APP_NAME);
+        f = FileUtil.newFile(dirDownloads, Constants.APP_NAME);
       }
     } catch (Throwable ignore) {
     }
     if (f == null) {
-      f = new File(SystemProperties.getDocPath(), "Downloads");
+      f = FileUtil.newFile(SystemProperties.getDocPath(), "Downloads");
     }
 	}else{
-    f = new File(SystemProperties.getDocPath(),  Constants.APP_NAME + " Downloads");
+    f = FileUtil.newFile(SystemProperties.getDocPath(),  Constants.APP_NAME + " Downloads");
 	}
 
 	def.put("Default save path", f.getAbsolutePath());
@@ -402,7 +407,8 @@ public class ConfigurationDefaults {
     def.put( "Watch Torrent Always Rename", FALSE );
     def.put( "Watch Torrent Folder Path", "" );
     def.put( "Watch Torrent Folder Path Count", ONE );
-    def.put( "Prioritize First Piece", FALSE );
+    def.put( ConfigKeys.Transfer.BCFG_PRIORITIZE_FIRST_PIECE, FALSE );
+    def.put( ConfigKeys.Transfer.BCFG_PRIORITIZE_FIRST_PIECE_FORCE, FALSE );
     def.put( "Prioritize Most Completed Files", FALSE );
     def.put( "Piece Picker Request Hint Enabled", TRUE );
     def.put( "Use Lazy Bitfield", FALSE );
@@ -636,6 +642,7 @@ public class ConfigurationDefaults {
     def.put( "peercontrol.hide.piece", FALSE );
     def.put( "peercontrol.scheduler.use.priorities", TRUE );
     def.put( "peercontrol.prefer.udp", FALSE );
+    def.put( ConfigKeys.Connection.BCFG_PEERCONTROL_PREFER_IPV6_CONNECTIONS, FALSE );
 
     def.put( "File.truncate.if.too.large", FALSE);
     def.put( "Enable System Tray", TRUE);
@@ -745,7 +752,8 @@ public class ConfigurationDefaults {
     def.put( "br.backup.auto.everydays", ONE );
     def.put( "br.backup.auto.everyhours", ZERO );
     def.put( "br.backup.auto.retain", 5L );
-
+    def.put( ConfigKeys.BackupRestore.BCFG_BACKUP_PLUGINS, TRUE );
+    
     //temp section for SpeedManagerAlgorithmProviderV2
 
     try{
