@@ -373,6 +373,115 @@ DHTTransportUDPImpl
 		logger.log( "Initial external address: " + address );
 
 		local_contact = new DHTTransportUDPContactImpl( true, this, address, address, protocol_version, random.nextInt(), 0, (byte)0 );
+		
+		if ( network == DHT.NW_AZ_MAIN || network == DHT.NW_AZ_MAIN_V6 ){
+			
+				// local provider for use by the plugin for xfer
+			
+			DHTUDPUtils.registerAlternativeNetwork( 
+				new DHTTransportAlternativeNetwork(){
+					
+					@Override
+					public int 
+					getNetworkType()
+					{
+						return( network == DHT.NW_AZ_MAIN?DHTTransportAlternativeNetwork.AT_BIGLYBT_IPV4:DHTTransportAlternativeNetwork.AT_BIGLYBT_IPV6 );
+					}
+					
+					@Override
+					public List<DHTTransportAlternativeContact> 
+					getContacts(
+						int		max )
+					{
+						List<DHTTransportAlternativeContact>	result = new ArrayList<>( max );
+						
+						if ( max > 0 ){
+							
+							try{
+								this_mon.enter();
+								
+								long start = SystemTime.getMonotonousTime();
+								
+								int net = getNetworkType();
+								
+								for ( DHTTransportContact c: routable_contact_history.values()){
+	
+									InetSocketAddress ia = c.getAddress();
+									
+									Map<String,Object> properties = new HashMap<>();
+									
+									properties.put( "a",  ia.getAddress().getHostAddress());
+									properties.put( "p",  (long)ia.getPort());
+									
+									int _id;
+									
+									try{
+										_id = Arrays.hashCode( BEncoder.encode( properties ));
+										
+									}catch( Throwable e ){
+										
+										_id = 0;
+									}
+									
+									int id = _id;
+									
+									result.add( 
+										new DHTTransportAlternativeContact()
+										{
+											public int
+											getNetworkType()
+											{
+												return( net );
+											}
+	
+											public int
+											getVersion()
+											{
+												return( 1 );
+											}
+				
+											public int
+											getID()
+											{
+												return( id );
+											}
+	
+											public int
+											getLastAlive()
+											{
+												return( 0 );	// deprecated
+											}
+											
+											public int
+											getAge()
+											{
+												return((int)((SystemTime.getMonotonousTime() - start )/1000 ));
+											}
+	
+											public Map<String,Object>
+											getProperties()
+											{
+												return( properties );
+											}
+										});
+									
+									max--;
+									
+									if ( max == 0 ){
+										
+										break;
+									}
+								}
+							}finally{
+	
+								this_mon.exit();
+							}
+						}
+						
+						return( result );
+					}
+				});
+		}
 	}
 
 	protected void
@@ -680,7 +789,8 @@ DHTTransportUDPImpl
 	@Override
 	public void 
 	setExplicitBindAddress(
-		InetAddress		address )
+		InetAddress		address,
+		boolean			autoDelegate )
 	{
 		try{
 			class_mon.enter();
@@ -692,7 +802,7 @@ DHTTransportUDPImpl
 				external_address = address.getHostAddress();
 			}
 			
-			packet_handler.getPacketHandler().setExplicitBindAddress( address );
+			packet_handler.getPacketHandler().setExplicitBindAddress( address, autoDelegate );
 		
 		}finally{
 			
@@ -3344,7 +3454,7 @@ outer:
 	private final Object	alt_net_providers_lock = new Object();
 
 	{
-		for ( Integer net: DHTTransportAlternativeNetwork.AT_ALL ){
+		for ( Integer net: DHTTransportAlternativeNetwork.AT_ALL_PUB ){
 
 			alt_net_states.put( net, new DHTTransportAlternativeNetworkImpl( net ));
 		}
