@@ -49,8 +49,8 @@ public class
 DiskManagerFileInfoImpl
 	implements DiskManagerFileInfo, CacheFileOwner
 {
-  private String				root_dir;
-  private final File			relative_file;
+  private File					root_dir;
+  private final String	relative_file;
 
   final int			file_index;
   CacheFile		cache_file;
@@ -72,8 +72,8 @@ DiskManagerFileInfoImpl
   public
   DiskManagerFileInfoImpl(
 	DiskManagerHelper	_disk_manager,
-  	String				_root_dir,
-  	File				_relative_file,
+  	File				_root_dir,
+  	String				_relative_file,
   	int					_file_index,
 	TOTorrentFile		_torrent_file,
 	int					_storage_type )
@@ -83,14 +83,14 @@ DiskManagerFileInfoImpl
     diskManager 	= _disk_manager;
     torrent_file	= _torrent_file;
 
-    root_dir		= _root_dir.endsWith(File.separator)?_root_dir:(_root_dir + File.separator);
+    root_dir		= _root_dir;
     relative_file	= _relative_file;
 
     file_index		= _file_index;
 
     int	cache_st = DiskManagerUtil.convertDMStorageTypeToCache( _storage_type );
 
-  	cache_file = CacheFileManagerFactory.getSingleton().createFile( this, FileUtil.newFile( root_dir + relative_file.toString()), cache_st);
+  	cache_file = CacheFileManagerFactory.getSingleton().createFile( this, FileUtil.newFile( root_dir, relative_file), cache_st, false );
 
   	if ( cache_st == CacheFile.CT_COMPACT || cache_st == CacheFile.CT_PIECE_REORDER_COMPACT ){
 
@@ -135,9 +135,14 @@ DiskManagerFileInfoImpl
   	cache_file.flushCache();
   }
 
+  @Override
+  public boolean exists(){
+	return( cache_file.exists());
+  }
+  
   public void
   moveFile(
-	String						new_root_dir,
+  	File						new_root_dir,
   	File						new_absolute_file,
   	boolean						link_only,
   	FileUtil.ProgressListener	pl )
@@ -149,7 +154,7 @@ DiskManagerFileInfoImpl
 		  cache_file.moveFile( new_absolute_file, pl );
 	  }
 
-	 root_dir	= new_root_dir.endsWith(File.separator)?new_root_dir:(new_root_dir + File.separator);
+	 root_dir	= new_root_dir;
   }
 
   public void
@@ -215,19 +220,19 @@ DiskManagerFileInfoImpl
   public File
   getFile(
 	boolean	follow_link )
-  	{
-	  if ( follow_link ){
+	{
+		File file = FileUtil.newFile(root_dir, relative_file);
 
-		  File	res = getLink();
+		if (!follow_link) {
 
-		  if ( res != null ){
+			return file;
+		}
 
-			  return( res );
-		  }
-	  }
-
-	  return( FileUtil.newFile( root_dir + relative_file.toString()));
-  	}
+		// Same as getLink(), except saves redundant getFile(false) call
+		File	res = diskManager.getDownloadState().getFileLink(file_index, file);
+		
+		return res == null ? file : res;
+	}
 
   	@Override
 	  public TOTorrentFile
@@ -282,11 +287,11 @@ DiskManagerFileInfoImpl
 	}
 
 	@Override
-	public boolean setStorageType(int type) {
+	public boolean setStorageType(int type, boolean force ) {
 		DiskManagerFileInfoSet set = diskManager.getFileSet();
 		boolean[] toSet = new boolean[set.nbFiles()];
 		toSet[file_index] = true;
-		return set.setStorageTypes(toSet, type)[file_index];
+		return set.setStorageTypes(toSet, type, force )[file_index];
 	}
 
 	@Override
