@@ -20,48 +20,32 @@
 
 package com.biglybt.core.disk.impl;
 
-import java.io.File;
-import java.io.IOException;
-import java.io.UnsupportedEncodingException;
-import java.lang.ref.WeakReference;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.List;
-import java.util.Locale;
-import java.util.Map;
-
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.*;
-import com.biglybt.core.disk.DiskManager.OperationStatus;
-import com.biglybt.core.disk.impl.piecemapper.DMPieceList;
-import com.biglybt.core.disk.impl.piecemapper.DMPieceMap;
-import com.biglybt.core.disk.impl.piecemapper.DMPieceMapper;
-import com.biglybt.core.disk.impl.piecemapper.DMPieceMapperFactory;
-import com.biglybt.core.disk.impl.piecemapper.DMPieceMapperFile;
+import com.biglybt.core.disk.impl.piecemapper.*;
 import com.biglybt.core.disk.impl.resume.RDResumeHandler;
 import com.biglybt.core.diskmanager.access.DiskAccessController;
 import com.biglybt.core.diskmanager.cache.CacheFile;
 import com.biglybt.core.diskmanager.cache.CacheFileManagerFactory;
 import com.biglybt.core.diskmanager.cache.CacheFileOwner;
-import com.biglybt.core.download.DownloadManager;
-import com.biglybt.core.download.DownloadManagerException;
-import com.biglybt.core.download.DownloadManagerState;
-import com.biglybt.core.download.DownloadManagerStats;
+import com.biglybt.core.download.*;
 import com.biglybt.core.download.impl.DownloadManagerStatsImpl;
 import com.biglybt.core.internat.LocaleTorrentUtil;
 import com.biglybt.core.internat.LocaleUtilDecoder;
-import com.biglybt.core.internat.MessageText;
-import com.biglybt.core.logging.LogAlert;
-import com.biglybt.core.logging.LogEvent;
-import com.biglybt.core.logging.LogIDs;
-import com.biglybt.core.logging.Logger;
+import com.biglybt.core.logging.*;
 import com.biglybt.core.peermanager.piecepicker.util.BitFlags;
 import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrent.TOTorrentException;
 import com.biglybt.core.torrent.TOTorrentFile;
 import com.biglybt.core.util.*;
 
+import java.io.File;
+import java.io.IOException;
+import java.io.UnsupportedEncodingException;
+import java.lang.ref.WeakReference;
+import java.util.*;
+
 public class
 DiskManagerUtil
 {
@@ -209,7 +193,7 @@ DiskManagerUtil
 					if( st == DiskManagerFileInfo.ST_COMPACT || st == DiskManagerFileInfo.ST_REORDER_COMPACT ){
 						currentFile.delete();
 					}
-				} else if(allowAlloction && !currentFile.exists())	{
+				} else if(allowAlloction && !files[i].exists())	{
 					/*
 					 * file must exist, does not exist and we probably just changed to linear
 					 * mode, assume that (re)allocation of adjacent files is necessary
@@ -242,14 +226,17 @@ DiskManagerUtil
 		    	
 		    	if ( !FileUtil.areFilePathsIdentical( to_link, existing_file )){
 	        		
-		            if ( !FileUtil.renameFile( existing_file, to_link )){
-
-		            	String error = "Failed to rename '" + existing_file.toString() + "' to '" + to_link.toString() + "'";
-		            			
-		                Logger.log(new LogAlert(download_manager, LogAlert.REPEATABLE, LogAlert.AT_ERROR, error ));
-
-		                return( error );
-		            }
+		    		if ( existing_file.exists()){
+		    			
+			            if ( !FileUtil.renameFile( existing_file, to_link )){
+	
+			            	String error = "Failed to rename '" + existing_file.toString() + "' to '" + to_link.toString() + "'";
+			            			
+			                Logger.log(new LogAlert(download_manager, LogAlert.REPEATABLE, LogAlert.AT_ERROR, error ));
+	
+			                return( error );
+			            }
+		    		}
 		    	}
 		    }else{
 
@@ -370,17 +357,15 @@ DiskManagerUtil
 	        return( new DiskManagerFileInfoSetImpl(new DiskManagerFileInfoImpl[0],null) );
 	    }
 
-	    String  tempRootDir = download_manager.getAbsoluteSaveLocation().getParent();
+		File tempRootDir = download_manager.getAbsoluteSaveLocation().getParentFile();
 
-	    if(tempRootDir == null) // in case we alraedy are at the root
-	    	tempRootDir = download_manager.getAbsoluteSaveLocation().getPath();
+		if (tempRootDir == null) { // in case we already are at the root
+			tempRootDir = download_manager.getAbsoluteSaveLocation();
+		}
 
-
-	    if ( !torrent.isSimpleTorrent()){
-	    	tempRootDir += File.separator + download_manager.getAbsoluteSaveLocation().getName();
-	    }
-
-	    tempRootDir    += File.separator;
+		if ( !torrent.isSimpleTorrent()){
+			tempRootDir = FileUtil.newFile(tempRootDir, download_manager.getAbsoluteSaveLocation().getName());
+		}
 
 	    	// prevent attempted state saves and associated nastyness during population of
 	    	// the file skeleton entries
@@ -388,7 +373,7 @@ DiskManagerUtil
 	    final boolean[] loading = { true };
 
 	    try{
-		    final String root_dir = StringInterner.intern(tempRootDir) ;
+		    final File root_dir = tempRootDir;
 
 		    try{
 		        final LocaleUtilDecoder locale_decoder = LocaleTorrentUtil.getTorrentEncoding( torrent );
@@ -522,7 +507,7 @@ DiskManagerUtil
 					}
 
 					@Override
-					public boolean[] setStorageTypes(boolean[] toChange, int newStorageType) {
+					public boolean[] setStorageTypes(boolean[] toChange, int newStorageType, boolean force) {
 						if(toChange.length != res.length)
 							throw new IllegalArgumentException("array length mismatches the number of files");
 
@@ -596,7 +581,7 @@ DiskManagerUtil
 													}
 												},
 												target_file,
-												DiskManagerUtil.convertDMStorageTypeToCache( newStorageType ));
+												DiskManagerUtil.convertDMStorageTypeToCache( newStorageType ), force );
 
 										cache_file.getLength();	// need this to trigger recovery for re-order files :(
 
@@ -991,6 +976,15 @@ DiskManagerUtil
 		            		return( download_manager );
 		            	}
 
+		            	@Override
+		            	public boolean exists(){
+		            		if ( torrent_file.isPadFile()){
+		            			return( true );
+		            		}else{
+		            			return( getFile( true ).exists());
+		            		}
+		            	}
+		            	  
 		            	@Override
 			            public File
 		            	getFile(
@@ -1016,7 +1010,7 @@ DiskManagerUtil
 
 		            		TOTorrent tor = download_manager.getTorrent();
 
-		            		String  path_str = root_dir;
+		            		File  dataPath = root_dir;
 		            		File simpleFile = null;
 
 		            		// for a simple torrent the target file can be changed
@@ -1044,11 +1038,11 @@ DiskManagerUtil
 
 		            				comp = FileUtil.convertOSSpecificChars( comp,  j != path_comps.length-1 );
 
-		            				path_str += (j==0?"":File.separator) + comp;
+		            				dataPath = FileUtil.newFile(dataPath, comp);
 		            			}
 		            		}
 
-		            		dataFile = new WeakReference(toReturn = simpleFile != null ? simpleFile : FileUtil.newFile( path_str ));
+		            		dataFile = new WeakReference(toReturn = simpleFile != null ? simpleFile : dataPath );
 
 		            		//System.out.println("new file:"+toReturn);
 		            		return toReturn;
@@ -1123,10 +1117,10 @@ DiskManagerUtil
 	                	}
 
 	                	@Override
-		                public boolean setStorageType(int type) {
+		                public boolean setStorageType(int type, boolean force) {
 	                		boolean[] change = new boolean[res.length];
 	                		change[file_index] = true;
-	                		return fileSetSkeleton.setStorageTypes(change, type)[file_index];
+	                		return fileSetSkeleton.setStorageTypes(change, type, force)[file_index];
 	                	}
 
 	                	@Override
@@ -1192,7 +1186,7 @@ DiskManagerUtil
 	                								}
 	                							},
 	                							getFile( true ),
-	                							convertDMStorageTypeToCache( type ));
+	                							convertDMStorageTypeToCache( type ), false );
 
 	                				}catch( Throwable e ){
 
@@ -1619,6 +1613,11 @@ DiskManagerUtil
 			
 			TOTorrent torrent = dm.getTorrent();
 			
+			if ( torrent == null ){
+				
+				return( new DiskManagerPiece[0] );
+			}
+			
 	        DMPieceMapper piece_mapper    = DMPieceMapperFactory.create( torrent );
 	
 	        LocaleUtilDecoder   locale_decoder = LocaleTorrentUtil.getTorrentEncoding( torrent );
@@ -1644,6 +1643,18 @@ DiskManagerUtil
 	        DiskManagerHelper disk_manager =
 	        	new DiskManagerHelper(){
 					
+	        		@Override
+		        	public String getDisplayName(){
+		        		return( "" );
+		        	}
+	        		
+	        		@Override
+	        		public DownloadManager
+	        		getDownload()
+	        		{
+	        			return( dm );
+	        		}
+	        		
 					@Override
 					public boolean stop(boolean closing){
 						return false;
@@ -1813,8 +1824,8 @@ DiskManagerUtil
 					}
 					
 					@Override
-					public int getMoveProgress(){
-						return 0;
+					public long[] getMoveProgress(){
+						return null;
 					}
 					
 					@Override
