@@ -19,6 +19,7 @@
 
 package com.biglybt.core.disk.impl.access.impl;
 
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -305,7 +306,7 @@ DMCheckerImpl
 	 	new AEThread2("DMChecker::completeRecheck", true )
 			{
 		  		@Override
-				  public void
+				public void
 				run()
 		  		{
 		  			DiskManagerRecheckInstance	recheck_inst = disk_manager.getRecheckScheduler().register( disk_manager, true );
@@ -603,7 +604,16 @@ DMCheckerImpl
 							}
 						}
 					}
-				}, read_flush );
+					
+					@Override
+					public boolean 
+					hashRequest(
+						int 			piece_number, 
+						HashListener 	hash_listener)
+					{
+						return( listener.hashRequest( piece_number, hash_listener));
+					}
+				}, read_flush, false );
 	}
 
 
@@ -611,7 +621,8 @@ DMCheckerImpl
 	enqueueCheckRequestSupport(
 		final DiskManagerCheckRequest			request,
 		final DiskManagerCheckRequestListener	listener,
-		boolean									read_flush )
+		boolean									read_flush,
+		boolean									hash_requested )
 	{
 		if ( !checking_enabled ){
 
@@ -625,7 +636,48 @@ DMCheckerImpl
 		try{
 
 			final byte[]	required_hash = disk_manager.getPieceHash(pieceNumber);
-
+			
+			if ( required_hash == null ){
+				
+					// v2 torrent and hash not yet available, make an attempt to get it
+					// normally it will be available as requested prior to data
+				
+				if ( !hash_requested ){
+				
+					if ( listener.hashRequest( 
+							pieceNumber,
+							new DiskManagerCheckRequestListener.HashListener()
+							{
+								@Override
+								public int 
+								getPieceNumber()
+								{
+									return( pieceNumber );
+								}
+								
+								public void
+								complete(
+									boolean success )
+								{
+										// ignore success, just do it again and probably fail
+									
+									if ( !success ){
+										
+										Debug.out( "Failed to get hash for piece " + request.getPieceNumber());
+										
+									}
+									enqueueCheckRequestSupport( request, listener, read_flush, true );
+								}
+							})){
+					
+						return;
+					}
+				}				
+					
+				listener.checkFailed( request, new Exception( "V2 hash for piece " + pieceNumber + " not available" ));
+				
+				return;
+			}
 				// quick check that the files that make up this piece are at least big enough
 				// to warrant reading the data to check
 
@@ -773,8 +825,44 @@ DMCheckerImpl
 							try{
 						    	final	DirectByteBuffer	f_buffer	= buffer;
 
+						    	int piece_length = disk_manager.getPieceLength();
+						    	
+						    	int hash_version = required_hash.length==20?1:2;
+						    	
+						    	long	v2_file_length;
+						    	
+						    	ByteBuffer byte_buffer = buffer.getBuffer(DirectByteBuffer.SS_DW);
+						    	
+						    	if ( hash_version == 2 ){
+						    		
+						    		DMPieceMapEntry piece_entry = pieceList.get(0);
+						    		
+						    		v2_file_length = piece_entry.getFile().getLength();					    		
+						    		
+										// with v2 a piece only contains > 1 file if the second file is a dummy padding file added
+										// to 'make things work'
+
+						    		if ( pieceList.size() == 2 ){
+
+							    		int v2_piece_length = piece_entry.getLength();
+							    		
+							    		if ( v2_piece_length < piece_length ){
+							    			
+							    				// hasher will pad appropriately
+							    			
+							    			byte_buffer.limit( byte_buffer.position() + v2_piece_length );
+							    		}
+						    		}
+						    	}else{
+						    		
+						    		v2_file_length = -1;
+						    	}
+						    	
 							   	ConcurrentHasher.getSingleton().addRequest(
-						    			buffer.getBuffer(DirectByteBuffer.SS_DW),
+						    			byte_buffer,
+						    			hash_version,
+						    			piece_length,
+						    			v2_file_length,
 										new ConcurrentHasherRequestListener()
 										{
 						    				@Override
