@@ -22,6 +22,7 @@ package com.biglybt.core.disk.impl.resume;
 import java.util.*;
 
 import com.biglybt.core.config.COConfigurationManager;
+import com.biglybt.core.config.ConfigKeys;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.*;
 import com.biglybt.core.disk.impl.DiskManagerFileInfoImpl;
@@ -67,13 +68,15 @@ RDResumeHandler
 
 	static boolean	use_fast_resume;
 	static boolean	use_fast_resume_recheck_all;
+	static boolean	skip_comp_dl_file_checks;
 
 	static{
 
 		COConfigurationManager.addAndFireParameterListeners(
 			new String[]{
 					"Use Resume",
-					"On Resume Recheck All" },
+					"On Resume Recheck All",
+					ConfigKeys.File.BCFG_SKIP_COMP_DL_FILE_CHECKS },
 			new ParameterListener() {
 	    	    @Override
 		        public void
@@ -82,6 +85,7 @@ RDResumeHandler
 	    	    {
 	    	    	use_fast_resume				= COConfigurationManager.getBooleanParameter("Use Resume");
 	    	    	use_fast_resume_recheck_all	= COConfigurationManager.getBooleanParameter("On Resume Recheck All");
+	    	    	skip_comp_dl_file_checks	= COConfigurationManager.getBooleanParameter(ConfigKeys.File.BCFG_SKIP_COMP_DL_FILE_CHECKS);
 	    	    }
 	    	 });
 	}
@@ -136,12 +140,8 @@ RDResumeHandler
 	{
 		//long	start = System.currentTimeMillis();
 
-		DiskManagerRecheckInstance	recheck_inst = disk_manager.getRecheckScheduler().register( disk_manager, false );
-			
-		int concurrency = recheck_inst.getPieceConcurrency();
-		
-        final AESemaphore	 run_sem = new AESemaphore( "RDResumeHandler::checkAllPieces:runsem", concurrency );
-
+		DiskManagerRecheckInstance	recheck_inst_maybe_null = null;
+			 
         final List<DiskManagerCheckRequest>	failed_pieces = new ArrayList<>();
 
         listener.percentDone( 0 );
@@ -168,21 +168,30 @@ RDResumeHandler
 				DiskManagerPiece[]	pieces	= disk_manager.getPieces();
 
 
-				// calculate the current file sizes up front for performance reasons
-				DiskManagerFileInfo[]	files = disk_manager.getFiles();
-				Map	file_sizes = new HashMap();
-
-				for (int i=0;i<files.length;i++){
-					try{
-						Long	len = new Long(((DiskManagerFileInfoImpl)files[i]).getCacheFile().getLength());
-						file_sizes.put( files[i], len );
-					}catch( CacheFileManagerException e ){
-						Debug.printStackTrace(e);
+					// calculate the current file sizes up front for performance reasons
+				
+				final Map<DiskManagerFileInfo,Long>	file_sizes;
+								
+				if ( skip_comp_dl_file_checks && disk_manager.getDownloadManager().isDownloadComplete(false)){
+					
+					file_sizes = null;
+					
+				}else{
+					
+					DiskManagerFileInfo[]	files = disk_manager.getFiles();
+					
+					file_sizes = new HashMap<>();
+	
+					for (int i=0;i<files.length;i++){
+						try{
+							Long	len = new Long(((DiskManagerFileInfoImpl)files[i]).getCacheFile().getLength());
+							file_sizes.put( files[i], len );
+						}catch( CacheFileManagerException e ){
+							Debug.printStackTrace(e);
+						}
 					}
 				}
 
-
-
 				if ( resumeEnabled ){
 
 					boolean resumeValid = false;
@@ -330,45 +339,48 @@ RDResumeHandler
 
 								// at least check that file sizes are OK for this piece to be valid
 
-							DMPieceList list = disk_manager.getPieceList(i);
+							if ( file_sizes != null ){
 
-							for (int j=0;j<list.size();j++){
+								DMPieceList list = disk_manager.getPieceList(i);
 
-								DMPieceMapEntry	entry = list.get(j);
+								for (int j=0;j<list.size();j++){
 
-								Long	file_size 		= (Long)file_sizes.get(entry.getFile());
-
-								if ( file_size == null ){
-
-									piece_state	= PIECE_NOT_DONE;
-									pieceCannotExist = true;
-
-									if (Logger.isEnabled())
-										Logger.log(new LogEvent(disk_manager, LOGID,
-												LogEvent.LT_WARNING, "Piece #" + i
-														+ ": file is missing, " + "fails re-check."));
-
-									break;
-								}
-
-								long	expected_size 	= entry.getOffset() + entry.getLength();
-
-								if ( file_size.longValue() < expected_size ){
-
-									piece_state	= PIECE_NOT_DONE;
-									pieceCannotExist = true;
-
-									if ( file_size > 0 ){
-											// we get 0 for DND files, don't bother logging
-										
+									DMPieceMapEntry	entry = list.get(j);
+									
+									Long	file_size 		= (Long)file_sizes.get(entry.getFile());
+	
+									if ( file_size == null ){
+	
+										piece_state	= PIECE_NOT_DONE;
+										pieceCannotExist = true;
+	
 										if (Logger.isEnabled())
 											Logger.log(new LogEvent(disk_manager, LOGID,
 													LogEvent.LT_WARNING, "Piece #" + i
-															+ ": file is too small, fails re-check. File size = "
-															+ file_size + ", piece needs " + expected_size));
+															+ ": file is missing, " + "fails re-check."));
+	
+										break;
+									}
+								
+									long	expected_size 	= entry.getOffset() + entry.getLength();
+	
+									if ( file_size.longValue() < expected_size ){
+	
+										piece_state	= PIECE_NOT_DONE;
+										pieceCannotExist = true;
+	
+										if ( file_size > 0 ){
+												// we get 0 for DND files, don't bother logging
+											
+											if (Logger.isEnabled())
+												Logger.log(new LogEvent(disk_manager, LOGID,
+														LogEvent.LT_WARNING, "Piece #" + i
+																+ ": file is too small, fails re-check. File size = "
+																+ file_size + ", piece needs " + expected_size));
+										}
+										
+										break;
 									}
-									
-									break;
 								}
 							}
 						}
@@ -389,9 +401,18 @@ RDResumeHandler
 							if(pieceCannotExist)
 							{
 								dm_piece.setDone( false );
+								
 							} else if ( piece_state == PIECE_RECHECK_REQUIRED || !resumeValid ){
+								
 
-								run_sem.reserve();
+								if ( recheck_inst_maybe_null == null ){
+									
+									recheck_inst_maybe_null = disk_manager.getRecheckScheduler().register( disk_manager, false );
+								}
+								
+								final DiskManagerRecheckInstance recheck_inst = recheck_inst_maybe_null;
+								
+								recheck_inst.reserveSlot();
 
 								while( !stopped ){
 
@@ -462,7 +483,7 @@ RDResumeHandler
 												protected void
 												complete()
 												{
-													run_sem.release();
+													recheck_inst.releaseSlot();
 
 													pending_checks_sem.release();
 												}
@@ -524,31 +545,43 @@ RDResumeHandler
 						boolean pieceCannotExist = false;
 
 						// check if there is an underlying file for this piece, if not set it to not done
-						DMPieceList list = disk_manager.getPieceList(i);
-
-						for (int j=0;j<list.size();j++){
-							DMPieceMapEntry	entry = list.get(j);
-
-							Long	file_size 		= (Long)file_sizes.get(entry.getFile());
-							if ( file_size == null ){
-								pieceCannotExist = true;
-								break;
-							}
-
-							long	expected_size 	= entry.getOffset() + entry.getLength();
-							if ( file_size.longValue() < expected_size ){
-								pieceCannotExist = true;
-								break;
+						
+						if ( file_sizes != null ){
+							
+							DMPieceList list = disk_manager.getPieceList(i);
+	
+							for (int j=0;j<list.size();j++){
+								DMPieceMapEntry	entry = list.get(j);
+	
+								Long	file_size 		= (Long)file_sizes.get(entry.getFile());
+								if ( file_size == null ){
+									pieceCannotExist = true;
+									break;
+								}
+	
+								long	expected_size 	= entry.getOffset() + entry.getLength();
+								if ( file_size.longValue() < expected_size ){
+									pieceCannotExist = true;
+									break;
+								}
 							}
 						}
-
-						if(pieceCannotExist)
-						{
+						
+						if ( pieceCannotExist ){
+						
 							disk_manager.getPiece(i).setDone(false);
+							
 							continue;
 						}
 
-						run_sem.reserve();
+						if ( recheck_inst_maybe_null == null ){
+							
+							recheck_inst_maybe_null = disk_manager.getRecheckScheduler().register( disk_manager, false );
+						}
+						
+						final DiskManagerRecheckInstance recheck_inst = recheck_inst_maybe_null;
+
+						recheck_inst.reserveSlot();
 
 						while( ! stopped ){
 
@@ -619,8 +652,8 @@ RDResumeHandler
 										protected void
 										complete()
 										{
-											run_sem.release();
-
+											recheck_inst.releaseSlot();
+											
 											pending_checks_sem.release();
 										}
 									});
@@ -649,14 +682,24 @@ RDResumeHandler
 
 					for ( int i=0;i<piece_hashes.length;i++){
 
-						hash_map.put( piece_hashes[i], i );
+						byte[] hash = piece_hashes[i];
+						
+						if ( hash != null ){
+						
+							hash_map.put( hash, i );
+						}
+					}
+					
+					if ( recheck_inst_maybe_null == null ){
+						
+						recheck_inst_maybe_null = disk_manager.getRecheckScheduler().register( disk_manager, false );
 					}
 
 					for ( DiskManagerCheckRequest request: failed_pieces ){
 
 						while( ! stopped ){
 
-							if ( recheck_inst.getPermission()){
+							if ( recheck_inst_maybe_null.getPermission()){
 
 								break;
 							}
@@ -818,7 +861,10 @@ RDResumeHandler
 
 		}finally{
 
-			recheck_inst.unregister();
+			if ( recheck_inst_maybe_null != null ){
+			
+				recheck_inst_maybe_null.unregister();
+			}
 
 			// System.out.println( "Check of '" + disk_manager.getDownloadManager().getDisplayName() + "' completed in " + (System.currentTimeMillis() - start));
 		}
