@@ -34,6 +34,7 @@ import com.biglybt.core.diskmanager.file.FMFile;
 import com.biglybt.core.diskmanager.file.FMFileManagerException;
 import com.biglybt.core.diskmanager.file.FMFileOwner;
 import com.biglybt.core.diskmanager.file.impl.FMFileAccess.FileAccessor;
+import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrent.TOTorrentFile;
 import com.biglybt.core.util.*;
 
@@ -82,6 +83,8 @@ FMFileImpl
 
 	protected final AEMonitor			this_mon	= new AEMonitor( "FMFile" );
 
+	private volatile long		length_cache = -1;
+	
 	private boolean				clone;
 
 	protected
@@ -89,7 +92,8 @@ FMFileImpl
 		FMFileOwner			_owner,
 		FMFileManagerImpl	_manager,
 		File				_file,
-		int					_type )
+		int					_type,
+		boolean 			_force )
 
 		throws FMFileManagerException
 	{
@@ -106,11 +110,13 @@ FMFileImpl
 
 		try{
 
+			String linked_path = linked_file.getPath();
 			try {
 
 				canonical_path = linked_file.getCanonicalPath();
-				if(canonical_path.equals(linked_file.getPath()))
-					canonical_path = linked_file.getPath();
+				if(canonical_path.equals(linked_path)) {
+					canonical_path = linked_path;
+				}
 
 			}catch( IOException ioe ) {
 
@@ -121,7 +127,7 @@ FMFileImpl
 		          String abs_path = linked_file.getAbsolutePath();
 
 		          String error = "Caught 'There are no more files' exception during file.getCanonicalPath(). " +
-		                         "os=[" +Constants.OSName+ "], file.getPath()=[" +linked_file.getPath()+ "], file.getAbsolutePath()=[" +abs_path+ "]. ";
+		                         "os=[" +Constants.OSName+ "], file.getPath()=[" + linked_path + "], file.getAbsolutePath()=[" +abs_path+ "]. ";
 
 		          Debug.out( error, ioe );
 		        }
@@ -135,10 +141,8 @@ FMFileImpl
 
 			file_reserved	= true;
 
-			file_access = new FMFileAccessController( this, _type );
+			file_access = new FMFileAccessController( this, _type, _force );
 
-			last_modified = linked_file.lastModified();
-			
 			ok	= true;
 
 		}catch( Throwable e ){
@@ -180,10 +184,8 @@ FMFileImpl
 		clone			= true;
 
 		try{
-			file_access = new FMFileAccessController( this, basis.file_access.getStorageType());
+			file_access = new FMFileAccessController( this, basis.file_access.getStorageType(), false );
 
-			last_modified = linked_file.lastModified();
-			
 		}catch( Throwable e ){
 
 			if ( e instanceof FMFileManagerException ){
@@ -232,7 +234,8 @@ FMFileImpl
 	@Override
 	public void
 	setStorageType(
-		int		new_type )
+		int		new_type,
+		boolean	force )
 
 		throws FMFileManagerException
 	{
@@ -247,7 +250,7 @@ FMFileImpl
 			}
 
 			try{
-				file_access.setStorageType(  new_type );
+				file_access.setStorageType(  new_type, force );
 
 			}finally{
 
@@ -301,6 +304,11 @@ FMFileImpl
 		try{
 			this_mon.enter();
 
+				// when a file is being moved we want to prevent other operations from being blocked simply if they want
+				// to read the file length
+			
+			length_cache = getLength();
+			
 			TOTorrentFile tf = owner.getTorrentFile();
 
 			String	new_canonical_path;
@@ -381,6 +389,8 @@ FMFileImpl
 			}
 		}finally{
 
+			length_cache = -1;
+			
 			this_mon.exit();
 		}
 	}
@@ -395,9 +405,11 @@ FMFileImpl
 		try{
 			this_mon.enter();
 
+			length_cache = getLength();
+			
 			String	new_canonical_path;
 
-			File 	new_linked_file = new File( linked_file.getParentFile(), new_name );
+			File 	new_linked_file = FileUtil.newFile( linked_file.getParentFile(), new_name );
 
 			try{
 
@@ -471,6 +483,8 @@ FMFileImpl
 			}
 		}finally{
 
+			length_cache = -1;
+			
 			this_mon.exit();
 		}
 	}
@@ -498,6 +512,12 @@ FMFileImpl
 		}
 	}
 
+	protected long
+	getLengthCache()
+	{
+		return( length_cache );
+	}
+	
 	protected long
 	getLengthSupport()
 
@@ -527,7 +547,7 @@ FMFileImpl
 		long		length )
 
 		throws FMFileManagerException
-	{
+	{		
 		try{
 			file_access.setLength( fa, length );
 
@@ -575,9 +595,9 @@ FMFileImpl
 
 		file_access.aboutToOpen();
 
-		fa = new FMFileAccessController.FileAccessorRAF( linked_file, access_mode==FM_READ?READ_ACCESS_MODE:WRITE_ACCESS_MODE);
+		fa = FileUtil.newFileAccessor( linked_file, access_mode==FM_READ?READ_ACCESS_MODE:WRITE_ACCESS_MODE);
 
-		last_modified = linked_file.lastModified();
+		last_modified = 0;
 		
 		Debug.outNoStack( "Recovered connection to " + getName() + " after access failure" );
 	}
@@ -598,9 +618,9 @@ FMFileImpl
 		try{
 			file_access.aboutToOpen();
 
-			fa = new FMFileAccessController.FileAccessorRAF( linked_file, access_mode==FM_READ?READ_ACCESS_MODE:WRITE_ACCESS_MODE);
+			fa = FileUtil.newFileAccessor( linked_file, access_mode==FM_READ?READ_ACCESS_MODE:WRITE_ACCESS_MODE);
 
-			last_modified = linked_file.lastModified();
+			last_modified = 0;
 			
 		}catch( FileNotFoundException e ){
 
@@ -616,9 +636,9 @@ FMFileImpl
 
 				linked_file.createNewFile();
 
-				fa = new FMFileAccessController.FileAccessorRAF( linked_file, access_mode==FM_READ?READ_ACCESS_MODE:WRITE_ACCESS_MODE);
+				fa = FileUtil.newFileAccessor( linked_file, access_mode==FM_READ?READ_ACCESS_MODE:WRITE_ACCESS_MODE);
 
-				last_modified = linked_file.lastModified();
+				last_modified = 0;
 				
 				ok = true;
 
@@ -825,6 +845,9 @@ FMFileImpl
 	public long
 	getLastModified()
 	{
+		if (last_modified == 0) {
+			last_modified = linked_file.lastModified();
+		}
 		return( last_modified );
 	}
 	
@@ -946,9 +969,24 @@ FMFileImpl
 			int	read_access 		= 0;
 			int write_access		= 0;
 			int	write_access_lax	= 0;
+			int	write_access_hybrid	= 0;
 
 			TOTorrentFile	my_torrent_file = owner.getTorrentFile();
+			TOTorrent		my_torrent		= my_torrent_file==null?null:my_torrent_file.getTorrent();
+			
+			byte[] my_v2_hash = null;
+			
+			if ( my_torrent != null && my_torrent.getTorrentType() == TOTorrent.TT_V1_V2 ){
+				
+				try{
+					my_v2_hash = my_torrent.getFullHash( TOTorrent.TT_V2 );
+					
+				}catch( Throwable e ){
 
+					Debug.out( e );
+				}
+			}
+			
 			StringBuilder	users_sb = owners.size()==1?null:new StringBuilder( 128 );
 
 			for (Iterator it=owners.iterator();it.hasNext();){
@@ -963,9 +1001,30 @@ FMFileImpl
 
 					TOTorrentFile this_tf = this_owner.getTorrentFile();
 
-					if ( my_torrent_file != null && this_tf != null && my_torrent_file.getLength() == this_tf.getLength()){
+					if ( my_torrent_file != null && this_tf != null ){
+						
+						if ( my_torrent_file.getLength() == this_tf.getLength()){
 
-						write_access_lax++;
+							write_access_lax++;
+						}
+						
+						if ( my_v2_hash != null ){
+							
+							TOTorrent	this_torrent = this_tf.getTorrent();
+							
+							if ( this_torrent != null && this_torrent.getTorrentType() == TOTorrent.TT_V1_V2 ){
+								
+								try{
+									if ( Arrays.equals( my_v2_hash, this_torrent.getFullHash( TOTorrent.TT_V2 ))){
+									
+										write_access_hybrid++;
+									}
+								}catch( Throwable e ){
+									
+									Debug.out( e );
+								}
+							}
+						}
 					}
 
 					if ( users_sb != null ){
@@ -993,8 +1052,14 @@ FMFileImpl
 			if ( 	write_access > 1 ||
 					( write_access == 1 && read_access > 0 )){
 
-					// relax locking if strict is disabled and torrent file is same size
+					// relax locking for shared hybrid swarms				
 
+				if ( write_access_hybrid == write_access ){
+
+					return;
+				}
+				
+					// relax locking if strict is disabled and torrent file is same size
 
 				if ( !COConfigurationManager.getBooleanParameter( "File.strict.locking" )){
 
@@ -1169,7 +1234,7 @@ FMFileImpl
 	protected String
 	getString()
 	{
-		File cPath = new File(canonical_path);
+		File cPath = FileUtil.newFile(canonical_path);
 		String sPaths;
 		if (cPath.equals(linked_file))
 			sPaths = "can/link=" + Debug.secretFileName(canonical_path);
