@@ -37,6 +37,7 @@ import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.tracker.TrackerPeerSource;
 import com.biglybt.core.tracker.client.TRTrackerAnnouncer;
 import com.biglybt.core.tracker.client.TRTrackerScraperResponse;
+import com.biglybt.core.util.FileUtil;
 import com.biglybt.core.util.IndentWriter;
 import com.biglybt.pif.download.Download;
 import com.biglybt.pif.download.DownloadAnnounceResult;
@@ -317,19 +318,21 @@ DownloadManager
     public File
     getSaveLocation();
 
-        /**
-         * changes the save directory. Only call this if you know what you are doing!!!!
-         * @param sPath
-         */
+		/**
+		 * @deprecated Use setTorrentSaveDir(FileUtil.newFile(sPath), false)
+		 * @implNote Remove once xmwebui stops using this method
+		 */
+    default void setTorrentSaveDir(String sPath) {
+			setTorrentSaveDir(FileUtil.newFile(sPath), false);
+		}
 
-    public void
-    setTorrentSaveDir(
-        String sPath );
-
-    /**
-     * changes the save directory. Only call this if you know what you are doing!!!!
-     */
-    public void setTorrentSaveDir(String parent_dir, String dl_name);
+	/**
+	 * changes the save directory. Only call this if you know what you are doing!!!!
+	 * <p/>
+	 * When locationIncludesName, torrent name will also change (Data files will be stored under new_location)<br/>
+	 * When !locationIncludesName, only save dire is changed, name remains (Data files will be stored under new_location + / + old_location.getName()) 
+	 */
+	void setTorrentSaveDir(File _new_location, boolean locationIncludesName);
 
     public boolean isForceStart();
 
@@ -546,10 +549,10 @@ DownloadManager
 
     	/**
     	 * Progress of a move operation
-    	 * @return -1: not moving otherwise 0->1000
+    	 * @return null: not moving otherwise [ 0->1000, size ]
     	 */
     
-    public int
+    public long[]
     getMoveProgress();
     
         /**
@@ -596,14 +599,23 @@ DownloadManager
      */
     public void renameDownload(String new_name) throws DownloadManagerException;
 
-    /**
-     * Move the files and rename a download in one go.
-     *
-     * For convenience - either argument can be null, but not both.
-     *
-     * @see #moveDataFiles(File)
-     * @see #renameDownload(String)
-     */
+	/**
+	 * Move the files and rename a download in one go.
+	 *
+	 * For convenience - either argument can be null, but not both.
+	 *
+	 *
+	 * @param new_parent_dir new location to move torrent data files to.  
+	 * If null, and simple torrent, torrent data file will be renamed new_name.
+	 * If null, and not simple torrent, torrent end path will be changed to new_name.
+	 *
+	 * @param new_name For simple torrent, changes the filename of the downloaded file.
+	 * For non-simple torrent, changes the end path  for the downloaded files.
+	 * If null, torrent data files will be moved to new_parent_dir, with a subfolder of the existing name.
+	 * 
+	 * @see #moveDataFiles(File)
+	 * @see #renameDownload(String)
+	 */
     public void moveDataFiles(File new_parent_dir, String new_name) throws DownloadManagerException;
 
         /**
