@@ -1347,6 +1347,35 @@ DownloadManagerController
 		fileFacadeSet.destroyFileInfo();
 	}
 
+	@Override
+	public void 
+	saveTorrentState()
+	{
+		DownloadManagerState dms = download_manager.getDownloadState();
+	
+		TOTorrent dms_torrent = dms.getTorrent();
+		
+		if ( dms_torrent.getEffectiveTorrentType() == TOTorrent.TT_V2 ){
+		
+			if ( !dms.getBooleanAttribute( DownloadManagerState.AT_TORRENT_EXPORT_PROPAGATED )){
+
+					// this attribute is a hack to get the torrent to be serialised via the 'save'
+				
+				dms.setLongAttribute( DownloadManagerState.AT_TORRENT_SAVE_TIME, SystemTime.getCurrentTime());	
+			
+				dms.save( false );
+			
+				if ( dms_torrent.isExportable()){
+				
+					if ( TorrentUtils.propagateExportability( dms_torrent, FileUtil.newFile( download_manager.getTorrentFileName()))){
+						
+						dms.setBooleanAttribute( DownloadManagerState.AT_TORRENT_EXPORT_PROPAGATED, true );
+					}
+				}
+			}
+		}
+	}
+	
 	@Override
 	public boolean
 	isPeerSourceEnabled(
@@ -1547,7 +1576,23 @@ DownloadManagerController
 
 		if ( secrets_map.size() == 0 ){
 
-			secrets_map.put( "p1", torrent.getPieces()[0] );
+			if ( torrent.getEffectiveTorrentType() == TOTorrent.TT_V2 ){
+				
+				TOTorrentFile[] files = torrent.getFiles();
+				
+				for ( TOTorrentFile file: files ){
+					
+					if ( file.getLength() > 0 ){
+				
+						secrets_map.put( "p1", file.getHashTree().getRootHash());
+						
+						break;
+					}
+				}
+			}else{
+			
+				secrets_map.put( "p1", torrent.getPieces()[0] );
+			}
 
 			download_manager.getDownloadState().setMapAttribute( DownloadManagerState.AT_SECRETS, secrets_map );
 		}
@@ -1980,13 +2025,22 @@ DownloadManagerController
 		DiskManagerFileInfo[] files = fileFacadeSet.getFiles();
 
 		for (int i = 0; i < files.length; i++) {
+			
 			DiskManagerFileInfo fileInfo = files[i];
-			if (!fileInfo.isSkipped()) {
-				File file = fileInfo.getFile(true);
+			
+			if ( fileInfo.getTorrentFile().isPadFile()){
+				
+				continue;
+			}
+			
+			if (!fileInfo.isSkipped()){
+				
 				try {
 					long start = SystemTime.getMonotonousTime();
 
-					boolean 	exists = file.exists();
+					boolean 	exists = fileInfo.exists();
+
+					File file = fileInfo.getFile(true);
 
 					long elapsed = SystemTime.getMonotonousTime() - start;
 
@@ -2864,8 +2918,8 @@ DownloadManagerController
 		}
 
 		@Override
-		public boolean[] setStorageTypes(boolean[] toChange, int newStroageType) {
-			return delegate.setStorageTypes(toChange, newStroageType);
+		public boolean[] setStorageTypes(boolean[] toChange, int newStorageType, boolean force ) {
+			return delegate.setStorageTypes(toChange, newStorageType, force );
 		}
 
 		/** XXX Don't call me, call makeSureFilesFacadeFilled() */
@@ -3090,9 +3144,10 @@ DownloadManagerController
 		@Override
 		public boolean
 		setStorageType(
-			int		type )
+			int		type,
+			boolean	force )
 		{
-			return( delegate.setStorageType( type ));
+			return( delegate.setStorageType( type, force ));
 		}
 
 		@Override
@@ -3172,6 +3227,11 @@ DownloadManagerController
 			return( delegate.isSkipped());
 		}
 
+		@Override
+		public boolean exists(){
+			return( delegate.exists());
+		}
+
 		@Override
 		public int
 		getIndex()
