@@ -30,6 +30,7 @@ import com.biglybt.core.CoreFactory;
 import com.biglybt.core.category.Category;
 import com.biglybt.core.category.CategoryManager;
 import com.biglybt.core.config.COConfigurationManager;
+import com.biglybt.core.config.ConfigKeys;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.DiskManagerFactory;
 import com.biglybt.core.disk.DiskManagerFileInfo;
@@ -44,6 +45,7 @@ import com.biglybt.core.peer.PEPeerSource;
 import com.biglybt.core.torrent.*;
 import com.biglybt.core.tracker.client.TRTrackerAnnouncer;
 import com.biglybt.core.util.*;
+import com.biglybt.pif.logging.LoggerChannel;
 
 /**
  * @author parg
@@ -80,6 +82,29 @@ DownloadManagerStateImpl
 			FileUtil.mkdirs(ACTIVE_DIR);
 		}
 	}
+	
+	private static boolean disable_interim_saves;
+	// private static LoggerChannel 	save_log;
+	
+	
+	static{
+		COConfigurationManager.addAndFireParameterListener(
+			ConfigKeys.File.BCFG_DISABLE_SAVE_INTERIM_DOWNLOAD_STATE,
+			(n)->{
+				disable_interim_saves = COConfigurationManager.getBooleanParameter( n );
+				
+				/*
+				if ( disable_interim_saves && Constants.isCVSVersion() && save_log == null ){
+					
+					save_log = CoreFactory.getSingleton().getPluginManager().getDefaultPluginInterface().getLogger().getChannel( "DownloadStateSaves" );
+
+					save_log.setDiagnostic();
+
+					save_log.setForce( true );
+				}
+				*/
+			});
+	}
 
 	private static final Random	random = RandomUtils.SECURE_RANDOM;
 
@@ -250,7 +275,7 @@ DownloadManagerStateImpl
 	private final TorrentUtils.ExtendedTorrent	torrent;
 
 	private boolean						write_required_soon;
-	private boolean						write_required_sometime;
+	private long						write_required_sometime = -1;
 
 	private Category 	category;
 
@@ -431,7 +456,7 @@ DownloadManagerStateImpl
 
 		if ( saved_state == null ){
 
-			original_torrent = TorrentUtils.readDelegateFromFile( new File(torrent_file), discard_pieces );
+			original_torrent = TorrentUtils.readDelegateFromFile( FileUtil.newFile(torrent_file), discard_pieces );
 
 			torrent_hash = original_torrent.getHash();
 
@@ -508,15 +533,15 @@ DownloadManagerStateImpl
 				
 				if ( file_str != null ){
 					
-					File file = new File( file_str );
+					File file = FileUtil.newFile( file_str );
 					
-					File torrentDir = new File(COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory"));
+					File torrentDir = FileUtil.newFile(COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory"));
 	
 					if ( torrentDir.isDirectory() && torrentDir.equals( file.getParentFile())){
 						
 						file.delete();
-						
-						new File( file.getAbsolutePath() + ".bak" ).delete();
+
+						FileUtil.newFile( file.getAbsolutePath() + ".bak" ).delete();
 					}
 				}
 			}catch( Throwable e ){
@@ -530,13 +555,13 @@ DownloadManagerStateImpl
 	getStateFile(
 		byte[]		torrent_hash )
 	{
-		return( new File( ACTIVE_DIR, ByteFormatter.encodeString( torrent_hash ) + ".dat" ));
+		return( FileUtil.newFile( ACTIVE_DIR, ByteFormatter.encodeString( torrent_hash ) + ".dat" ));
 	}
 
 	protected static File
 	getGlobalStateFile()
 	{
-		return( new File( ACTIVE_DIR, "cache.dat" ));
+		return( FileUtil.newFile( ACTIVE_DIR, "cache.dat" ));
 	}
 
 	public static void
@@ -549,7 +574,7 @@ DownloadManagerStateImpl
 			return;
 		}
 
-		try( 	FileInputStream fis = new FileInputStream( file );
+		try( 	FileInputStream fis = FileUtil.newFileInputStream( file );
 				BufferedInputStream is = new BufferedInputStream( new GZIPInputStream( fis ))){
 
 			try{
@@ -617,7 +642,7 @@ DownloadManagerStateImpl
 				}
 			}
 
-			GZIPOutputStream	os = new GZIPOutputStream( new FileOutputStream( getGlobalStateFile()));
+			GZIPOutputStream	os = new GZIPOutputStream( FileUtil.newFileOutputStream( getGlobalStateFile()));
 
 			try{
 
@@ -672,8 +697,8 @@ DownloadManagerStateImpl
 
 		String	state_file = hash_str + ".dat";
 
-		File	target_state_file 	= new File( ACTIVE_DIR, state_file );
-		File	source_state_file 	= new File( source_dir, state_file );
+		File	target_state_file 	= FileUtil.newFile( ACTIVE_DIR, state_file );
+		File	source_state_file 	= FileUtil.newFile( source_dir, state_file );
 
 		if ( !source_state_file.exists()){
 
@@ -692,7 +717,7 @@ DownloadManagerStateImpl
 			throw( new DownloadManagerException( "Failed to copy state file: " + source_state_file + " -> " + target_state_file ));
 		}
 
-		File	source_state_dir = new File( source_dir, hash_str );
+		File	source_state_dir = FileUtil.newFile( source_dir, hash_str );
 
 		if ( source_state_dir.exists()){
 
@@ -728,7 +753,7 @@ DownloadManagerStateImpl
 
 		String	state_file = hash_str + ".dat";
 
-		File	target_state_file 	= new File( source_dir, state_file );
+		File	target_state_file 	= FileUtil.newFile( source_dir, state_file );
 
 		if ( target_state_file.exists()){
 
@@ -738,9 +763,9 @@ DownloadManagerStateImpl
 			}
 		}
 
-		new File( source_dir, state_file + ".bak" ).delete();
-		
-		File	target_state_dir = new File( source_dir, hash_str );
+		FileUtil.newFile( source_dir, state_file + ".bak" ).delete();
+
+		File	target_state_dir = FileUtil.newFile( source_dir, hash_str );
 
 		if ( target_state_dir.exists()){
 
@@ -856,7 +881,7 @@ DownloadManagerStateImpl
 	getStateFile( )
 	{
 		try{
-			File	parent = new File( ACTIVE_DIR, ByteFormatter.encodeString( torrent.getHash()) + File.separatorChar);
+			File	parent = FileUtil.newFile( ACTIVE_DIR, ByteFormatter.encodeString( torrent.getHash()));
 
 			return( StringInterner.internFile(parent));
 
@@ -970,10 +995,12 @@ DownloadManagerStateImpl
 			
 			Map existing = torrent.getAdditionalMapProperty(RESUME_KEY  );
 			
+			long new_resume_state;
+			
 			if ( data == null ){
 
-				setLongAttribute( AT_RESUME_STATE, 1 );
-				
+				new_resume_state = 1;
+								
 				if ( existing != null ){
 				
 					torrent.removeAdditionalProperty( RESUME_KEY );
@@ -988,7 +1015,14 @@ DownloadManagerStateImpl
 				
 				boolean complete = DiskManagerFactory.isTorrentResumeDataComplete( this );
 
-				setLongAttribute( AT_RESUME_STATE, complete?2:1 );
+				new_resume_state = complete?2:1;
+			}
+			
+			if ( getLongAttribute( AT_RESUME_STATE ) != new_resume_state ){
+				
+				setLongAttribute( AT_RESUME_STATE, new_resume_state );
+				
+				changed = true;
 			}
 
 			if ( changed ){
@@ -1003,7 +1037,15 @@ DownloadManagerStateImpl
 
 			// we need to ensure this is persisted now as it has implications regarding crash restarts etc
 
-		saveSupport(false,false);
+		if ( disable_interim_saves && !changed ){
+			
+				// to be honest if the data hasn't changed I'm not sure why we need a save at all here but for the moment
+				// I'll disable it if the user has already opted in to reducing writes
+			
+		}else{
+		
+			saveSupport(false,false);
+		}
 	}
 
 	@Override
@@ -1067,15 +1109,15 @@ DownloadManagerStateImpl
 
 			String	state_file = hash_str + ".dat";
 
-			File	existing_state_file = new File( ACTIVE_DIR, state_file );
-			File	target_state_file 	= new File( target_dir, state_file );
+			File	existing_state_file = FileUtil.newFile( ACTIVE_DIR, state_file );
+			File	target_state_file 	= FileUtil.newFile( target_dir, state_file );
 
 			if ( !FileUtil.copyFile( existing_state_file, target_state_file )){
 
 				throw( new IOException( "Failed to copy state file" ));
 			}
 
-			File	existing_state_dir = new File( ACTIVE_DIR, hash_str );
+			File	existing_state_dir = FileUtil.newFile( ACTIVE_DIR, hash_str );
 
 			if ( existing_state_dir.exists()){
 
@@ -1110,10 +1152,13 @@ DownloadManagerStateImpl
 		boolean		slightly )
 	{
 		//Debug.out( (slightly?"slightly":"dirty" )+ ": " + new String(torrent.getName()));
-		
+				
 		if ( slightly ){
 			
-			write_required_sometime = true;
+			if ( write_required_sometime == -1 ){
+			
+				write_required_sometime = SystemTime.getMonotonousTime();
+			}
 			
 		}else{
 		
@@ -1134,10 +1179,21 @@ DownloadManagerStateImpl
 		boolean interim, 
 		boolean force )
 	{
-		if ( supressWrites > 0 && !force ){
-
-			return;
+		if ( !force ){
+			
+			if ( supressWrites > 0 ){
+	
+				return;
+			}
+			
+			if ( interim && disable_interim_saves ){
+				
+				return;
+			}
 		}
+		
+		//boolean soon = write_required_soon;
+		//long some = write_required_sometime;
 
  		boolean do_write;
 
@@ -1148,10 +1204,18 @@ DownloadManagerStateImpl
 				
 				do_write = true;
 				
-			}else if ( write_required_sometime ){
-								
-				do_write = !interim;
+			}else if ( write_required_sometime != -1 ){
+					
+				if ( interim ){
 				
+						// avoid all the 'sometime' writes ending up being done on closedown
+					
+					do_write = SystemTime.getMonotonousTime() - write_required_sometime > 5*60*1000;
+				
+				}else{
+					
+					do_write = true;
+				}
 			}else{
 				
 				do_write = false;
@@ -1160,7 +1224,7 @@ DownloadManagerStateImpl
 			if ( do_write ){
 			
 				write_required_soon 	= false;
-				write_required_sometime	= false;
+				write_required_sometime	= -1;
 			}
 		}finally{
 
@@ -1171,12 +1235,19 @@ DownloadManagerStateImpl
 
 			try{
 				
-				// System.out.println( "writing download state for '" + new String(torrent.getName()));
+				//Debug.out( "writing download state for '" + new String(torrent.getName()) + ", interim=" + interim + ", soon/some=" + soon + "/" + some );
 
 				if (Logger.isEnabled())
 					Logger.log(new LogEvent(torrent, LOGID, "Saving state for download '"
 							+ TorrentUtils.getLocalisedName(torrent) + "'"));
 
+				/*
+				if ( save_log != null ){
+					
+					save_log.log( TorrentUtils.getLocalisedName(torrent) + ": " + Debug.getCompressedStackTrace());
+				}
+				*/
+				
 				torrent.setAdditionalMapProperty( ATTRIBUTE_KEY, attributes );
 
 				TorrentUtils.writeToFile(torrent, true);
@@ -1208,7 +1279,7 @@ DownloadManagerStateImpl
 	        
 			String	state_file = hash_str + ".dat";
 			
-			File	target_state_file 	= new File( ACTIVE_DIR, state_file );
+			File	target_state_file 	= FileUtil.newFile( ACTIVE_DIR, state_file );
 
 			if ( target_state_file.exists()){
 
@@ -1218,9 +1289,9 @@ DownloadManagerStateImpl
 				}
 			}
 
-			new File( ACTIVE_DIR, state_file + ".bak" ).delete();
+			FileUtil.newFile( ACTIVE_DIR, state_file + ".bak" ).delete();
 			
-			File	dir = new File( ACTIVE_DIR, hash_str );
+			File	dir = FileUtil.newFile( ACTIVE_DIR, hash_str );
 
 			if ( dir.exists() && dir.isDirectory()){
 
@@ -1549,7 +1620,7 @@ DownloadManagerStateImpl
 
 		if (name.equals(AT_RELATIVE_SAVE_PATH)) {
 			if (value.length() > 0) {
-				File relative_path_file = new File(value);
+				File relative_path_file = FileUtil.newFile(value);
 				relative_path_file = DownloadManagerDefaultPaths.normaliseRelativePath(relative_path_file);
 				value = (relative_path_file == null) ? "" : relative_path_file.getPath();
 			}
@@ -2237,8 +2308,8 @@ DownloadManagerStateImpl
 
 					try{
 						int		index 	= Integer.parseInt( bits[0].trim());
-						File	source	= new File(bits[1]);
-						File	target	= bits.length<3?null:new File(bits[2]);
+						File	source	= FileUtil.newFile(bits[1]);
+						File	target	= bits.length<3?null:FileUtil.newFile(bits[2]);
 
 						if( index >= 0 ){
 
@@ -2268,9 +2339,9 @@ DownloadManagerStateImpl
 
 				if ( sep != -1 ){
 
-					File target = (sep == entry.length()-1)?null:new File( entry.substring( sep+1 ));
+					File target = (sep == entry.length()-1)?null:FileUtil.newFile( entry.substring( sep+1 ));
 
-					res.putMigration( new File( entry.substring(0,sep)), target );
+					res.putMigration( FileUtil.newFile( entry.substring(0,sep)), target );
 				}
 			}
 		}
@@ -2486,8 +2557,23 @@ DownloadManagerStateImpl
 				attributes.put( attribute_name, new Long( attribute_value) );
 
 				changed = true;
-								
-				setDirty( attribute_name == DownloadManagerState.AT_SCRAPE_CACHE );
+					
+				boolean	set_dirty = true;
+				
+				boolean is_scrape_cache = attribute_name == DownloadManagerState.AT_SCRAPE_CACHE ;
+				
+				if ( is_scrape_cache && download_manager.getGlobalManager().isStopping()){
+					
+					 if ( download_manager.getState() == DownloadManager.STATE_STOPPED ){
+						 
+						 set_dirty = false;
+					 }
+				}
+				
+				if ( set_dirty ){
+				
+					setDirty( is_scrape_cache );
+				}
 			}
 		}finally{
 
@@ -3506,6 +3592,7 @@ DownloadManagerStateImpl
 		boolean		discard_pieces;
 		boolean		logged_failure;
 
+		Integer		torrent_type;
 		Boolean		simple_torrent;
 		long		size;
 		Boolean		is_private;
@@ -3549,6 +3636,13 @@ DownloadManagerStateImpl
 				}
 			}
 
+			Long	tt = (Long)cache.get( "tt" );
+
+			if ( tt != null ){
+
+				torrent_type = tt.intValue();
+			}
+			
 			Long	st = (Long)cache.get( "simple" );
 
 			if ( st != null ){
@@ -3668,6 +3762,18 @@ DownloadManagerStateImpl
 
 					cache.put( "fc", new Long( fc ));
 				}
+				
+				Integer	tt = csw.torrent_type;
+
+				if ( tt != null ){
+
+					cache.put( "tt", new Long( tt.intValue()));
+
+				}else{
+
+					Debug.out( "Failed to cache torrent type" );
+				}
+				
 			}else{
 				if ( t instanceof TorrentUtils.torrentDelegate ){
 
@@ -3677,6 +3783,8 @@ DownloadManagerStateImpl
 
 					cache.put( "fc", t.getFileCount());
 
+					cache.put( "tt", new Long(t.getTorrentType()));
+					
 				}else{
 
 					Debug.out( "Hmm, torrent isn't cache-state-wrapper, it is " + t );
@@ -4033,7 +4141,7 @@ DownloadManagerStateImpl
 
 				// try reading from original
 
-			TOTorrent original_torrent = TorrentUtils.readFromFile( new File(torrent_file), true );
+			TOTorrent original_torrent = TorrentUtils.readFromFile( FileUtil.newFile(torrent_file), true );
 
 			torrent_hash_wrapper = original_torrent.getHashWrapper();
 
@@ -4124,6 +4232,52 @@ DownloadManagerStateImpl
 			return null;
 		}
 
+	  	@Override
+	    public int
+    	getTorrentType()
+    	{
+    		if ( torrent_type != null ){
+
+    			return( torrent_type );
+    		}
+
+    		if ( fixup()){
+
+    			int tt = delegate.getTorrentType();
+
+    			torrent_type = tt;
+
+    			return( tt );
+    		}
+
+    		return( TOTorrent.TT_V1 );
+    	}
+	  	
+		@Override
+		public boolean 
+		isExportable()
+		{
+			if ( fixup()){
+
+				return( delegate.isExportable());
+			}
+			
+			return( false );
+		}
+	  	
+		@Override
+		public boolean 
+		updateExportability(
+			TOTorrent from )
+		{
+			if ( fixup()){
+
+				return( delegate.updateExportability( from ));
+			}
+			
+			return( false );
+		}
+		
     	@Override
 	    public boolean
     	isSimpleTorrent()
@@ -4445,6 +4599,23 @@ DownloadManagerStateImpl
 
     		return( torrent_hash_wrapper.getBytes());
     	}
+    	
+    	@Override
+    	public byte[]
+    	getFullHash(
+    		int		type ) 
+    	
+    		throws TOTorrentException
+    	{
+   			if ( fixup()){
+
+   				return( delegate.getFullHash( type ));
+   				
+   			}else{
+    				
+   				return( null );
+   			}
+    	}
 
     	@Override
 	    public HashWrapper
@@ -4454,7 +4625,24 @@ DownloadManagerStateImpl
 	   	{
     		return( torrent_hash_wrapper );
     	}
-
+    	
+    	@Override
+    	public TOTorrent 
+    	selectHybridHashType(
+    		int type ) 
+    		
+    		throws TOTorrentException
+    	{
+    		if ( fixup()){
+    			
+    			return( delegate.selectHybridHashType(type));
+    			
+    		}else{
+    			
+    			throw( new TOTorrentException( "fixup failed", TOTorrentException.RT_CREATE_FAILED ));
+    		}
+    	}
+    	
     	@Override
 	    public void
     	setHashOverride(
