@@ -20,6 +20,12 @@
 
 package com.biglybt.core.download.impl;
 
+import java.net.URL;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+
 /**
  * @author parg
  */
@@ -34,15 +40,24 @@ import com.biglybt.core.download.DownloadManagerStats;
 import com.biglybt.core.peer.PEPeerManager;
 import com.biglybt.core.peer.PEPeerManagerStats;
 import com.biglybt.core.torrent.TOTorrent;
+import com.biglybt.core.tracker.AllTrackersManager;
+import com.biglybt.core.tracker.AllTrackersManager.AllTrackers;
+import com.biglybt.core.tracker.client.TRTrackerAnnouncerRequest;
+import com.biglybt.core.util.AERunnable;
+import com.biglybt.core.util.Base32;
 import com.biglybt.core.util.Constants;
 import com.biglybt.core.util.Debug;
+import com.biglybt.core.util.FrequencyLimitedDispatcher;
 import com.biglybt.core.util.IndentWriter;
+import com.biglybt.core.util.SimpleTimer;
 import com.biglybt.core.util.SystemTime;
 
 public class
 DownloadManagerStatsImpl
 	implements DownloadManagerStats
 {
+	private static final AllTrackers	all_trackers = AllTrackersManager.getAllTrackers();
+
 	static int share_ratio_progress_interval;
 
 	static{
@@ -101,6 +116,7 @@ DownloadManagerStatsImpl
 
 	private static final int HISTORY_MAX_SECS = 30*60;
 	private volatile boolean history_retention_required;
+	private int			history_last_tick = -1;
 	private long[][]	history;
 	private int			history_pos;
 	private boolean		history_wrapped;
@@ -597,7 +613,7 @@ DownloadManagerStatsImpl
 	}
 
 	private static final int HISTORY_RATE_DIV = 64;
-	private static final int HISTORY_TIME_DIV = 60;
+	private static final int HISTORY_TIME_DIV = 1;		//	used to be 60 but this messes up small ETAs (obviously) - rethink if need to re-introduce 
 
 	@Override
 	public int[][]
@@ -611,6 +627,13 @@ DownloadManagerStatsImpl
 
 			}else{
 
+				int ticks = SimpleTimer.getTickCount();
+				
+				if ( ticks != history_last_tick ){
+					
+					timerTick( ticks );
+				}
+				
 				int	entries = history_wrapped?HISTORY_MAX_SECS:history_pos;
 				int	start	= history_wrapped?history_pos:0;
 
@@ -632,7 +655,7 @@ DownloadManagerStatsImpl
 					int	recv_rate 	= (int)((entry1>>21)&0x001fffffL);
 					int	swarm_rate 	= (int)((entry1)&0x001fffffL);
 
-					int	eta 	= (int)((entry2)&0x001fffffL);
+					int	eta 	= (int)((entry2)&0x000000007fffffffL);
 
 					result[0][i] = send_rate*HISTORY_RATE_DIV;
 					result[1][i] = recv_rate*HISTORY_RATE_DIV;
@@ -701,31 +724,72 @@ DownloadManagerStatsImpl
 
 		PEPeerManager pm = download_manager.getPeerManager();
 
-		if ( pm == null ){
-
-			return;
-		}
-
-		PEPeerManagerStats stats = pm.getStats();
-
-		long send_rate 			= stats.getDataSendRate() + stats.getProtocolSendRate();
-		long receive_rate 		= stats.getDataReceiveRate() + stats.getProtocolReceiveRate();
-		long peer_swarm_average = getTotalAveragePerPeer();
-
-		long eta	= Math.max( getETA(), 0);
+		long	entry1;
+		long	entry2;
 		
-		long	entry1 =
-			((((send_rate-1+HISTORY_RATE_DIV/2)/HISTORY_RATE_DIV)<<42) 	&  0x7ffffc0000000000L ) |
-			((((receive_rate-1+HISTORY_RATE_DIV/2)/HISTORY_RATE_DIV)<<21)  & 0x000003ffffe00000L ) |
-			((((peer_swarm_average-1+HISTORY_RATE_DIV/2)/HISTORY_RATE_DIV))& 0x00000000001fffffL );
-
-		long	entry2 =
-			((((eta-1+HISTORY_TIME_DIV/2)/HISTORY_TIME_DIV))& 0x00000000001fffffL );
+		if ( pm != null ){
 
+			PEPeerManagerStats stats = pm.getStats();
+	
+			long send_rate 			= stats.getDataSendRate() + stats.getProtocolSendRate();
+			long receive_rate 		= stats.getDataReceiveRate() + stats.getProtocolReceiveRate();
+			long peer_swarm_average = getTotalAveragePerPeer();
+	
+			long eta	= Math.max( getETA(), 0);
+			
+			entry1 =
+				((((send_rate-1+HISTORY_RATE_DIV/2)/HISTORY_RATE_DIV)<<42) 	&  0x7ffffc0000000000L ) |
+				((((receive_rate-1+HISTORY_RATE_DIV/2)/HISTORY_RATE_DIV)<<21)  & 0x000003ffffe00000L ) |
+				((((peer_swarm_average-1+HISTORY_RATE_DIV/2)/HISTORY_RATE_DIV))& 0x00000000001fffffL );
+	
+			//long	entry2 =
+			//	((((eta-1+HISTORY_TIME_DIV/2)/HISTORY_TIME_DIV))& 0x00000000001fffffL );
+	
+			entry2 = eta&0x000000007fffffffL;
+			
+		}else{
+			
+			entry1	= 0;
+			entry2	= 0;
+		}
+		
 		synchronized( this ){
 
 			if ( history != null ){
 
+				if ( history_last_tick == tick_count ){
+				
+						// already processed
+					
+					return;
+				}
+				
+				if ( history_last_tick != -1 ){
+				
+					int missed = tick_count - history_last_tick - 1;
+					
+					if ( missed > 0 ){
+						
+						missed = Math.min( missed, HISTORY_MAX_SECS );
+						
+						while( missed > 0 ){
+							
+							history[history_pos][0] 	= 0;
+							history[history_pos++][1] 	= 0;
+
+							if ( history_pos == HISTORY_MAX_SECS ){
+
+								history_pos 	= 0;
+								history_wrapped	= true;
+							}
+							
+							missed--;
+						}
+					}
+				}
+				
+				history_last_tick = tick_count;
+				
 				history[history_pos][0] 	= entry1;
 				history[history_pos++][1] 	= entry2;
 
@@ -858,7 +922,16 @@ DownloadManagerStatsImpl
 			return( -1 );
 		}
 
-		return (int) ((1000 * uploaded) / downloaded);
+		long ratio = (1000 * uploaded) / downloaded;
+		
+		if ( ratio > Integer.MAX_VALUE ){
+			
+			return( Integer.MAX_VALUE );
+			
+		}else{
+			
+			return((int)ratio);
+		}
 	}
 
 	@Override
@@ -1122,7 +1195,277 @@ DownloadManagerStatsImpl
 
 		return( state.getLongAttribute( DownloadManagerState.AT_AVAIL_BAD_TIME ));
 	}
+	
+	Map<String,Map<Long,long[]>>	tracker_session_stats = new HashMap<>();
 
+	private void
+	loadTrackerStats()
+	{
+		DownloadManagerState state = download_manager.getDownloadState();
+
+		synchronized( tracker_session_stats ){
+
+			if ( !tracker_session_stats.isEmpty()){
+				
+				return;		// already loaded so holds most up-to-date values
+			}
+			
+			Map<String,Map<String,List<Number>>> stats = state.getMapAttribute( DownloadManagerState.AT_TRACKER_SESSION_STATS );
+			
+			if ( stats == null ){
+				
+				return;
+			}
+						
+			for ( Map.Entry<String,Map<String,List<Number>>> entry1: stats.entrySet()){
+				
+				String key = entry1.getKey();
+				
+				key = new String( Base32.decode( key ), Constants.UTF_8 );
+				
+				Map<Long, long[]> session_stats = new HashMap<>();
+				
+				tracker_session_stats.put( key, session_stats );
+				
+				for ( Map.Entry<String,List<Number>> entry: entry1.getValue().entrySet()){
+					
+					try{
+						long id = Long.parseLong( entry.getKey());
+						
+						List<Number> nums = entry.getValue();
+						
+						long[] vals = new long[nums.size()];
+						
+						for ( int i=0; i<vals.length; i++){
+							
+							vals[i] = nums.get(i).longValue();
+						}
+						
+						session_stats.put( id,  vals );
+						
+					}catch( Throwable e ){
+						
+						Debug.out( e );
+					}
+				}
+			}
+		}
+	}
+	
+	private void
+	saveTrackerStats()
+	{
+		DownloadManagerState state = download_manager.getDownloadState();
+
+		synchronized( tracker_session_stats ){
+
+			Map<String,Map<String,List<Long>>> stats = new HashMap<>();
+			
+			for (Map.Entry<String,Map<Long,long[]>> entry1: tracker_session_stats.entrySet()){
+				
+				String key = entry1.getKey();
+				
+				key = Base32.encode( key.getBytes( Constants.UTF_8 ));	// protect against non-ascii keys
+				
+				Map<String,List<Long>> ss = new HashMap<>();
+				
+				stats.put( key, ss );
+				
+				Map<Long,long[]> session_stats = entry1.getValue();
+				
+					// only ever persist consolidated stats as session-ids are relative to a biglybt run only
+					// but don't touch the existing state as it is reused
+				
+				long[]	consolidated = null;
+				
+				for ( long[] vals: session_stats.values()){
+					
+					if ( consolidated == null ){
+						
+						consolidated = vals;
+						
+					}else{
+						
+						for ( int i=1;i<Math.min( vals.length, consolidated.length ); i++){
+							
+							consolidated[i] = consolidated[i] + vals[i];
+						}
+					}
+				}
+				
+				if ( consolidated != null ){
+					
+					consolidated[0] = SystemTime.getCurrentTime();
+					
+					List<Long> vals = new ArrayList<>();
+					
+					ss.put( "0", vals );
+					
+					for ( long l: consolidated ){
+						
+						vals.add( l );
+					}
+				}
+			}
+			
+			state.setMapAttribute( DownloadManagerState.AT_TRACKER_SESSION_STATS, stats );
+		}		
+	}
+	
+	FrequencyLimitedDispatcher stats_saver = 
+		new FrequencyLimitedDispatcher(
+			AERunnable.create(()->{
+				saveTrackerStats();
+			}),1000 );
+	
+	protected void
+	updateTrackerSession(
+		TRTrackerAnnouncerRequest	request )
+	{
+		long session	= request.getSessionID();
+		
+		URL url = request.getURL();
+
+		if ( session != 0 && url != null ){
+																	
+			String key = all_trackers.ingestURL( url );
+	
+			if ( key == null ){
+				
+				return;
+			}
+	
+			all_trackers.updateTracker( key, request );
+				
+			long down 		= request.getReportedDownload();
+			long up 		= request.getReportedUpload();
+
+			if ( down > 0 || up > 0 ){
+					
+				int group = download_manager.getTrackerURLGroup( key );
+				
+				synchronized( tracker_session_stats ){
+					
+					while( true ){
+						
+						Map<Long,long[]> session_stats = tracker_session_stats.get( key );
+						
+						if ( session_stats == null ){
+							
+							session_stats = new HashMap<>();
+							
+							tracker_session_stats.put( key, session_stats );
+						}
+						
+						session_stats.put( session, new long[]{ SystemTime.getCurrentTime(), up, down });
+						
+						while( session_stats.size() > 5 ){
+							
+							long oldest_time 	= Long.MAX_VALUE;
+							long oldest_session	= 0;
+							
+							long[]	consolidated = session_stats.remove( 0L );
+							
+							for ( Map.Entry<Long, long[]> entry: session_stats.entrySet()){
+								
+								long[] 	vals = entry.getValue();
+									
+								long time = vals[0];
+									
+								if ( time < oldest_time ){
+										
+									long 	sid = entry.getKey();							
+			
+									oldest_time 	= time;
+									oldest_session	= sid;
+								}
+							}
+							
+							long[] oldest = session_stats.remove( oldest_session );
+							
+							if ( consolidated == null ){
+								
+								consolidated = oldest;
+								
+							}else{
+								
+								for ( int i=1;i<Math.min( oldest.length, consolidated.length ); i++){
+									
+									consolidated[i] = consolidated[i] + oldest[i];
+								}
+							}
+							
+							consolidated[0] = SystemTime.getCurrentTime();
+							
+							session_stats.put( 0L, consolidated );
+						}
+						
+						if ( group == -1 ){
+							
+							break;
+							
+						}else{
+							
+							key = String.valueOf( group );
+							
+							group = -1;
+						}
+					}
+				}
+				
+				if ( download_manager.getPeerManager() == null ){
+		
+					stats_saver.dispatch();	// can get a bunch of these when stopping a download
+				}
+			}
+		}
+	}
+
+	protected long[] 
+	getTrackerReportedStats(
+		URL 		url )
+	{
+		String key = all_trackers.ingestURL( url );
+		
+		int group = download_manager.getTrackerURLGroup( key );
+		
+		long group_up	= 0;
+		long group_down	= 0;
+		
+		long url_up 	= 0;
+		long url_down	= 0;
+
+		synchronized( tracker_session_stats ){
+			
+			Map<Long,long[]> stats = tracker_session_stats.get( key );
+			
+			if ( stats != null ){
+								
+				for (long[] entry: stats.values()){
+					
+					url_up 		+= entry[1];
+					url_down 	+= entry[2];
+				}
+			}
+			
+			if ( group != -1 ){
+				
+				stats = tracker_session_stats.get( String.valueOf( group ));
+				
+				if ( stats != null ){
+									
+					for (long[] entry: stats.values()){
+						
+						group_up 		+= entry[1];
+						group_down 	+= entry[2];
+					}
+				}
+			}
+		}
+		
+		return( new long[]{ group_up, group_down, url_up, url_down });
+	}
+	
 	protected void
 	saveSessionTotals()
 	{
@@ -1160,6 +1503,8 @@ DownloadManagerStatsImpl
 
 		state.setLongAttribute( DownloadManagerState.AT_PEAK_RECEIVE_RATE, saved_peak_receive_rate );
 		state.setLongAttribute( DownloadManagerState.AT_PEAK_SEND_RATE, saved_peak_send_rate );
+		
+		saveTrackerStats();
 	}
 
  	protected void
@@ -1204,6 +1549,8 @@ DownloadManagerStatsImpl
 			// when there's actually completed data.  Force a recalc on next request
 			saved_completed_download_bytes = -1;
 		}
+		
+		loadTrackerStats();
 	}
 
 	public void
