@@ -224,8 +224,9 @@ CoreImpl
 	
 	private boolean					ll_started;
 	
-	private final List				operation_listeners		= new ArrayList();
-
+	private final List<CoreOperationListener>				operation_listeners		= new ArrayList<>();
+	private final CopyOnWriteList<CoreOperation>			operations				= new CopyOnWriteList<>();
+	
 	private final CopyOnWriteList<PowerManagementListener>	power_listeners = new CopyOnWriteList<>();
 
 	final AESemaphore			stopping_sem	= new AESemaphore( "Core::stopping" );
@@ -276,8 +277,8 @@ CoreImpl
 	
 				// set up a backwards pointer from config -> app dir so we can derive one from the other. It'll get saved on closedown, no need to do so now
 	
-			COConfigurationManager.setParameter( "azureus.application.directory", new File( SystemProperties.getApplicationPath()).getAbsolutePath());
-			COConfigurationManager.setParameter( "azureus.user.directory", new File( SystemProperties.getUserPath()).getAbsolutePath());
+			COConfigurationManager.setParameter( "azureus.application.directory", FileUtil.newFile( SystemProperties.getApplicationPath()).getAbsolutePath());
+			COConfigurationManager.setParameter( "azureus.user.directory", FileUtil.newFile( SystemProperties.getUserPath()).getAbsolutePath());
 				
 			crypto_manager = CryptoManagerFactory.getSingleton();
 	
@@ -886,7 +887,7 @@ CoreImpl
 	public File
 	getLockFile()
 	{
-		return( new File(SystemProperties.getUserPath(), ".azlock" ));
+		return( FileUtil.newFile(SystemProperties.getUserPath(), ".azlock" ));
 	}
 
 	private FileLock file_lock;
@@ -1085,7 +1086,7 @@ CoreImpl
 
 											if ( tf != null ){
 
-												File file = new File( tf );
+												File file = FileUtil.newFile( tf );
 
 												if ( !file.canRead() || file.isDirectory()){
 
@@ -1106,7 +1107,7 @@ CoreImpl
 
 										if ( save_folder != null ){
 
-											dest = new File( save_folder, torrent.getName());
+											dest = FileUtil.newFile( save_folder, torrent.getName());
 
 										}else{
 
@@ -1114,7 +1115,7 @@ CoreImpl
 
 											if ( save_file != null ){
 
-												dest = new File( save_file );
+												dest = FileUtil.newFile( save_file );
 											}
 										}
 
@@ -1647,7 +1648,10 @@ CoreImpl
 						// it has been a minute - turn off logging if it is enabled as this can significantly
 						// slow things down
 					
-					Logger.setClosingTakingTooLong();
+					if ( System.getProperty( SystemProperties.SYSPROP_LOGGING_DISABLE_STOP_ON_SLOW_CLOSE, "0" ).equals( "0" )){
+					
+						Logger.setClosingTakingTooLong();
+					}
 					
 					last_progress.set( SystemTime.getMonotonousTime());
 					
@@ -2088,14 +2092,14 @@ CoreImpl
 										
 										break;
 									}
+								
+									Thread.sleep(1*1000);
+								
+									loops++;
 								}
-
-								Thread.sleep(1*1000);
-								
-								loops++;
-								
 							}catch( Throwable e ){
 
+								Debug.out( e );
 							}
 						}
 					}.start();
@@ -3118,7 +3122,7 @@ CoreImpl
 
 		if ( script_type == "" ){
 
-			script_file = new File( script.trim());
+			script_file = FileUtil.newFile( script.trim());
 
 			if ( !script_file.isFile()){
 
@@ -3195,12 +3199,10 @@ CoreImpl
 
 	@Override
 	public void
-	createOperation(
-		final int					type,
-		CoreOperationTask task )
+	executeOperation(
+		int						type,
+		CoreOperationTask 		task )
 	{
-		final CoreOperationTask[] f_task = { task };
-
 		CoreOperation op =
 				new CoreOperation()
 				{
@@ -3215,30 +3217,90 @@ CoreImpl
 					public CoreOperationTask
 					getTask()
 					{
-						return( f_task[0] );
+						return( task );
 					}
 				};
 
-
-		for (int i=0;i<operation_listeners.size();i++){
-
-				// don't catch exceptions here as we want errors from task execution to propagate
-				// back to the invoker
-
-			if (((CoreOperationListener)operation_listeners.get(i)).operationCreated( op )){
-
-				f_task[0] = null;
+		boolean	run_it = true;
+				
+		try{
+			addOperation( op );
+			
+			for ( CoreOperationListener l: operation_listeners ){
+	
+					// don't catch exceptions here as we want errors from task execution to propagate
+					// back to the invoker
+	
+				if ( l.operationExecuteRequest( op )){
+	
+					run_it = false;
+					
+					break;
+				}
+			}
+	
+				// nobody volunteeered to run it for us, we'd better do it
+	
+			if ( run_it ){
+	
+				task.run( op );
+			}
+		}finally{
+			
+			if ( run_it ){
+				
+				removeOperation( op );
 			}
-		}
-
-			// nobody volunteeered to run it for us, we'd better do it
-
-		if ( f_task[0] != null ){
-
-			task.run( op );
 		}
 	}
 
+	@Override
+	public void
+	addOperation(
+		CoreOperation		op )
+	{
+		operations.add( op );
+		
+		for ( CoreOperationListener l: operation_listeners ){
+			
+			try{
+				l.operationAdded( op );
+				
+			}catch( Throwable e ){
+				
+				Debug.out( e );
+			}
+		}
+	}
+	
+	@Override
+	public void
+	removeOperation(
+		CoreOperation		op )
+	{
+		if ( operations.remove( op )){
+		
+			for ( CoreOperationListener l: operation_listeners ){
+				
+				try{
+					l.operationRemoved( op );
+					
+				}catch( Throwable e ){
+					
+					Debug.out( e );
+				}
+			}
+		}
+	}
+	
+	@Override
+	public List<CoreOperation>
+	getOperations()
+	{
+		return( operations.getList());
+	}
+	
+	
 	@Override
 	public void
 	addLifecycleListener(
