@@ -303,7 +303,7 @@ public class MessageText {
 	String key,
 	String sDefault)
   {
-	  if (key == null) {
+	  if (key == null || key.length() == 0) {
 		  return "";
 	  }
 
@@ -439,34 +439,39 @@ public class MessageText {
   		String		key,
 		String[] params )
   {
-  	String	res = getString(key, (String) null);
+	  String	res = getString(key, (String) null);
 
-		if (res == null) {
-			if (LOG_MISSING_MESSAGES) {
-				System.err.println("Missing message key '" + key + "', params " + Arrays.toString(params));
-			}
+	  if (res == null) {
+		  if (LOG_MISSING_MESSAGES) {
+			  System.err.println("Missing message key '" + key + "', params " + Arrays.toString(params));
+		  }
 
-			if (params == null) {
-				return  "!" + key + "!";
-			}
-			return "!" + key + "(" + Arrays.toString(params) + ")" + "!";
-		}
+		  if (params == null || params.length == 0 ) {
+			  return  "!" + key + "!";
+		  }
+		  return "!" + key + "(" + Arrays.toString(params) + ")" + "!";
+	  }
 
-  	if (params == null) {
-  		return res;
-  	}
+	  if (params == null || params.length == 0 ) {
+		  return res;
+	  }
 
-  	for(int i=0;i<params.length;i++){
+	  for(int i=0;i<params.length;i++){
 
-  		String	from_str 	= "%" + (i+1);
-  		String	to_str		= params[i];
+		  String	from_str 	= "%" + (i+1);
+		  String	to_str		= params[i];
 
-  		to_str = to_str.replace( '%', '\uFDE5' );	// invalid char to avoid %n in to_str being expanded
-  		
-  		res = replaceStrings( res, from_str, to_str );
-  	}
+		  if ( to_str == null ){
 
-  	return( res.replace( '\uFDE5', '%' ));
+			  to_str = "<null>";
+		  }
+
+		  to_str = to_str.replace( '%', '\uFDE5' );	// invalid char to avoid %n in to_str being expanded
+
+		  res = replaceStrings( res, from_str, to_str );
+	  }
+
+	  return( res.replace( '\uFDE5', '%' ));
   }
 
   protected static String
@@ -555,7 +560,7 @@ public class MessageText {
 				}
 			}
 		} else {
-      File bundleDirectory = new File(URI.create(urlString)).getParentFile();
+      File bundleDirectory = FileUtil.newFile(URI.create(urlString)).getParentFile();
       //      System.out.println("bundleDirectory: " +
       // bundleDirectory.getAbsolutePath());
 
@@ -570,7 +575,7 @@ public class MessageText {
     HashSet<String> bundleSet = new HashSet<>();
 
     // Add local first
-    File localDir = new File(SystemProperties.getUserPath());
+    File localDir = FileUtil.newFile(SystemProperties.getUserPath());
     String[] localBundles = localDir.list(new FilenameFilter() {
       @Override
       public boolean accept(File dir, String name) {
@@ -585,7 +590,7 @@ public class MessageText {
     }
 
     // Add AppDir 2nd
-    File appDir = new File(SystemProperties.getApplicationPath());
+    File appDir = FileUtil.newFile(SystemProperties.getApplicationPath());
     String[] appBundles = appDir.list(new FilenameFilter() {
       @Override
       public boolean accept(File dir, String name) {
@@ -657,8 +662,8 @@ public class MessageText {
     	  newLocale = LOCALE_DEFAULT;
 
       try {
-        File userBundleFile = new File(SystemProperties.getUserPath());
-        File appBundleFile = new File(SystemProperties.getApplicationPath());
+        File userBundleFile = FileUtil.newFile(SystemProperties.getUserPath());
+        File appBundleFile = FileUtil.newFile(SystemProperties.getApplicationPath());
 
         // Get the jarURL
         // XXX Is there a better way to get the JAR name?
