@@ -31,6 +31,7 @@ import com.biglybt.core.disk.impl.piecemapper.*;
 import com.biglybt.core.diskmanager.access.DiskAccessController;
 import com.biglybt.core.diskmanager.cache.CacheFile;
 import com.biglybt.core.diskmanager.cache.CacheFileOwner;
+import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.download.DownloadManagerState;
 import com.biglybt.core.internat.LocaleTorrentUtil;
 import com.biglybt.core.internat.LocaleUtilDecoder;
@@ -102,7 +103,20 @@ LWSDiskManager
 		return( lws.getName());
 	}
 
+	@Override
+	public String 
+	getDisplayName()
+	{
+		return( getName());
+	}
 
+	@Override
+	public DownloadManager
+	getDownload()
+	{
+		return( null );
+	}
+	
 	@Override
 	public int
 	getCacheMode()
@@ -177,15 +191,13 @@ LWSDiskManager
 
 				DMPieceMapperFile pm_info = pm_files[i];
 
-				File	relative_file = pm_info.getDataFile();
-
 				long target_length = pm_info.getLength();
 
 				DiskManagerFileInfoImpl	file_info =
 					new DiskManagerFileInfoImpl(
 											this,
-											save_location.toString(),
-											relative_file,
+											save_location,
+											pm_info.getRelativeDataPath(),
 											i,
 											pm_info.getTorrentFile(),
 											DiskManagerFileInfo.ST_LINEAR );
@@ -583,10 +595,10 @@ LWSDiskManager
 	}
 
     @Override
-    public int
+    public long[]
     getMoveProgress()
     {
-    	return( -1 );
+    	return( null );
     }
 
     @Override
@@ -929,4 +941,4 @@ LWSDiskManager
 			//throw( new RuntimeException( "setDone not implemented" ));
 		}
 	}
-}
\ No newline at end of file
+}
