@@ -113,6 +113,28 @@ LWSTorrent
 		lws				= _lws;
 	}
 
+	@Override
+	public int 
+	getTorrentType()
+	{
+		return( TT_V1 );
+	}
+	
+	@Override
+	public boolean 
+	isExportable()
+	{
+		return( true );
+	}
+	
+	@Override
+	public boolean
+	updateExportability(
+		TOTorrent		from )
+	{
+		return( true );
+	}
+	
 	protected TOTorrent
 	getDelegate()
 	{
@@ -285,6 +307,20 @@ LWSTorrent
 		return( lws.getHash().getBytes());
 	}
 
+	@Override
+	public byte[]
+	getFullHash(
+		int	type )
+	
+		throws TOTorrentException
+	{
+		if ( type == TT_V1 ){
+			return( getHash());
+		}else{
+			return( null );
+		}
+	}
+	
 	@Override
 	public HashWrapper
 	getHashWrapper()
@@ -293,7 +329,17 @@ LWSTorrent
 	{
 		return( lws.getHash());
 	}
-
+	
+	@Override
+	public TOTorrent 
+	selectHybridHashType(
+		int type ) 
+		
+		throws TOTorrentException
+	{
+		throw( new TOTorrentException( "Not supported", TOTorrentException.RT_CREATE_FAILED ));
+	}
+	
    	@Override
     public void
 	setHashOverride(
@@ -525,7 +571,7 @@ LWSTorrent
 	public String
 	getRelationText()
 	{
-		return "LWTorrent: '" + new String(getName()) + "'";
+		return "Internal: '" + new String(getName()) + "'";
 	}
 
 	@Override
