@@ -1716,7 +1716,7 @@ SubscriptionImpl
 								// make a version based filename to avoid issues regarding multiple
 								// versions
 
-							final File	versioned_data_location = new File( original_data_location.getParent(), original_data_location.getName() + "." + getVersion());
+							final File	versioned_data_location = FileUtil.newFile( original_data_location.getParent(), original_data_location.getName() + "." + getVersion());
 
 							if ( !versioned_data_location.exists()){
 
@@ -2196,8 +2196,7 @@ SubscriptionImpl
 	}
 
 	protected void
-	fireDownloaded(
-		boolean	was_auto )
+	fireDownloaded()
 	{
 
 		Iterator it = listeners.iterator();
@@ -2205,7 +2204,7 @@ SubscriptionImpl
 		while( it.hasNext()){
 
 			try{
-				((SubscriptionListener)it.next()).subscriptionDownloaded( this, was_auto );
+				((SubscriptionListener)it.next()).subscriptionDownloaded( this );
 
 			}catch( Throwable e ){
 
