@@ -20,11 +20,13 @@
 
 package com.biglybt.core.subs.impl;
 
+import java.io.File;
 import java.io.InputStream;
 import java.net.URL;
 import java.text.SimpleDateFormat;
 import java.util.*;
 
+import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.metasearch.Engine;
 import com.biglybt.core.metasearch.impl.web.WebEngine;
 import com.biglybt.core.proxy.AEProxyFactory;
@@ -58,12 +60,11 @@ SubscriptionSchedulerImpl
 
 	private SubscriptionManagerImpl		manager;
 
-	private Map	active_subscription_downloaders = new HashMap();
-	private boolean active_subs_download_is_auto;
+	private Map<Subscription, List<AESemaphore>>	active_subscription_downloaders = new IdentityHashMap<>();
 
 	private Map<String,Long>	rate_limit_map = new HashMap<>();
 
-	private Set	active_result_downloaders		= new HashSet();
+	private Set<String>	active_result_downloaders		= new HashSet<>();
 
 	private ThreadPool	result_downloader = new ThreadPool( "SubscriptionDownloader", 5, true );
 
@@ -171,9 +172,8 @@ SubscriptionSchedulerImpl
 
 		throws SubscriptionException
 	{
-		SubscriptionDownloader downloader;
-
-		AESemaphore	sem = null;
+		SubscriptionDownloader 	downloader;
+		AESemaphore				sem;
 
 		String rate_limits = manager.getRateLimits().trim();
 
@@ -238,7 +238,7 @@ SubscriptionSchedulerImpl
 				}
 			}
 
-			List	waiting = (List)active_subscription_downloaders.get( subs );
+			List<AESemaphore>	waiting = active_subscription_downloaders.get( subs );
 
 			if ( waiting != null ){
 
@@ -246,22 +246,22 @@ SubscriptionSchedulerImpl
 
 				waiting.add( sem );
 
-				if ( !is_auto ){
-
-					active_subs_download_is_auto = false;
-				}
+				downloader = null;
+				
 			}else{
 
-				active_subscription_downloaders.put( subs, new ArrayList());
-
-				active_subs_download_is_auto = is_auto;
+				sem = null;
+				
+				active_subscription_downloaders.put( subs, new ArrayList<>());
+				
+				downloader = new SubscriptionDownloader(manager, (SubscriptionImpl)subs );
 			}
-
-			downloader = new SubscriptionDownloader(manager, (SubscriptionImpl)subs );
 		}
 
+			// either downloader or sem is non-null
+		
 		try{
-			if ( sem == null ){
+			if ( downloader != null ){
 
 				downloader.download();
 
@@ -274,24 +274,31 @@ SubscriptionSchedulerImpl
 
 		}finally{
 
-			boolean	was_auto;
+				// only release waiters and fire event if this thread did the actual download
+			
+			if ( downloader != null ){
 
-			synchronized( active_subscription_downloaders ){
+				try{	
+					((SubscriptionImpl)subs).fireDownloaded();
 
-				List waiting = (List)active_subscription_downloaders.remove( subs );
-
-				if ( waiting != null ){
-
-					for (int i=0;i<waiting.size();i++){
-
-						((AESemaphore)waiting.get(i)).release();
+				}catch( Throwable e ){
+					
+					Debug.out( e );
+				}
+				
+				synchronized( active_subscription_downloaders ){
+	
+					List<AESemaphore> waiting = active_subscription_downloaders.remove( subs );
+	
+					if ( waiting != null ){
+	
+						for ( AESemaphore waiter: waiting ){
+							
+							waiter.release();
+						}
 					}
 				}
-
-				was_auto = active_subs_download_is_auto;
 			}
-
-			((SubscriptionImpl)subs).fireDownloaded( was_auto );
 		}
 	}
 
@@ -331,13 +338,15 @@ SubscriptionSchedulerImpl
 					public void
 					runSupport()
 					{
-							// need to fix up to the latest history due to the lazy nature of things :(
-
-						SubscriptionResult result = subs.getHistory().getResult( original_result.getID());
-
 						boolean	success = false;
 
+						SubscriptionResult result = null;
+						
+							// need to fix up to the latest history due to the lazy nature of things :(
 						try{
+
+							result = subs.getHistory().getResult( original_result.getID());
+
 							if ( result == null ){
 
 								log( subs.getName() + ": result has been deleted - " + original_result.getID());
@@ -484,13 +493,51 @@ SubscriptionSchedulerImpl
 										manager.addPrepareTrigger( hash, new Subscription[]{ subs }, new SubscriptionResult[]{ result } );
 
 										try{
+											File data_location 	= null;
+											File torrent_location	= null;
+											
+											if ( manager.getAddHashDirs()){
+												
+												String torrent_name = FileUtil.convertOSSpecificChars( torrent.getName(), false );
+												
+												String hash_str = ByteFormatter.encodeString( hash ).substring( 0, 8 );
+												
+												String data_dir = COConfigurationManager.getStringParameter("Default save path");
+
+												if ( data_dir != null && !data_dir.isEmpty()){
+													
+													data_location = FileUtil.newFile( data_dir, torrent_name + "_" + hash_str );
+												}
+												
+												if ( COConfigurationManager.getBooleanParameter("Save Torrent Files")){
+
+											      	String torrent_dir = COConfigurationManager.getDirectoryParameter("General_sDefaultTorrent_Directory");
+
+											      	if ( torrent_dir != null && !torrent_dir.isEmpty()){
+
+											      		torrent_location = FileUtil.newFile( torrent_dir, torrent_name + "_" + hash_str + ".torrent" );
+											      		
+											      		try{
+													    	torrent.writeToFile( torrent_location );
+													    	
+											      		}catch( Throwable e ){
+											      			
+											      			Debug.out( e );
+											      			
+											      			torrent_location = null;
+											      		}
+											      	}
+												}
+											}
+										    
+										    
 											if ( auto_start && !stop_override ){
 
-												download = dm.addDownload( torrent );
+												download = dm.addDownload( torrent, torrent_location, data_location );
 
 											}else{
 
-												download = dm.addDownloadStopped( torrent, null, null );
+												download = dm.addDownloadStopped( torrent, torrent_location, data_location );
 											}
 										}finally{
 
@@ -541,7 +588,7 @@ SubscriptionSchedulerImpl
 						}finally{
 
 							try{
-								if ( !success ){
+								if ( result != null && !success ){
 
 									if ( dl.startsWith( "azplug:" ) || dl.startsWith( "chat:" )){
 
@@ -573,14 +620,16 @@ SubscriptionSchedulerImpl
 							}catch( Throwable e ){
 
 								Debug.out( e );
+								
+							}finally{
+
+								synchronized( active_result_downloaders ){
+	
+									active_result_downloaders.remove( key );
+								}
+	
+								calculateSchedule();
 							}
-
-							synchronized( active_result_downloaders ){
-
-								active_result_downloaders.remove( key );
-							}
-
-							calculateSchedule();
 						}
 					}
 				});
