@@ -1,5 +1,6 @@
 package com.biglybt.core.tag;
 
+import java.io.File;
 import java.util.List;
 
 public interface 
@@ -15,6 +16,13 @@ TagGroup
 	setExclusive(
 		boolean		b );
 	
+	public void
+	setRootMoveOnAssignLocation(
+		File		loc );
+	
+	public File
+	getRootMoveOnAssignLocation();
+	
 	public TagType
 	getTagType();
 	
