@@ -962,13 +962,25 @@ TagManagerImpl
 
 											if ( !new_loc.equals( old_loc )){
 
-												manager.setTorrentSaveDir( new_loc.getAbsolutePath());
+													// it is possible some folder structure has already been created, we need to remove this
+													// as simply setting the save dir doesn't do this.
+													// example is if we have a dnd-dir configured and one or more skipped files. this
+													// results in the dnd-dir(s) being pre-created and they will remain in the old
+													// save location if we don't remove them+it
+												
+												if ( old_loc.isDirectory()){
+													
+													TorrentUtils.recursiveEmptyDirDelete( old_loc, false );
+												}
+												
+												manager.setTorrentSaveDir(FileUtil.newFile(
+													new_loc.getAbsolutePath()), false);
 											}
 										}
 
 										if ( set_torrent ){
 
-											File old_torrent_file = new File( manager.getTorrentFileName());
+											File old_torrent_file = FileUtil.newFile( manager.getTorrentFileName());
 
 											if ( old_torrent_file.exists()){
 
@@ -1152,7 +1164,7 @@ TagManagerImpl
 
 											if ( manager.getAssumedComplete()){
 
-												if ( dm.getMoveProgress() == -1 && dm.getCompleteRecheckStatus() == -1 ){
+												if ( dm.getMoveProgress() == null && dm.getCompleteRecheckStatus() == -1 ){
 
 													long	now = SystemTime.getMonotonousTime();
 
@@ -1212,11 +1224,11 @@ TagManagerImpl
 
 			if ( copy_torrent ){
 
-				File old_file = new File( manager.getTorrentFileName());
+				File old_file = FileUtil.newFile( manager.getTorrentFileName());
 
 				if ( old_file.exists()){
 
-					File new_file = new File( new_loc, old_file.getName());
+					File new_file = FileUtil.newFile( new_loc, old_file.getName());
 
 					FileUtil.copyFile( old_file, new_file );
 				}
@@ -1507,13 +1519,14 @@ TagManagerImpl
 
 										if ( ! ( existing_save_loc.equals( save_loc ) || existing_save_loc.exists())){
 
-											dm.setTorrentSaveDir( save_loc.getAbsolutePath());
+											dm.setTorrentSaveDir(FileUtil.newFile(
+												save_loc.getAbsolutePath()), false);
 										}
 									}
 
 									if ( set_torrent ){
 
-										File old_torrent_file = new File( dm.getTorrentFileName());
+										File old_torrent_file = FileUtil.newFile( dm.getTorrentFileName());
 
 										if ( old_torrent_file.exists()){
 
@@ -1624,7 +1637,7 @@ TagManagerImpl
 	
 			if ( set_torrent ){
 	
-				File old_torrent_file = new File( dm.getTorrentFileName());
+				File old_torrent_file = FileUtil.newFile( dm.getTorrentFileName());
 	
 				if ( old_torrent_file.exists()){
 	
@@ -1912,6 +1925,8 @@ TagManagerImpl
 		
 		pi.getUtilities().getLocaleUtilities().integrateLocalisedMessageBundle( props );
 		
+		int[]	interesting_tts = { TagType.TT_DOWNLOAD_MANUAL, TagType.TT_DOWNLOAD_CATEGORY };
+		
 		tm.registerColumn(
 			Download.class,
 			col_id_text,
@@ -1935,7 +1950,7 @@ TagManagerImpl
 								return;
 							}
 							
-							List<Tag> tags = TagManagerImpl.this.getTagsForTaggable( TagType.TT_DOWNLOAD_MANUAL, PluginCoreUtils.unwrap( dl ));
+							List<Tag> tags = TagManagerImpl.this.getTagsForTaggable( interesting_tts, PluginCoreUtils.unwrap( dl ));
 
 							String sTags = null;
 							
