@@ -48,6 +48,8 @@ import com.biglybt.core.tracker.client.TRTrackerScraperResponse;
 import com.biglybt.core.util.*;
 import com.biglybt.pif.download.Download;
 import com.biglybt.pif.download.DownloadListener;
+import com.biglybt.pif.sharing.ShareException;
+import com.biglybt.pif.sharing.ShareManager;
 import com.biglybt.pifimpl.local.PluginCoreUtils;
 
 public class
@@ -64,7 +66,8 @@ TagPropertyConstraintHandler
 	
 	private final Core core;
 	private final TagManagerImpl	tag_manager;
-
+	private final ShareManager		share_manager;
+	
 	
 	private boolean		initialised;
 	private boolean 	initial_assignment_complete;
@@ -110,30 +113,43 @@ TagPropertyConstraintHandler
 	{
 		core				= null;
 		tag_manager			= null;
+		share_manager		= null;
 	}
 
 	protected
 	TagPropertyConstraintHandler(
-		Core _core,
+		Core 			_core,
 		TagManagerImpl	_tm )
 	{
 		core			= _core;
 		tag_manager		= _tm;
 
-		if( core != null ){
-
-			core.addLifecycleListener(
-				new CoreLifecycleAdapter()
-				{
-					@Override
-					public void
-					stopping(Core core)
-					{
-						stopping	= true;
-					}
-				});
+		ShareManager sm;
+		
+		try{
+			sm	= core.getPluginManager().getDefaultPluginInterface().getShareManager();
+			
+		}catch(  Throwable e ){
+			
+			Debug.out( e );
+			
+			sm = null;
 		}
 		
+		share_manager = sm;
+		
+		core.addLifecycleListener(
+			new CoreLifecycleAdapter()
+			{
+				@Override
+				public void
+				stopping(Core core)
+				{
+					stopping	= true;
+				}
+			});
+		
+		
 		tag_manager.addTaggableLifecycleListener(
 			Taggable.TT_DOWNLOAD,
 			new TaggableLifecycleAdapter()
@@ -2051,6 +2067,7 @@ TagPropertyConstraintHandler
 		private static final int FT_IS_SUPER_SEEDING	= 32;		
 		private static final int FT_IS_SEQUENTIAL		= 33;		
 		private static final int FT_TAG_POSITION		= 34;		
+		private static final int FT_IS_SHARE			= 35;		
 		
 		private static final int	DEP_STATIC		= 0;
 		private static final int	DEP_RUNNING		= 1;
@@ -2095,6 +2112,7 @@ TagPropertyConstraintHandler
 		private static final int	KW_FILE_NAMES_SELECTED	= 34;
 		private static final int	KW_FILE_EXTS			= 35;
 		private static final int	KW_FILE_EXTS_SELECTED	= 36;
+		private static final int	KW_TORRENT_TYPE			= 37;
 
 		static{
 			keyword_map.put( "shareratio", 				new int[]{KW_SHARE_RATIO,			DEP_RUNNING });
@@ -2177,6 +2195,10 @@ TagPropertyConstraintHandler
 			keyword_map.put( "max_up", 					new int[]{KW_MAX_UP,				DEP_RUNNING });
 			keyword_map.put( "maxdown", 				new int[]{KW_MAX_DOWN,				DEP_RUNNING });
 			keyword_map.put( "max_down", 				new int[]{KW_MAX_DOWN,				DEP_RUNNING });
+			
+			keyword_map.put( "torrent_type",			new int[]{KW_TORRENT_TYPE,			DEP_STATIC });
+			keyword_map.put( "torrenttype", 			new int[]{KW_TORRENT_TYPE,			DEP_STATIC });
+
 		}
 
 		private class
@@ -2284,6 +2306,12 @@ TagPropertyConstraintHandler
 					fn_type = FT_IS_PRIVATE;
 
 					params_ok = num_params == 0;
+					
+				}else if ( func_name.equals( "isShare" )){
+
+					fn_type = FT_IS_SHARE;
+
+					params_ok = num_params == 0;
 
 				}else if ( func_name.equals( "isForceStart" )){
 
@@ -2839,6 +2867,23 @@ TagPropertyConstraintHandler
 
 						return( t != null && t.getPrivate());
 					}
+					case FT_IS_SHARE:{
+
+						ShareManager sm = handler.share_manager;
+						
+						if ( sm == null ){
+							
+							return( false );
+						}
+							
+						try{
+							return( sm.lookupShare( dm.getTorrent().getHash()) != null );
+							
+						}catch( Throwable e ){
+							
+							return( false );
+						}
+					}
 					case FT_IS_FORCE_START:{
 
 						return( dm.isForceStart());
@@ -3931,6 +3976,12 @@ TagPropertyConstraintHandler
 								
 								return( dm.getStats().getDownloadRateLimitBytesPerSecond());
 							}
+							case KW_TORRENT_TYPE:{
+								
+								TOTorrent t = dm.getTorrent();
+
+								return( t==null?0:t.getTorrentType());
+							}
 							default:{
 
 								setError( "Invalid constraint keyword: " + str );
