@@ -38,6 +38,7 @@ TOTorrentCreateImpl
 	implements	TOTorrentFileHasherListener
 {
 	private static final Comparator<File> file_comparator;
+	private static final Comparator<File> file_comparator_v2;
 
 	static{
 		if ( System.getProperty( "az.create.torrent.alphanumeric.sort", "0" ).equals( "1" )){
@@ -131,19 +132,41 @@ TOTorrentCreateImpl
 
 			file_comparator = null;
 		}
+		
+		file_comparator_v2 =
+				new Comparator<File>()
+				{
+					@Override
+					public int
+					compare(
+						File	f1,
+						File	f2 )
+					{
+						return( f1.getName().compareTo( f2.getName()));
+					}
+				};
 	}
-
+	
+	private final int						torrent_type;
+	
 	private File							torrent_base;
 	private long							piece_length;
 
 	private TOTorrentFileHasher			file_hasher;
 
-	private long	total_file_size		= -1;
-	private long	total_file_count	= 0;
+	private long	total_file_size_no_pad		= -1;
+	private long	total_file_count_no_pad		= 0;
 
-	private long							piece_count;
+	private long	piece_count_no_pad;
+	
 	private boolean							add_other_hashes;
-
+	
+	private boolean							add_pad_files = false;
+	private int								pad_file_num;
+	private long							pad_file_sizes;
+	private final boolean					add_v1;
+	private final boolean					add_v2;
+	
 	private final List<TOTorrentProgressListener>							progress_listeners = new ArrayList<>();
 
 	private int	reported_progress;
@@ -153,10 +176,11 @@ TOTorrentCreateImpl
 	private Map<String,File>	linkage_map;
 	private final Map<String,String>	linked_tf_map = new HashMap<>();
 
-	private boolean	cancelled;
+	private volatile boolean	cancelled;
 
 	protected
 	TOTorrentCreateImpl(
+		int							_torrent_type,
 		Map<String,File>			_linkage_map,
 		File						_torrent_base,
 		URL							_announce_url,
@@ -171,10 +195,28 @@ TOTorrentCreateImpl
 		torrent_base		= _torrent_base;
 		piece_length		= _piece_length;
 		add_other_hashes	= _add_other_hashes;
+		
+		torrent_type = _torrent_type;
+		
+		add_v1	= torrent_type == TT_V1 || torrent_type == TT_V1_V2;
+		add_v2	= torrent_type == TT_V2 || torrent_type == TT_V1_V2;
+		
+		if ( add_v2 ){
+			
+			long highestOneBit = Long.highestOneBit( piece_length );
+			
+				// if not already power of two round up to next
+			
+			if ( piece_length != highestOneBit ) {
+										
+				piece_length =  highestOneBit << 1;	// round up to next power of 2
+			}
+		}
 	}
 
 	protected
 	TOTorrentCreateImpl(
+		int							_torrent_type,
 		Map<String,File>			_linkage_map,
 		File						_torrent_base,
 		URL							_announce_url,
@@ -192,31 +234,99 @@ TOTorrentCreateImpl
 		torrent_base		= _torrent_base;
 		add_other_hashes	= _add_other_hashes;
 
+		torrent_type = _torrent_type;
+
+		add_v1	= torrent_type == TT_V1 || torrent_type == TT_V1_V2;
+		add_v2	= torrent_type == TT_V2 || torrent_type == TT_V1_V2;
+
 		long	total_size = calculateTotalFileSize( _torrent_base );
 
 		piece_length = getComputedPieceSize( total_size, _piece_min_size, _piece_max_size, _piece_num_lower, _piece_num_upper );
+		
+		if ( add_v2 ){
+			
+			long highestOneBit = Long.highestOneBit( piece_length );
+			
+				// if not already power of two round up to next
+			
+			if ( piece_length != highestOneBit ) {
+										
+				piece_length =  highestOneBit << 1;	// round up to next power of 2
+			}
+		}
 	}
 
 	protected void
-	create()
+	create(
+		boolean		skip_hashing )
 
 		throws TOTorrentException
 	{
+		if ( !( add_v1 || add_v2 )){
+			
+			throw( new TOTorrentException( "No torrent versions selected", TOTorrentException.RT_CREATE_FAILED ));
+		}
+	
+		if ( add_v2 ){
+			
+			add_pad_files = true;
+		}
+		
 		try{
-			int ignored = constructFixed( torrent_base, piece_length );
-	
-				// linkage map doesn't include ignored files, if it is supplied, so take account of this when
-				// checking that linkages have resolved correctly
-	
-			if ( 	linkage_map.size() > 0 &&
-					linkage_map.size() != ( linked_tf_map.size() + ignored )){
-	
-				throw( new TOTorrentException( "TOTorrentCreate: unresolved linkages: required=" + linkage_map + ", resolved=" + linked_tf_map,
-						TOTorrentException.RT_DECODE_FAILS));
+			setIgnoreList();
+
+			setTorrentType( torrent_type );
+			
+			setCreationDate( SystemTime.getCurrentTime() / 1000);
+
+			setCreatedBy( Constants.BIGLYBT_NAME + "/" + Constants.BIGLYBT_VERSION );
+
+			setPieceLength( piece_length );
+
+			piece_count_no_pad = calculateNumberOfPieces( torrent_base, piece_length );
+
+			if ( piece_count_no_pad == 0 ){
+
+				throw( new TOTorrentException( "Specified files have zero total length",
+												TOTorrentException.RT_ZERO_LENGTH ));
 			}
-	
+
+			report( "Torrent.create.progress.piecelength", piece_length );
+
+			if ( add_v1 ){
+				
+				int ignored = createV1( skip_hashing );
+		
+					// linkage map doesn't include ignored files, if it is supplied, so take account of this when
+					// checking that linkages have resolved correctly
+		
+				if ( 	linkage_map.size() > 0 &&
+						linkage_map.size() != ( linked_tf_map.size() + ignored )){
+		
+					throw( new TOTorrentException( "Unresolved V1 linkages: required=" + linkage_map + ", resolved=" + linked_tf_map,
+							TOTorrentException.RT_DECODE_FAILS));
+				}
+			}
+			
+			if ( add_v2 ){
+				
+				linked_tf_map.clear();
+				
+				int ignored = createV2( skip_hashing );
+				
+					// linkage map doesn't include ignored files, if it is supplied, so take account of this when
+					// checking that linkages have resolved correctly
+		
+				if ( 	linkage_map.size() > 0 &&
+						linkage_map.size() != ( linked_tf_map.size() + ignored )){
+		
+					throw( new TOTorrentException( "Unresolved V2 linkages: required=" + linkage_map + ", resolved=" + linked_tf_map,
+							TOTorrentException.RT_DECODE_FAILS));
+				}
+			}
+			
 			if ( linked_tf_map.size() > 0 ){
-	
+				
 				Map	m = getAdditionalMapProperty( TOTorrent.AZUREUS_PRIVATE_PROPERTIES );
 	
 				if ( m == null ){
@@ -256,31 +366,12 @@ TOTorrentCreateImpl
 	}
 
 	private int
-	constructFixed(
-		File		_torrent_base,
-		long		_piece_length )
-
+	createV1(
+		boolean		skip_hashing )
+	
 		throws TOTorrentException
 	{
-		setIgnoreList();
-
-		setCreationDate( SystemTime.getCurrentTime() / 1000);
-
-		setCreatedBy( Constants.BIGLYBT_NAME + "/" + Constants.BIGLYBT_VERSION );
-
-		setPieceLength( _piece_length );
-
-		report( "Torrent.create.progress.piecelength", _piece_length );
-
-		piece_count = calculateNumberOfPieces( _torrent_base,_piece_length );
-
-		if ( piece_count == 0 ){
-
-			throw( new TOTorrentException( "TOTorrentCreate: specified files have zero total length",
-											TOTorrentException.RT_ZERO_LENGTH ));
-		}
-
-		report( "Torrent.create.progress.hashing");
+		report( "Torrent.create.progress.hashing", (add_v2?": V1":"" ));
 
 		for (int i=0;i<progress_listeners.size();i++){
 
@@ -293,28 +384,33 @@ TOTorrentCreateImpl
 			new TOTorrentFileHasher(
 					add_other_hashes,
 					add_other_per_file_hashes,
-					(int)_piece_length,
+					(int)piece_length,
 					progress_listeners.size()==0?null:this );
 
+		if ( skip_hashing ){
+			
+			file_hasher.setSkipHashing( true );
+		}
+		
 		int	ignored = 0;
 
 		try{
 			if ( cancelled ){
 
-				throw( new TOTorrentException( 	"TOTorrentCreate: operation cancelled",
+				throw( new TOTorrentException( 	"Operation cancelled",
 												TOTorrentException.RT_CANCELLED ));
 			}
 
 			if ( getSimpleTorrent()){
 
-				File link = linkage_map.get( _torrent_base.getName());
+				File link = linkage_map.get( torrent_base.getName());
 
 				if ( link != null ){
 
 					linked_tf_map.put( "0", link.getAbsolutePath());
 				}
 
-				long length = file_hasher.add( link==null?_torrent_base:link );
+				long length = file_hasher.add( link==null?torrent_base:link );
 
 				setFiles( new TOTorrentFileImpl[]{ new TOTorrentFileImpl( this, 0, 0, length, new byte[][]{ getName()})});
 
@@ -324,7 +420,7 @@ TOTorrentCreateImpl
 
 				List<TOTorrentFileImpl>	encoded = new ArrayList<>();
 
-				ignored = processDir( file_hasher, _torrent_base, encoded, _torrent_base.getName(), "" );
+				ignored = processDir( file_hasher, torrent_base, encoded, torrent_base.getName(), "", new long[1] );
 
 				TOTorrentFileImpl[] files = new TOTorrentFileImpl[ encoded.size()];
 
@@ -342,9 +438,6 @@ TOTorrentCreateImpl
 
 				addAdditionalInfoProperty( "sha1", sha1_digest );
 				addAdditionalInfoProperty( "ed2k", ed2k_digest );
-
-				//System.out.println( "overall:sha1 = " + ByteFormatter.nicePrint( sha1_digest, true));
-				//System.out.println( "overall:ed2k = " + ByteFormatter.nicePrint( ed2k_digest, true));
 			}
 
 			return( ignored );
@@ -355,13 +448,118 @@ TOTorrentCreateImpl
 		}
 	}
 
+	private int
+	createV2(
+		boolean		skip_hashing )	// not implemented yet
+	
+		throws TOTorrentException
+	{
+		report( "Torrent.create.progress.hashing", (add_v1?": V2":"" ));
+
+		for (int i=0;i<progress_listeners.size();i++){
+
+			((TOTorrentProgressListener)progress_listeners.get(i)).reportProgress( 0 );
+		}
+
+		TOTorrentCreateV2Impl v2_creator = 
+			new TOTorrentCreateV2Impl( 
+				torrent_base, 
+				piece_length,
+				new TOTorrentCreateV2Impl.Adapter(){
+					
+					@Override
+					public File 
+					resolveFile(
+						int			index,
+						File 		file, 
+						String 		relative_file)
+					{
+						File link = linkage_map.get( relative_file );
+							
+						if ( link != null ){
+
+							linked_tf_map.put( String.valueOf( index ), link.getAbsolutePath());
+						}
+
+						return( link );
+					}
+					
+					@Override
+					public void 
+					reportHashedBytes(
+						long 	bytes )
+					{
+						pieceHashed( (int)( bytes/piece_length ));
+					}
+					
+					@Override
+					public void 
+					report(
+						String resource_key)
+					{
+						TOTorrentCreateImpl.this.report( resource_key );
+					}
+					
+					@Override
+					public boolean 
+					ignore(
+						String name)
+					{
+						return( ignoreFile( name ));
+					}
+					
+					@Override
+					public boolean 
+					cancelled()
+					{
+						return( cancelled );
+					}
+				});
+			
+		Map<String,Object> v2_torrent = v2_creator.create();
+		
+		pieceHashed( (int)piece_count_no_pad );
+		
+		if ( add_v1 ){
+						
+			if ( v2_creator.getTotalFileSize() != total_file_size_no_pad ){
+				
+				throw( new TOTorrentException( "V1 and V2 total file sizes inconsistent",
+						TOTorrentException.RT_READ_FAILS ));
+			}
+			
+			if ( v2_creator.getTotalPadding() != pad_file_sizes ){
+				
+				throw( new TOTorrentException( "V1 and V2 padding file sizes inconsistent",
+						TOTorrentException.RT_READ_FAILS ));
+			}
+			
+			// verify files
+		}
+		
+		setAdditionalStringProperty( TK_ENCODING, "UTF-8" );
+		
+		setAdditionalMapProperty( TK_V2_PIECE_LAYERS, (Map)v2_torrent.get( TK_V2_PIECE_LAYERS  ));
+		
+		Map v2_info = (Map)v2_torrent.get( TK_INFO );
+		
+		Map v2_file_tree = (Map)v2_info.get( TK_V2_FILE_TREE );
+		
+		addAdditionalInfoProperty( TK_V2_FILE_TREE, v2_file_tree );
+		
+		addAdditionalInfoProperty( TK_V2_META_VERSION, v2_info.get( TK_V2_META_VERSION ));
+		
+		return( v2_creator.getIgnoredFiles());
+	}
+	
 	private int
 	processDir(
 		TOTorrentFileHasher			hasher,
 		File						dir,
 		List<TOTorrentFileImpl>		encoded,
 		String						base_name,
-		String						root )
+		String						root,
+		long[]						torrent_offset )
 
 		throws TOTorrentException
 	{
@@ -369,7 +567,7 @@ TOTorrentCreateImpl
 
 		if ( dir_file_list == null ){
 
-			throw( new TOTorrentException( "TOTorrentCreate: directory '" + dir.getAbsolutePath() + "' returned error when listing files in it",
+			throw( new TOTorrentException( "Directory '" + dir.getAbsolutePath() + "' returned error when listing files in it",
 					TOTorrentException.RT_FILE_NOT_FOUND ));
 
 		}
@@ -378,17 +576,21 @@ TOTorrentCreateImpl
 
 		List<File> file_list = new ArrayList<>(Arrays.asList(dir_file_list));
 
-		if ( file_comparator == null ){
+		if ( add_v2 ){
 
-			Collections.sort(file_list);
+				// v2 requires files to be in 'natural' order, default File comparator isn't
+			
+			Collections.sort( file_list, file_comparator_v2 );
+			
+		}else  if ( file_comparator == null ){
+			
+			Collections.sort(file_list); 
 
 		}else{
 
 			Collections.sort( file_list,file_comparator);
 		}
 
-		long	offset	= 0;
-
 		int	ignored = 0;
 
 		for (int i=0;i<file_list.size();i++){
@@ -406,7 +608,7 @@ TOTorrentCreateImpl
 						file_name = root + File.separator + file_name ;
 					}
 
-					ignored += processDir( hasher, file, encoded, base_name, file_name );
+					ignored += processDir( hasher, file, encoded, base_name, file_name, torrent_offset );
 
 				}else{
 
@@ -416,6 +618,34 @@ TOTorrentCreateImpl
 
 					}else{
 
+						if ( add_pad_files ){
+							
+							long offset = torrent_offset[0];
+							
+							long l = offset%piece_length;
+							
+							if ( l > 0 ){
+								
+								long pad_size = piece_length - l;
+							
+									// System.out.println( "V1: " + file_name + ", " + offset + ", " + pad_size );
+								
+								hasher.addPad((int)pad_size);
+
+								String pad_file = ".pad" + File.separator + (++pad_file_num) + "_" + pad_size;
+									
+								pad_file_sizes += pad_size;
+								
+								TOTorrentFileImpl	tf = new TOTorrentFileImpl( this, i, torrent_offset[0], pad_size, pad_file );
+
+								tf.setAdditionalProperty( TOTorrentImpl.TK_BEP47_ATTRS, "p".getBytes( Constants.UTF_8 ));
+								
+								torrent_offset[0] += pad_size;
+								
+								encoded.add( tf );
+							}
+						}
+						
 						if ( root.length() > 0 ){
 
 							file_name = root + File.separator + file_name;
@@ -430,9 +660,9 @@ TOTorrentCreateImpl
 
 						long length = hasher.add( link==null?file:link );
 
-						TOTorrentFileImpl	tf = new TOTorrentFileImpl( this, i, offset, length, file_name );
+						TOTorrentFileImpl	tf = new TOTorrentFileImpl( this, i, torrent_offset[0], length, file_name );
 
-						offset += length;
+						torrent_offset[0] += length;
 
 						if ( add_other_hashes ){
 
@@ -460,14 +690,19 @@ TOTorrentCreateImpl
 	pieceHashed(
 		int		piece_number )
 	{
-		for (int i=0;i<progress_listeners.size();i++){
+		int	this_progress = (int)((piece_number*100)/piece_count_no_pad );
 
-			int	this_progress = (int)((piece_number*100)/piece_count );
+		if ( this_progress > 100 ){
+			
+			this_progress = 100;
+		}
+		
+		if ( this_progress != reported_progress ){
 
-			if ( this_progress != reported_progress ){
-
-				reported_progress = this_progress;
+			reported_progress = this_progress;
 
+			for (int i=0;i<progress_listeners.size();i++){
+	
 				((TOTorrentProgressListener)progress_listeners.get(i)).reportProgress( reported_progress );
 			}
 		}
@@ -493,12 +728,12 @@ TOTorrentCreateImpl
 
 		throws TOTorrentException
 	{
-		if ( total_file_size == -1 ){
+		if ( total_file_size_no_pad == -1 ){
 
-			total_file_size = getTotalFileSize( file );
+			total_file_size_no_pad = getTotalFileSize( file );
 		}
 
-		return( total_file_size );
+		return( total_file_size_no_pad );
 	}
 
 	protected long
@@ -513,7 +748,7 @@ TOTorrentCreateImpl
 
 		report( "Torrent.create.progress.totalfilesize", res );
 
-		report( "Torrent.create.progress.totalfilecount", ""+total_file_count );
+		report( "Torrent.create.progress.totalfilecount", ""+total_file_count_no_pad );
 
 		return( res );
 	}
@@ -534,7 +769,7 @@ TOTorrentCreateImpl
 
 		if ( !file.exists()){
 
-			throw( new TOTorrentException( "TOTorrentCreate: file '" + file.getName() + "' doesn't exist",
+			throw( new TOTorrentException( "File '" + file.getName() + "' doesn't exist",
 											TOTorrentException.RT_FILE_NOT_FOUND ));
 		}
 
@@ -542,7 +777,7 @@ TOTorrentCreateImpl
 
 			if ( !ignoreFile( name )){
 
-				total_file_count++;
+				total_file_count_no_pad++;
 
 				if ( root.length() > 0 ){
 
@@ -563,7 +798,7 @@ TOTorrentCreateImpl
 
 			if ( dir_files == null ){
 
-				throw( new TOTorrentException( "TOTorrentCreate: directory '" + file.getAbsolutePath() + "' returned error when listing files in it",
+				throw( new TOTorrentException( "Directory '" + file.getAbsolutePath() + "' returned error when listing files in it",
 						TOTorrentException.RT_FILE_NOT_FOUND ));
 
 			}
@@ -616,7 +851,7 @@ TOTorrentCreateImpl
 
 			for (int i=0;i<progress_listeners.size();i++){
 
-				((TOTorrentProgressListener)progress_listeners.get(i)).reportCurrentTask( prefix + (additional_text==null?"":additional_text ));
+				((TOTorrentProgressListener)progress_listeners.get(i)).reportCurrentTask( prefix + (additional_text==null?"":( " " + additional_text )));
 			}
 		}
 	}
@@ -668,7 +903,7 @@ TOTorrentCreateImpl
 		return( (total_size + (piece_size-1))/piece_size );
 	}
 
-	protected void
+	private void
 	setIgnoreList()
 	{
 		try{
