@@ -23,6 +23,7 @@ package com.biglybt.core.torrent.impl;
 
 import java.io.*;
 import java.net.URL;
+import java.security.MessageDigest;
 import java.util.*;
 
 import com.biglybt.core.CoreFactory;
@@ -60,10 +61,18 @@ TOTorrentImpl
 
 	protected static final String TK_HASH_OVERRIDE		= "hash-override";
 
+	protected static final String TK_ENCODING			= "encoding";
+		
 	protected static final List	TK_ADDITIONAL_OK_ATTRS =
 		Arrays.asList(new String[]{ TK_COMMENT_UTF8, AZUREUS_PROPERTIES, TK_WEBSEED_BT, TK_WEBSEED_GR });
 
+
+	protected static final String TK_BEP47_ATTRS		= "attr";
 	
+	protected static final String TK_V2_META_VERSION	= "meta version";
+	protected static final String TK_V2_FILE_TREE		= "file tree";
+	protected static final String TK_V2_PIECE_LAYERS	= "piece layers";
+	protected static final String TK_V2_PIECES_ROOT		= "pieces root";
 	
 	private static CopyOnWriteList<TOTorrentListener>		global_listeners = new CopyOnWriteList<>();
 	
@@ -81,6 +90,8 @@ TOTorrentImpl
 		global_listeners.remove( listener );
 	}
 	
+	private int								torrent_type;
+	
 	private byte[]							torrent_name;
 	private byte[]							torrent_name_utf8;
 
@@ -89,14 +100,21 @@ TOTorrentImpl
 	private final TOTorrentAnnounceURLGroupImpl	announce_group = new TOTorrentAnnounceURLGroupImpl(this);
 
 	private long		piece_length;
+	
 	private byte[][]	pieces;
 	private int			number_of_pieces;
 
 	private byte[]		torrent_hash_override;
-
-	private byte[]		torrent_hash;
+	
+	private byte[]		torrent_hash;		
 	private HashWrapper	torrent_hash_wrapper;
 
+	private byte[]		torrent_hash_v1;
+	private HashWrapper	torrent_hash_wrapper_v1;
+	
+	private byte[]		torrent_hash_v2;
+	private HashWrapper	torrent_hash_wrapper_v2;
+
 	private boolean				simple_torrent;
 	private TOTorrentFileImpl[]	files;
 
@@ -104,7 +122,7 @@ TOTorrentImpl
 	private byte[]				created_by;
 
 	private Map					additional_properties 		= new LightHashMap(4);
-	private final Map					additional_info_properties	= new LightHashMap(4);
+	private final Map			additional_info_properties	= new LightHashMap(4);
 
 	private boolean				created;
 	private boolean				serialising;
@@ -114,6 +132,7 @@ TOTorrentImpl
 	protected final AEMonitor this_mon 	= new AEMonitor( "TOTorrent" );
 
 	private boolean	constructing = true;
+	private boolean fixed_up_root_hashes;
 	
 	/**
 	 * Constructor for deserialisation
@@ -145,8 +164,45 @@ TOTorrentImpl
 
 	protected void
 	setConstructed()
+	
+		throws TOTorrentException
 	{
-		constructing = false;
+		try{
+			if ( torrent_type == TT_V2 ){
+			
+				TOTorrentCreateV2Impl.lashUpV1Info( this );
+				
+				if ( files.length == 1 ){
+					
+					if ( files[0].getPathComponents().length == 1 ){
+						
+						setSimpleTorrent( true );
+					}
+				}
+			}else if ( torrent_type == TT_V1_V2 ){
+									
+				// defer until needed
+				// TOTorrentCreateV2Impl.setV2FileHashes( this );
+			}
+				
+		}finally{
+		
+			constructing = false;
+		}
+	}
+	
+	protected void
+	fixupRootHashes()
+	{
+		if ( torrent_type == TT_V1_V2 ){
+			
+			if ( !fixed_up_root_hashes ){
+				
+				TOTorrentCreateV2Impl.setV2FileHashes( this );
+				
+				fixed_up_root_hashes = true;
+			}
+		}
 	}
 	
 	@Override
@@ -210,7 +266,7 @@ TOTorrentImpl
 			} // end if (!parent.isDirectory)
 
 
-			File temp = new File( parent, output_file.getName() + ".saving");
+			File temp = FileUtil.newFile( parent, output_file.getName() + ".saving");
 
 			if ( temp.exists()){
 
@@ -235,7 +291,7 @@ TOTorrentImpl
 				}
 			}
 
-            FileOutputStream fos = new FileOutputStream( temp, false );
+            FileOutputStream fos = FileUtil.newFileOutputStream( temp );
 
             bos = new BufferedOutputStream( fos, 8192 );
 
@@ -341,7 +397,7 @@ TOTorrentImpl
 
 		TOTorrentAnnounceURLSet[] sets = announce_group.getAnnounceURLSets();
 
-		if (sets.length > 0 ){
+		if ( sets.length > 0 ){
 
 			List	announce_list = new ArrayList();
 
@@ -386,27 +442,105 @@ TOTorrentImpl
 
 			root.put( TK_CREATED_BY, created_by );
 		}
-
+		
 		Map info = new HashMap();
 
 		root.put( TK_INFO, info );
 
 		info.put( TK_PIECE_LENGTH, new Long( piece_length ));
 
-		if ( pieces == null ){
+		if ( torrent_type != TT_V2 ){
+			
+			if ( pieces == null ){
+	
+				throw( new TOTorrentException( "Pieces is null", TOTorrentException.RT_WRITE_FAILS ));
+			}
+	
+			byte[]	flat_pieces = new byte[pieces.length*20];
+	
+			for (int i=0;i<pieces.length;i++){
+	
+				System.arraycopy( pieces[i], 0, flat_pieces, i*20, 20 );
+			}
+	
+			info.put( TK_PIECES, flat_pieces );
+			
+			if ( simple_torrent ){
 
-			throw( new TOTorrentException( "Pieces is null", TOTorrentException.RT_WRITE_FAILS ));
+				TOTorrentFile	file = files[0];
+
+				info.put( TK_LENGTH, new Long( file.getLength()));
+
+			}else{
+
+				List	meta_files = new ArrayList();
+
+				info.put( TK_FILES, meta_files );
+
+				for (int i=0;i<files.length;i++){
+
+					TOTorrentFileImpl	file	= files[i];
+
+					Map	file_map = file.serializeToMap();
+
+					meta_files.add( file_map );
+
+				}
+			}
+		}else{
+			
+				// files can be null here if we're in an error path trying to grab the torrent's hash
+			
+			if ( getAdditionalMapProperty( TOTorrentImpl.TK_V2_PIECE_LAYERS ) == null && files != null ){
+			
+				ByteEncodedKeyHashMap<String,byte[]>	pl_map 		= new ByteEncodedKeyHashMap<>();
+				
+				Map<String,Object>	hash_tree_state = new HashMap<>();
+
+				boolean complete = true;
+				
+				for ( TOTorrentFileImpl file: files ){
+				
+					if ( file.getLength() > piece_length ){
+						
+						TOTorrentFileHashTreeImpl tree = file.getHashTree();
+						
+						byte[] root_hash = tree.getRootHash();
+						
+						byte[] piece_layer = tree.getPieceLayer();
+				
+						if ( piece_layer == null ){
+							
+							complete = false;
+							
+							Map export = tree.exportState();
+							
+							if ( export != null ){
+								
+								hash_tree_state.put( String.valueOf( file.getIndex()), export );
+							}
+						}else{
+						
+							pl_map.put( new String( root_hash, Constants.BYTE_ENCODING_CHARSET ), piece_layer );
+						}
+					}
+				}
+				
+				if ( complete ){
+										
+					setAdditionalMapProperty( TOTorrentImpl.TK_V2_PIECE_LAYERS, pl_map );
+					
+				}else{
+					
+					
+					TorrentUtils.setHashTreeState( this, hash_tree_state );
+				}
+			}else{
+				
+				TorrentUtils.setHashTreeState( this, null );
+			}
 		}
-
-		byte[]	flat_pieces = new byte[pieces.length*20];
-
-		for (int i=0;i<pieces.length;i++){
-
-			System.arraycopy( pieces[i], 0, flat_pieces, i*20, 20 );
-		}
-
-		info.put( TK_PIECES, flat_pieces );
-
+		
 		info.put( TK_NAME, torrent_name );
 
 		if ( torrent_name_utf8 != null ){
@@ -419,29 +553,6 @@ TOTorrentImpl
 			info.put( TK_HASH_OVERRIDE, torrent_hash_override );
 		}
 
-		if ( simple_torrent ){
-
-			TOTorrentFile	file = files[0];
-
-			info.put( TK_LENGTH, new Long( file.getLength()));
-
-		}else{
-
-			List	meta_files = new ArrayList();
-
-			info.put( TK_FILES, meta_files );
-
-			for (int i=0;i<files.length;i++){
-
-				TOTorrentFileImpl	file	= files[i];
-
-				Map	file_map = file.serializeToMap();
-
-				meta_files.add( file_map );
-
-			}
-		}
-
 		Iterator info_it = additional_info_properties.keySet().iterator();
 
 		while( info_it.hasNext()){
@@ -485,6 +596,80 @@ TOTorrentImpl
 		serialiser.serialiseToFile( file );
 	}
 
+	@Override
+	public int 
+	getTorrentType()
+	{
+		if ( torrent_type == 0 ){
+			
+			Debug.out( "Torrent type not set!" );
+			
+			return( TT_V1 );
+		}
+		
+		return( torrent_type );
+	}
+	
+	protected void
+	setTorrentType(
+		int		type )
+	{
+		torrent_type		= type;
+	}
+	
+	@Override
+	public boolean
+	isExportable()
+	{
+		if ( torrent_type == TT_V2 ){
+			
+			return( getAdditionalMapProperty( TOTorrentImpl.TK_V2_PIECE_LAYERS ) != null );
+		}
+		
+		return( true );
+	}
+	
+	@Override
+	public boolean
+	updateExportability(
+		TOTorrent		from )
+	{
+		if ( torrent_type == TT_V2 ){
+			
+			if ( isExportable()){
+				
+				return( true );
+			}
+			
+			try{
+				if ( !Arrays.equals( getHash(), from.getHash())){
+					
+					return( false );
+				}
+				
+				Map piece_layers = (Map)from.getAdditionalMapProperty( TOTorrentImpl.TK_V2_PIECE_LAYERS );
+				
+				if ( piece_layers == null ){
+					
+					return( false );
+				}
+				
+				setAdditionalMapProperty( TOTorrentImpl.TK_V2_PIECE_LAYERS, piece_layers );
+				
+				return( true );
+				
+			}catch( Throwable e ){
+				
+				Debug.out( e );
+				
+				return( false );
+			}
+		}else{
+			
+			return( true );
+		}
+	}
+	
 	@Override
 	public byte[]
 	getName()
@@ -656,6 +841,25 @@ TOTorrentImpl
 
 		return( torrent_hash );
 	}
+	
+	@Override
+	public byte[]
+	getFullHash(
+		int		type )
+
+		throws TOTorrentException
+	{
+		if ( torrent_hash == null ){
+
+			Map	root = serialiseToMap();
+
+			Map info = (Map)root.get( TK_INFO );
+
+			setHashFromInfo( info );
+		}
+
+		return( type==TT_V1?torrent_hash_v1:torrent_hash_v2 );
+	}
 
 	@Override
 	public HashWrapper
@@ -670,7 +874,7 @@ TOTorrentImpl
 
 		return( torrent_hash_wrapper );
 	}
-
+	
 	@Override
 	public boolean
 	hasSameHashAs(
@@ -698,10 +902,73 @@ TOTorrentImpl
 		try{
 			if ( torrent_hash_override == null ){
 
-				SHA1Hasher s = new SHA1Hasher();
-
-				torrent_hash = s.calculateHash(BEncoder.encode(info));
+				if ( torrent_type == 0 ){
+					
+					Debug.out( "Torrent type unknown" );
+				}
 				
+				byte[] encoded = BEncoder.encode(info);
+							
+				
+				if ( torrent_type == TT_V1_V2 ){
+					
+					Map private_props = getAdditionalMapProperty( AZUREUS_PRIVATE_PROPERTIES );
+					
+					boolean hybrid_hash_is_v2 = false;
+					
+					if ( private_props != null ){
+						
+						Number	l = (Long)private_props.get( TorrentUtils.TORRENT_AZ_PROP_HYBRID_HASH_V2 );
+						
+						if ( l != null ){
+							
+							hybrid_hash_is_v2 = ((Number)l).longValue() == 1;
+						}
+					}
+					
+					SHA1Hasher s = new SHA1Hasher();
+					
+					torrent_hash_v1 		= s.calculateHash( encoded );
+					torrent_hash_wrapper_v1 = new HashWrapper( torrent_hash_v1 );
+
+					MessageDigest sha256 = MessageDigest.getInstance( "SHA-256" );
+					
+					torrent_hash_v2 		= sha256.digest( encoded );
+					torrent_hash_wrapper_v2 = new HashWrapper( torrent_hash_v2 );
+					
+					if ( hybrid_hash_is_v2 ){
+												
+						torrent_hash = new byte[20];
+						
+						System.arraycopy( torrent_hash_v2, 0, torrent_hash, 0, 20 );
+						
+					}else{
+
+						torrent_hash = torrent_hash_v1;
+					}
+					
+				}else if ( torrent_type == TT_V2 ){
+					
+					MessageDigest sha256 = MessageDigest.getInstance( "SHA-256" );
+					
+					torrent_hash_v2 = sha256.digest( encoded );
+					
+					torrent_hash_wrapper_v2 = new HashWrapper( torrent_hash_v2 );
+					
+					torrent_hash = new byte[20];
+					
+					System.arraycopy( torrent_hash_v2, 0, torrent_hash, 0, 20 );
+					
+				}else{
+					
+					SHA1Hasher s = new SHA1Hasher();
+	
+					torrent_hash = s.calculateHash( encoded );
+					
+					torrent_hash_v1 		= torrent_hash;
+					torrent_hash_wrapper_v1 = new HashWrapper( torrent_hash_v1 );
+
+				}
 			}else{
 
 				torrent_hash = torrent_hash_override;
@@ -716,6 +983,53 @@ TOTorrentImpl
 		}
 	}
 
+	@Override
+	public TOTorrent 
+	selectHybridHashType(
+		int		type )
+		
+		throws TOTorrentException
+	{
+		if ( torrent_type != TT_V1_V2 ){
+		
+			throw( new TOTorrentException( 	"Torrent isn't hybrid", TOTorrentException.RT_CREATE_FAILED ));
+		}
+		
+		TOTorrent clone = TOTorrentFactory.deserialiseFromBEncodedByteArray( serialiseToByteArray());
+
+		TorrentUtils.clearTorrentFileName( clone );
+		
+		Map<String,Object> private_props = (Map<String,Object>)clone.getAdditionalMapProperty( AZUREUS_PRIVATE_PROPERTIES );
+
+		if ( type == TT_V1 ){
+			
+			if ( private_props != null ){
+				
+				private_props.remove( TorrentUtils.TORRENT_AZ_PROP_HYBRID_HASH_V2 );
+			}
+		}else if ( type == TT_V2 ){
+			
+			if ( private_props == null ){
+				
+				private_props = new HashMap<>();
+			}
+			
+			private_props.put( TorrentUtils.TORRENT_AZ_PROP_HYBRID_HASH_V2, 1L );
+			
+			clone.setAdditionalMapProperty( AZUREUS_PRIVATE_PROPERTIES, private_props );
+		}
+		
+		try{
+				// recreate with new properties so result represents this
+			
+			return( TOTorrentFactory.deserialiseFromBEncodedByteArray( BEncoder.encode( clone.serialiseToMap())));
+			
+		}catch( Throwable e ){
+			
+			throw( new TOTorrentException( "Encode failed", TOTorrentException.RT_CREATE_FAILED, e ));
+		}
+	}
+	
 	@Override
 	public void
 	setHashOverride(
@@ -756,7 +1070,21 @@ TOTorrentImpl
 
 		throws TOTorrentException
 	{
-		additional_info_properties.put( TK_PRIVATE, new Long(_private_torrent?1:0));
+		Long existing = (Long)additional_info_properties.get( TK_PRIVATE );
+		
+		if ( existing == null && !_private_torrent ){
+			
+			return;
+		}
+	
+		Long l_private = new Long(_private_torrent?1:0);
+
+		if ( existing != null && existing.equals( l_private )){
+			
+			return;
+		}
+		
+		additional_info_properties.put( TK_PRIVATE, l_private );
 
 			// update torrent hash
 
@@ -853,6 +1181,14 @@ TOTorrentImpl
 		pieces = _pieces;
 	}
 
+	protected void
+	setPiece(
+		int		index,
+		byte[]	piece )
+	{
+		pieces[index] = piece;
+	}
+	
 	@Override
 	public int
 	getFileCount()
@@ -861,7 +1197,7 @@ TOTorrentImpl
 	}
 
 	@Override
-	public TOTorrentFile[]
+	public TOTorrentFileImpl[]
 	getFiles()
 	{
 		return( files );
@@ -1337,4 +1673,4 @@ TOTorrentImpl
 
 		return null;
 	}
-}
\ No newline at end of file
+}
