@@ -21,6 +21,7 @@
 
 package com.biglybt.core.torrentdownloader.impl;
 
+import javax.net.ssl.*;
 import java.io.*;
 import java.net.*;
 import java.util.*;
@@ -30,8 +31,6 @@ import java.util.regex.Pattern;
 import java.util.zip.GZIPInputStream;
 import java.util.zip.InflaterInputStream;
 
-import javax.net.ssl.*;
-
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.internat.MessageText;
 import com.biglybt.core.proxy.AEProxyFactory;
@@ -41,13 +40,12 @@ import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrentdownloader.TorrentDownloader;
 import com.biglybt.core.torrentdownloader.TorrentDownloaderCallBackInterface;
 import com.biglybt.core.util.*;
-import com.biglybt.core.util.protocol.magnet.MagnetConnection;
-import com.biglybt.core.util.protocol.magnet.MagnetConnection2;
 import com.biglybt.core.vuzefile.VuzeFileHandler;
-import com.biglybt.pif.clientid.ClientIDGenerator;
 import com.biglybt.pifimpl.local.clientid.ClientIDManagerImpl;
 import com.biglybt.pifimpl.local.utils.xml.rss.RSSUtils;
 
+import com.biglybt.pif.clientid.ClientIDGenerator;
+
 /**
  * @author Tobias Minich
  */
@@ -582,27 +580,12 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 
         	String	query = tmp.toUpperCase();
 
-    		int	pos = query.indexOf( "XT=URN:SHA1:");
-
-    		if ( pos == -1 ){
-
-    	   		pos = query.indexOf( "XT=URN:BTIH:");
-    		}
-
-    		if ( pos != -1 ){
-
-    			pos += 12;
-
-    			int	p2 = query.indexOf( "&", pos );
-
-    			if ( p2 == -1 ){
-
-    				filename = query.substring(pos);
-
-    			}else{
-
-    				filename = query.substring(pos,p2);
-    			}
+        	byte[] hash = UrlUtils.getTruncatedHashFromMagnetURI( query );
+    		
+        	if ( hash != null ){
+    			
+        		filename = Base32.encode( hash );
+    			
         	}else{
 
         		filename = "Torrent" + (long)(Math.random()*Long.MAX_VALUE);
@@ -651,7 +634,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 
         	// this code removes any parent directories from the filename we've extracted
 
-        File temp = new File(filename);
+        File temp = FileUtil.newFile(filename);
         filename = temp.getName();
       }
 
@@ -662,7 +645,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 
       if (file_str != null) {
       	// not completely sure about the whole logic in this block
-        File temp = new File(file_str);
+        File temp = FileUtil.newFile(file_str);
 
         //if we're not using a default torrent save dir
         if (!useTorrentSave || directoryname.length() == 0) {
@@ -904,7 +887,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 
 		    if ( this.state != STATE_ERROR ){
 
-		    	this.file = new File(this.directoryname, filename);
+		    	this.file = FileUtil.newFile(this.directoryname, filename);
 
 		    	boolean useTempFile = file.exists();
 		    	if (!useTempFile) {
@@ -917,12 +900,12 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 		    	}
 
 		    	if (useTempFile) {
-		    		this.file = File.createTempFile("AZU", ".tmp", new File(
+		    		this.file = File.createTempFile("AZU", ".tmp", FileUtil.newFile(
 								this.directoryname));
 		    		this.file.createNewFile();
 		    	}
 
-		        fileout = new FileOutputStream(this.file, false);
+		        fileout = FileUtil.newFileOutputStream(this.file);
 
 		        bufBytes = 0;
 
@@ -985,12 +968,12 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 		        		  
 		        		  for ( int i=0;i<16;i++){
 		        			  
-			        		  File	new_file	= new File( directoryname, prefix + name );
+			        		  File	new_file	= FileUtil.newFile( directoryname, prefix + name );
 	
 			        		  if ( file.renameTo( new_file )){
-	
-			        			  new File( file.getParentFile(), file.getName() + ".bak" ).delete();
-			        			  
+
+											FileUtil.newFile( file.getParentFile(), file.getName() + ".bak" ).delete();
+
 			        			  filename	= name;
 	
 			        			  file	= new_file;
@@ -1018,12 +1001,12 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
 
 		        				  String	name = VuzeFileHandler.getVuzeFileName( filename );
 
-		        				  File	new_file	= new File( directoryname, name );
+		        				  File	new_file	= FileUtil.newFile( directoryname, name );
 
 		        				  if ( file.renameTo( new_file )){
 
-		        					  new File( file.getParentFile(), file.getName() + ".bak" ).delete();
-		        					  
+												FileUtil.newFile( file.getParentFile(), file.getName() + ".bak" ).delete();
+
 		        					  filename	= name;
 
 		        					  file	= new_file;
@@ -1165,7 +1148,7 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
   @Override
   public java.io.File getFile() {
     if ((!this.isAlive()) || (this.file == null))
-      this.file = new File(this.directoryname, filename);
+      this.file = FileUtil.newFile(this.directoryname, filename);
     return this.file;
   }
 
@@ -1194,19 +1177,11 @@ public class TorrentDownloaderImpl extends AEThread implements TorrentDownloader
     closeConnection();
   }
 
-  protected void
-  closeConnection()
-  {
-	if ( con instanceof MagnetConnection ){
-	  	((MagnetConnection)con).disconnect();
-
-	} else if ( con instanceof MagnetConnection2 ){
-	  	((MagnetConnection2)con).disconnect();
-
-	} else if (con instanceof HttpURLConnection) {
-		((HttpURLConnection)con).disconnect();
+	protected void closeConnection() {
+		if (con instanceof HttpURLConnection) {
+			((HttpURLConnection) con).disconnect();
+		}
 	}
-  }
 
   @Override
   public void setDownloadPath(String path, String file) {
