@@ -916,6 +916,7 @@ TRTrackerAnnouncerMuxer
 	protected void
 	informResponse(
 		TRTrackerAnnouncerHelper		helper,
+		TRTrackerAnnouncerRequest		request,
 		TRTrackerAnnouncerResponse		response )
 	{
 		URL	url = response.getURL();
@@ -932,7 +933,7 @@ TRTrackerAnnouncerMuxer
 
 				if ( summary != null ){
 
-					summary.updateFrom( response );
+					summary.updateFrom( request, response );
 				}
 			}
 		}
@@ -943,7 +944,7 @@ TRTrackerAnnouncerMuxer
 
 		last_best_active_set_time = 0;
 
-		super.informResponse( helper, response );
+		super.informResponse( helper, request, response );
 
 		if ( response.getStatus() != TRTrackerAnnouncerResponse.ST_ONLINE ){
 
@@ -1485,6 +1486,13 @@ TRTrackerAnnouncerMuxer
 					return( url_strs[0] );
 				}
 
+				@Override
+				public String
+				getDetails()
+				{
+					return( null);
+				}
+
 				@Override
 				public URL
 				getURL()
@@ -1693,6 +1701,20 @@ TRTrackerAnnouncerMuxer
 					}
 				}
 
+				@Override
+				public long[]
+				getReportedStats()
+				{
+					StatusSummary summary = fixup();
+
+					if ( summary != null && enabled ){
+
+						return( summary.getReportedStats());
+					}
+
+					return( null );
+				}
+				
 				@Override
 				public boolean
 				canDelete()
@@ -1737,6 +1759,9 @@ TRTrackerAnnouncerMuxer
 		private int			interval;
 		private int			min_interval;
 
+		private long		reported_sent;
+		private long		reported_received;
+		
 		protected
 		StatusSummary(
 			TRTrackerAnnouncerHelper		_helper,
@@ -1757,6 +1782,7 @@ TRTrackerAnnouncerMuxer
 
 		protected void
 		updateFrom(
+			TRTrackerAnnouncerRequest		request,
 			TRTrackerAnnouncerResponse		response )
 		{
 			time	= SystemTime.getMonotonousTime();
@@ -1771,7 +1797,10 @@ TRTrackerAnnouncerMuxer
 				leechers	= response.getScrapeIncompleteCount();
 				completed	= response.getScrapeDownloadedCount();
 				peers		= response.getPeers().length;
-
+					
+				reported_sent		= request.getReportedUpload();
+				reported_received 	= request.getReportedDownload();
+				
 			}else{
 
 				status = TrackerPeerSource.ST_ERROR;
@@ -1866,5 +1895,11 @@ TRTrackerAnnouncerMuxer
 		{
 			helper.update( true );
 		}
+		
+		public long[]
+		getReportedStats()
+		{
+			return( new long[]{ reported_sent, reported_received });
+		}
 	}
 }
