@@ -70,6 +70,28 @@ TRHostExternalTorrent
 		}
 	}
 
+	@Override
+	public int 
+	getTorrentType()
+	{
+		return( TT_V1 );
+	}
+	
+	@Override
+	public boolean 
+	isExportable()
+	{
+		return( true );
+	}
+	
+	@Override
+	public boolean
+	updateExportability(
+		TOTorrent		from )
+	{
+		return( true );
+	}
+	
 	@Override
 	public byte[]
 	getName()
@@ -78,11 +100,11 @@ TRHostExternalTorrent
 	}
 
 	@Override
-	public String getUTF8Name() {
+	public String 
+	getUTF8Name() {
 		return null;
 	}
 
-
 	@Override
 	public boolean
 	isSimpleTorrent()
@@ -128,7 +150,7 @@ TRHostExternalTorrent
 	}
 
  	@Override
-  public void
+ 	public void
 	setCreatedBy(
 		byte[]		cb )
    	{
@@ -283,6 +305,23 @@ TRHostExternalTorrent
 		return( hash );
 	}
 
+	@Override
+	public byte[]
+	getFullHash(
+		int	type ) 
+	
+		throws TOTorrentException
+	{
+		if ( type == TT_V1 ){
+			
+			return( hash );
+			
+		}else{
+			
+			return( null );
+		}
+	}
+	
 	@Override
 	public HashWrapper
 	getHashWrapper()
@@ -291,8 +330,18 @@ TRHostExternalTorrent
 	{
 		return( hash_wrapper );
 	}
-
-   	@Override
+	
+	@Override
+	public TOTorrent 
+	selectHybridHashType(
+		int type ) 
+		
+		throws TOTorrentException
+	{
+		throw( new TOTorrentException( "Not supported", TOTorrentException.RT_CREATE_FAILED ));
+	}
+	
+	@Override
     public void
 	setHashOverride(
 		byte[] hash )
