@@ -350,7 +350,7 @@ AEDiagnosticsLogger
 
 			if ( current_writer == null ){
 
-				FileOutputStream os = new FileOutputStream(log_file, true);
+				FileOutputStream os = FileUtil.newFileOutputStream(log_file, true);
 				if (log_file.length() == 0) {
 					// UTF-8 BOM
 					os.write(new byte[] { (byte) 239, (byte) 187, (byte) 191 });
@@ -416,7 +416,7 @@ AEDiagnosticsLogger
 
 				if ( current_writer == null ){
 
-					current_writer = new PrintWriter( new OutputStreamWriter( new FileOutputStream( log_file, true ), "UTF-8" ));
+					current_writer = new PrintWriter( new OutputStreamWriter( FileUtil.newFileOutputStream( log_file, true ), "UTF-8" ));
 				}
 
 				for ( StringBuilder str: pending ){
@@ -452,7 +452,7 @@ AEDiagnosticsLogger
 	public File
 	getLogFile()
 	{
-		return( new File( debug_dir, getName() + "_" + (first_file?"1":"2") + ".log" ));
+		return( FileUtil.newFile( debug_dir, getName() + "_" + (first_file?"1":"2") + ".log" ));
 	}
 
 	private static String
