@@ -65,8 +65,8 @@ Constants
 	public static final String NAT_TEST_SERVER_HTTP		= "http://nettest.biglybt.com/";
 	public static final String NAT_TEST_SERVER_V6_HTTP	= "http://nettest.biglybt.com/";	// nettest.biglybt.com loadbalancer both v4+v6
 
-	public static final String PAIRING_SERVER			= "pair.vuze.com";
-	public static final String WEB_REMOTE_SERVER		= "remote.vuze.com";
+	public static final String PAIRING_SERVER			= "pair.biglybt.com";
+	public static final String WEB_REMOTE_SERVER		= "remote.biglybt.com";
 
 	public static final String SPEED_TEST_SERVER	= "speedtest.vuze.com";
 
@@ -150,11 +150,11 @@ Constants
 	public static final String BIGLY_PEER_ID						= "BI";
 
 	
-	public static final String BIGLYBT_VERSION  = "2.4.0.0";
+	public static final String BIGLYBT_VERSION  = "2.6.0.0";
 	
 	//public static final String BUILD_VERSION  = "@build.version@";   //Ant replace - believed dead
 	public static final String SUBVERSION		= "";
-	public static final byte[] VERSION_ID       = ("-" + BIGLY_PEER_ID + "2400" + "-").getBytes();  //MUST be 8 chars long!
+	public static final byte[] VERSION_ID       = ("-" + BIGLY_PEER_ID + "2600" + "-").getBytes();  //MUST be 8 chars long!
 
 	private static final boolean FORCE_NON_CVS = System.getProperty( "az.force.noncvs", "0" ).equals( "1" );
 
@@ -350,13 +350,13 @@ Constants
 				|| System.getProperty("java.vendor", "").contains("Android");
 	}
 
-	// Android is roughly 1.6 (reports as 0 for java.version)
+	// Android is roughly 1.8 (reports as 0 for java.version)
 
 	public static final String	JAVA_VERSION;
 	public static final int		API_LEVEL;
 
 	static{
-		String java_version 	= isAndroid?"1.6":System.getProperty("java.version");
+		String java_version 	= isAndroid?"1.8":System.getProperty("java.version");
 		int	 api_level 		= 0;
 
 		if ( isAndroid ){
@@ -366,10 +366,6 @@ Constants
 			try{
 				api_level = Integer.parseInt( sdk_int );
 
-				if ( api_level > 0  && api_level <= 8 ){
-
-					java_version = "1.5";
-				}
 			}catch( Throwable e ){
 			}
 		}
