@@ -797,28 +797,51 @@ DisplayFormatters
 		String	tmp = "";
 
 		switch (state) {
-			case DownloadManager.STATE_QUEUED:
-				tmp = ManagerItem_queued;
-				break;
+			case DownloadManager.STATE_QUEUED:{
+				
+				long[]	mp = manager.getMoveProgress();
+				
+				if ( mp != null ){
+					
+					 tmp = ManagerItem_moving + ": "	+ formatPercentFromThousands( (int)mp[0] );
+					 
+				}else{	 
 
-			case DownloadManager.STATE_DOWNLOADING:
-				tmp = ManagerItem_downloading;
-				if ( manager.isSwarmMerging()){
-					tmp += " + " + ManagerItem_swarmMerge;
+					tmp = ManagerItem_queued;
+				}
+				
+				break;
+			}
+			case DownloadManager.STATE_DOWNLOADING:{
+				
+				long[]	mp = manager.getMoveProgress();
+				
+				if ( mp != null ){
+					
+					 tmp = ManagerItem_moving + ": "	+ formatPercentFromThousands( (int)mp[0] );
+					 
+				}else{
+					
+					tmp = ManagerItem_downloading;
+					
+					if ( manager.isSwarmMerging()){
+						
+						tmp += " + " + ManagerItem_swarmMerge;
+					}
 				}
 				break;
-
+			}
 			case DownloadManager.STATE_SEEDING:{
 
 				DiskManager diskManager = manager.getDiskManager();
 
 				if ( diskManager != null ){
 
-					int	mp = diskManager.getMoveProgress();
+					long[]	mp = diskManager.getMoveProgress();
 
-					if ( mp != -1 ){
+					if ( mp != null ){
 
-						tmp = ManagerItem_moving + ": "	+ formatPercentFromThousands( mp );
+						tmp = ManagerItem_moving + ": "	+ formatPercentFromThousands( (int)mp[0] );
 
 					}else{
 						int done = diskManager.getCompleteRecheckStatus();
@@ -847,11 +870,11 @@ DisplayFormatters
 			}
 			case DownloadManager.STATE_STOPPED:
 				
-				int	mp = manager.getMoveProgress();
+				long[]	mp = manager.getMoveProgress();
 				
-				if ( mp != -1 ){
+				if ( mp != null ){
 					
-					 tmp = ManagerItem_moving + ": "	+ formatPercentFromThousands( mp );
+					 tmp = ManagerItem_moving + ": "	+ formatPercentFromThousands( (int)mp[0] );
 					 
 				}else{
 					
