@@ -27,7 +27,6 @@ import java.util.*;
 import java.util.regex.Pattern;
 
 import com.biglybt.core.security.SESecurityManager;
-import com.biglybt.core.security.impl.SESecurityManagerImpl;
 import com.biglybt.pifimpl.local.utils.UtilitiesImpl.runnableWithException;
 
 public class
@@ -382,31 +381,65 @@ NetUtils
 
 							if ( ifs != null ){
 								
+								NetworkInterface 	best 			= null;
+								boolean				best_has_ias 	= false;
+								
 								for ( NetworkInterface x: ifs ){
 									
 									String dn = x.getDisplayName();
 									
 									if ( dn != null ){
-																			
+																						
 										if ( p.matcher( dn ).find()){
 											
-											if ( result == null ){
+											boolean has_ias = false;
 											
-												result = x;
+											Enumeration<InetAddress> ias = x.getInetAddresses();
+
+											while( ias.hasMoreElements()){
+												InetAddress ia = ias.nextElement();
+												if ( ia instanceof Inet4Address ){
+													has_ias = true;
+													break;
+												}else{
+													if ( AddressUtils.isGlobalAddressV6(ia)){
+														has_ias = true;
+														break;
+													}
+												}
+											}
+											
+											if ( best == null ){
+											
+												best 			= x;
+												best_has_ias	= has_ias;
 												
 											}else{
 												
-												result = null;
+												if ( has_ias ){
+																									
+													if ( best_has_ias ){
 												
-												Debug.out( "Multiple network interface matches for regex " + expr );
-												
-												failed = true;
-												
-												break;
+														best = null;
+														
+														Debug.out( "Multiple network interface matches for regex " + expr );
+														
+														failed = true;
+														
+														break;
+														
+													}else{
+													
+														best 			= x;
+														best_has_ias	= has_ias;
+													}
+												}
 											}
 										}
 									}
 								}
+								
+								result = best;
 							}
 						}catch( Throwable e ){
 							
