@@ -19,6 +19,7 @@
 
 package com.biglybt.core.util.jman;
 
+import java.io.File;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.lang.management.ManagementFactory;
@@ -37,6 +38,29 @@ public class
 AEThreadMonitor
 	implements ThreadStuff, AEDiagnosticsEvidenceGenerator 
 {
+	private static final boolean THREAD_USER_TIME;
+	
+	static{
+		String prop = System.getProperty( SystemProperties.SYSPROP_THREAD_MON_USERONLY, null );
+		
+		if ( prop == null ){
+			
+			if ( Constants.isLinux ){
+				
+					// prevents pthread crash on linux
+				
+				THREAD_USER_TIME = true;
+				
+			}else{
+				
+				THREAD_USER_TIME = false;
+			}
+		}else{
+			
+			THREAD_USER_TIME = !prop.equals( "0" );
+		}
+	}
+			
 	private final ThreadMXBean	thread_bean;
 
 	private final LinkedList<String>	memory_history	= new LinkedList<>();
@@ -63,7 +87,29 @@ AEThreadMonitor
 			return( 0 );
 		}
 
-		return( thread_bean.getCurrentThreadCpuTime());
+		if ( THREAD_USER_TIME ){
+			
+			return( thread_bean.getCurrentThreadUserTime());
+			
+		}else{
+		
+			return( thread_bean.getCurrentThreadCpuTime());
+		}
+	}
+	
+	private long
+	getThreadCpuTime(
+		ThreadMXBean		bean,
+		long				thread_id )
+	{
+		if ( THREAD_USER_TIME ){
+			
+			return( bean.getThreadUserTime( thread_id ));
+			
+		}else{
+			
+			return( bean.getThreadCpuTime( thread_id ));
+		}
 	}
 	
 	@Override
@@ -180,7 +226,7 @@ AEThreadMonitor
 
 				long	id = ids[i];
 
-				long	time = bean.getThreadCpuTime( id )/1000000;	// nanos -> millis
+				long	time = getThreadCpuTime( bean, id )/1000000;	// nanos -> millis
 
 				Long	old_time = last_times.get( id );
 
@@ -288,7 +334,13 @@ AEThreadMonitor
 
 		iw.close();
 
-		Debug.out( sw.toString());
+		String dump = sw.toString();
+		
+		Debug.out( dump );
+		
+		File logDir = AEDiagnostics.getLogDir();
+		
+		FileUtil.writeStringAsFile( FileUtil.newFile( logDir, "ThreadDump.log" ), dump );
 	}
 
 	private void
@@ -312,8 +364,8 @@ AEThreadMonitor
 			@Override
 			public int compare(ThreadInfo o1, ThreadInfo o2) {
 
-				long diff = threadBean.getThreadCpuTime(o2.getThreadId())
-						- threadBean.getThreadCpuTime(o1.getThreadId());
+				long diff = getThreadCpuTime( threadBean, o2.getThreadId())
+						- getThreadCpuTime(threadBean, o1.getThreadId());
 				if (diff == 0) {
 					return o1.getThreadName().compareToIgnoreCase(o2.getThreadName());
 				}
@@ -325,7 +377,7 @@ AEThreadMonitor
 			try {
 				ThreadInfo threadInfo = threadInfos.get(i);
 
-				long lCpuTime = threadBean.getThreadCpuTime(threadInfo.getThreadId());
+				long lCpuTime = getThreadCpuTime(threadBean,threadInfo.getThreadId());
 				if (lCpuTime == 0)
 					break;
 
