@@ -852,33 +852,33 @@ DownloadImpl
 	}
 
  	@Override
-  public boolean
+ 	public boolean
 	isComplete()
  	{
  		return download_manager.isDownloadComplete(false);
  	}
 
  	@Override
-  public boolean isComplete(boolean bIncludeDND) {
+ 	public boolean isComplete(boolean bIncludeDND) {
  		return download_manager.isDownloadComplete(bIncludeDND);
  	}
 
  	@Override
-  public boolean
+ 	public boolean
  	isChecking()
  	{
  		return( download_stats.getCheckingDoneInThousandNotation() != -1 );
  	}
 
  	@Override
-  public boolean
+ 	public boolean
  	isMoving()
  	{
  		com.biglybt.core.disk.DiskManager dm = download_manager.getDiskManager();
 
  		if ( dm != null ){
 
- 			return( dm.getMoveProgress() != -1 );
+ 			return( dm.getMoveProgress() != null );
  		}
 
  		return( false );
@@ -1146,6 +1146,9 @@ DownloadImpl
 
 			long	mins = SystemTime.getCurrentTime()/(1000*60);
 
+			int	old_seeds 		= -2;
+			int old_leechers 	= -2;
+			
 			if ( cache != null ){
 
 				String[]	bits = cache.split(",");
@@ -1155,7 +1158,9 @@ DownloadImpl
 					long	updated_mins	= 0;
 
 					try{
-						updated_mins = Long.parseLong( bits[0] );
+						updated_mins 	= Long.parseLong( bits[0] );
+						old_seeds 		= Integer.parseInt( bits[1] );
+						old_leechers 	= Integer.parseInt( bits[2] );
 
 					}catch( Throwable e ){
 
@@ -1163,17 +1168,35 @@ DownloadImpl
 
 					if ( mins - updated_mins < 15 ){
 
-
+							// too recent
+						
 						do_update = false;
+						
+					}else{
+						
+						int dl_state = download_manager.getState();
+						
+						if ( dl_state != DownloadManager.STATE_DOWNLOADING && dl_state != DownloadManager.STATE_SEEDING ){
+
+								// not running, who cares
+							
+							do_update = false;
+						}
 					}
 				}
 			}
 
 			if ( do_update ){
 
-				String str = mins + "," + result.getSeedCount() + "," + result.getNonSeedCount();
-
-				download_manager.getDownloadState().setAttribute( DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE, str  );
+				int	new_seeds 		= result.getSeedCount();
+				int new_leechers	= result.getNonSeedCount();
+				
+				if ( new_seeds != old_seeds || new_leechers != old_leechers ){
+					
+					String str = mins + "," + new_seeds + "," + new_leechers;
+	
+					download_manager.getDownloadState().setAttribute( DownloadManagerState.AT_AGGREGATE_SCRAPE_CACHE, str  );
+				}
 			}
 		}
 
@@ -2531,9 +2554,9 @@ DownloadImpl
 		 if (!has_change) {return;}
 
 		 // Test that one of the locations is actually different.
-		 has_change = slc.isDifferentDownloadLocation(new File(this.getSavePath()));
+		 has_change = slc.isDifferentDownloadLocation(FileUtil.newFile(this.getSavePath()));
 		 if (!has_change) {
-			 has_change = slc.isDifferentTorrentLocation(new File(this.getTorrentFileName()));
+			 has_change = slc.isDifferentTorrentLocation(FileUtil.newFile(this.getTorrentFileName()));
 		 }
 
 		 if (!has_change) {return;}
@@ -2669,4 +2692,4 @@ DownloadImpl
 			return( r == null?null:r.getURL());
 		}
 	 }
-}
\ No newline at end of file
+}
