@@ -385,7 +385,7 @@ DownloadManagerImpl
 		    	throw( new DownloadException("DownloadManager::addDownload: default torrent save directory must be configured" ));
 		    }
 
-		    torrent_file = new File( torrent_dir + File.separator + torrent.getName() + ".torrent" );
+		    torrent_file = FileUtil.newFile( torrent_dir, FileUtil.convertOSSpecificChars( torrent.getName(), false ) + ".torrent" );
 
 		    try{
 		    	torrent.writeToFile( torrent_file );
@@ -414,7 +414,7 @@ DownloadManagerImpl
 		    	throw( new DownloadException("DownloadManager::addDownload: default data save directory must be configured" ));
 		    }
 
-		    data_location = new File(data_dir);
+		    data_location = FileUtil.newFile(data_dir);
 
 		    FileUtil.mkdirs(data_location);
 		}
@@ -747,6 +747,20 @@ DownloadManagerImpl
 
 						return( getDownload( torrent ));
 					}
+					
+						// support looking up by truncated v2 hash so that magnet downloads for hybrid torrents using just
+						// the btmh urn at least find content in our library
+					
+					byte[] v2_trunc = torrent.getTruncatedHash( TOTorrent.TT_V2 );
+					
+					if ( v2_trunc != null ){
+												
+						if ( Arrays.equals( v2_trunc, hash )){
+
+							return( getDownload( torrent ));
+						}
+					}
+					
 				}catch( DownloadException e ){
 
 						// not found
@@ -1354,8 +1368,19 @@ DownloadManagerImpl
 		try{
 			core_dm.getDownloadState().exportState( ARCHIVE_DIR );
 
+			String old_save_path = stub.getSavePath();
+			
 			download.remove( false, false );
 
+				// might have a 'move on remove' action setup
+			
+			String save_path_after_remove = download.getSavePath();
+			
+			if ( !old_save_path.contentEquals( save_path_after_remove )){
+				
+				stub.setSavePath( save_path_after_remove );
+			}
+			
 			synchronized( download_stubs ){
 
 				download_stubs.add( stub );
@@ -1527,7 +1552,7 @@ DownloadManagerImpl
 	getStubTorrent(
 		byte[]		hash  )
 	{
-		File torrent_file = new File( ARCHIVE_DIR, ByteFormatter.encodeString( hash ) + ".dat" );
+		File torrent_file = FileUtil.newFile( ARCHIVE_DIR, ByteFormatter.encodeString( hash ) + ".dat" );
 
 		if ( torrent_file.exists()){
 
@@ -1546,7 +1571,7 @@ DownloadManagerImpl
 	getTorrent(
 		DownloadStubImpl		stub )
 	{
-		File torrent_file = new File( ARCHIVE_DIR, ByteFormatter.encodeString( stub.getTorrentHash()) + ".dat" );
+		File torrent_file = FileUtil.newFile( ARCHIVE_DIR, ByteFormatter.encodeString( stub.getTorrentHash()) + ".dat" );
 
 		if ( torrent_file.exists()){
 
