@@ -45,6 +45,7 @@ import com.biglybt.core.tag.TagType;
 import com.biglybt.core.torrent.TOTorrent;
 import com.biglybt.core.torrent.TOTorrentAnnounceURLGroup;
 import com.biglybt.core.torrent.TOTorrentAnnounceURLSet;
+import com.biglybt.core.torrent.TOTorrentException;
 import com.biglybt.core.torrent.TOTorrentFactory;
 import com.biglybt.core.torrent.impl.TorrentOpenOptions;
 import com.biglybt.core.tracker.client.TRTrackerAnnouncerFactory;
@@ -74,6 +75,7 @@ import com.biglybt.pif.ui.config.ConfigSection;
 import com.biglybt.pif.ui.config.IntParameter;
 import com.biglybt.pif.ui.config.Parameter;
 import com.biglybt.pif.ui.config.StringListParameter;
+import com.biglybt.pif.ui.config.StringParameter;
 import com.biglybt.pif.ui.menus.MenuItem;
 import com.biglybt.pif.ui.menus.MenuItemListener;
 import com.biglybt.pif.ui.model.BasicPluginConfigModel;
@@ -108,8 +110,9 @@ MagnetPlugin
 	private static final int	SECONDARY_LOOKUP_DELAY		= 20*1000;
 	private static final int	SECONDARY_LOOKUP_MAX_TIME	= 2*60*1000;
 
-	private static final int	MD_LOOKUP_DELAY_SECS_DEFAULT		= 20;
+	private static final int	MD_LOOKUP_DELAY_SECS_DEFAULT		= 0;
 
+	private static final String[] MD_EXTRA_TRACKERS = { "udp://tracker.opentrackr.org:1337/announce" }; 
 
 	private static final String	PLUGIN_NAME				= "Magnet URI Handler";
 	private static final String PLUGIN_CONFIGSECTION_ID = "plugins.magnetplugin";
@@ -136,10 +139,12 @@ MagnetPlugin
 	// private BooleanParameter 	secondary_lookup; removed
 	private BooleanParameter 	md_lookup;
 	private IntParameter	 	md_lookup_delay;
+	private StringParameter 	md_extra_trackers;
 	private IntParameter	 	timeout_param;
 	private StringListParameter	sources_param;
 	private IntParameter	 	sources_extra_param;
 	private BooleanParameter	magnet_recovery;
+	private IntParameter	 	magnet_recovery_concurrency;
 
 	private Map<String,BooleanParameter> net_params = new HashMap<>();
 
@@ -186,7 +191,18 @@ MagnetPlugin
 		md_lookup 			= config.addBooleanParameter2( "MagnetPlugin.use.md.download", "MagnetPlugin.use.md.download", true );
 		md_lookup_delay		= config.addIntParameter2( "MagnetPlugin.use.md.download.delay", "MagnetPlugin.use.md.download.delay", MD_LOOKUP_DELAY_SECS_DEFAULT );
 
+		String et_default = "";
+		
+		for ( String etd: MD_EXTRA_TRACKERS ){
+			et_default += (et_default.isEmpty()?"":"\n") + etd;
+		}
+		
+		md_extra_trackers = config.addStringParameter2( "MagnetPlugin.md.extra.trackers", "MagnetPlugin.md.extra.trackers", et_default);
+		
+		md_extra_trackers.setMultiLine( 3 );
+		
 		md_lookup.addEnabledOnSelection( md_lookup_delay );
+		md_lookup.addEnabledOnSelection( md_extra_trackers );
 
 		timeout_param		= config.addIntParameter2( "MagnetPlugin.timeout.secs", "MagnetPlugin.timeout.secs", PLUGIN_DOWNLOAD_TIMEOUT_SECS_DEFAULT );
 
@@ -196,6 +212,12 @@ MagnetPlugin
 				
 		magnet_recovery		= config.addBooleanParameter2( "MagnetPlugin.recover.magnets", "MagnetPlugin.recover.magnets", true );
 		
+		magnet_recovery_concurrency	= config.addIntParameter2( "MagnetPlugin.recover.magnets.conc", "MagnetPlugin.recover.magnets.conc", 32, 8, 512 );
+
+		magnet_recovery_concurrency.setIndent( 1, true );
+		
+		magnet_recovery.addEnabledOnSelection( magnet_recovery_concurrency );
+
 		BooleanParameter rename = config.addBooleanParameter2( "MagnetPlugin.rename.using.dn", "MagnetPlugin.rename.using.dn", false );
 		
 		BooleanParameter rename_ext = config.addBooleanParameter2( "MagnetPlugin.rename.using.dn.only.with.ext", "MagnetPlugin.rename.using.dn.only.with.ext", false );	
@@ -900,7 +922,13 @@ MagnetPlugin
 
 		if ( recover && !active.isEmpty()){
 		
-			ThreadPool tp = new ThreadPool( "Magnet Recovery", 16, true );
+			int conc = magnet_recovery_concurrency.getValue();
+			
+			if ( conc < 1 ){
+				conc = 1;
+			}
+			
+			ThreadPool tp = new ThreadPool( "Magnet Recovery", conc, true );
 			
 			for ( Map map: active.values()){
 					
@@ -988,7 +1016,7 @@ MagnetPlugin
 												
 												if ( dir.length() > 0 ){
 													
-													File f = new File( dir );
+													File f = FileUtil.newFile( dir );
 													
 													if ( !f.exists()){
 														
@@ -1008,11 +1036,11 @@ MagnetPlugin
 									    
 									    if ( dir != null ){
 									    	
-									    	torrent_file = new File( dir, torrent_name );
+									    	torrent_file = FileUtil.newFile( dir, torrent_name );
 									    	
 									    }else {
 									    	
-									    	torrent_file = new File( AETemporaryFileHandler.getTempDirectory(), torrent_name );
+									    	torrent_file = FileUtil.newFile( AETemporaryFileHandler.getTempDirectory(), torrent_name );
 									    }
 									    
 									    if ( torrent_file.exists()){
@@ -1277,35 +1305,60 @@ MagnetPlugin
 		return( net_params.get( net ).getValue());
 	}
 
-	public URL
-	getMagnetURL(
-		Download		d )
-	{
-		Torrent	torrent = d.getTorrent();
-
-		if ( torrent == null ){
-
-			return( null );
-		}
-
-		return( getMagnetURL( torrent.getHash()));
-	}
-
 	public URL
 	getMagnetURL(
 		byte[]		hash )
 	{
 		try{
-			return( new URL( "magnet:?xt=urn:btih:" + Base32.encode(hash)));
-
+			return( new URL( UrlUtils.getMagnetURI( hash )));
+			
 		}catch( Throwable e ){
-
-			Debug.printStackTrace(e);
-
+			
+			Debug.out( e );
+			
+			return( null );
+		}
+	}
+	
+	public URL
+	getMagnetURL(
+		TOTorrent		torrent  )
+	
+		throws TOTorrentException
+	{
+		try{
+			return( new URL(  UrlUtils.getMagnetURI( torrent )));
+			
+		}catch( Throwable e ){
+			
+			Debug.out( e );
+			
 			return( null );
 		}
 	}
 
+	public List<String>
+	getExtraTrackers()
+	{
+		String extra = md_extra_trackers.getValue();
+		
+		List<String> result = new ArrayList<>();
+		
+		String[] bits = extra.split( "\n" );
+		
+		for ( String bit: bits ){
+			
+			bit = bit.trim();
+			
+			if ( !bit.isEmpty()){
+				
+				result.add( bit );
+			}
+		}
+		
+		return( result );
+	}
+	
 	public byte[]
 	badge()
 	{
