@@ -80,6 +80,8 @@ NetStatusProtocolTesterBT
 	setBindIP(
 		InetAddress	a )
 	{
+		log( "Bind IP set to " + a );
+		
 		explicit_bind = a;
 	}
 	
@@ -549,7 +551,7 @@ NetStatusProtocolTesterBT
 						connectSuccess(
 							ByteBuffer remaining_initial_data )
 						{
-							log( type + " connect success", true );
+							log( type + " connect success, protocol=" + connection.getTransport().getProtocol(), true );
 
 							connected	= true;
 
@@ -575,7 +577,7 @@ NetStatusProtocolTesterBT
 						{
 							if ( !closing ){
 
-								logError( type + " connection fail", e );
+								logError( type + " connect failed: " + Debug.getNestedExceptionMessage( e ));
 							}
 
 							close();
@@ -588,7 +590,7 @@ NetStatusProtocolTesterBT
 						{
 							if ( !closing ){
 
-								logError( type + " connection fail", e );
+								logError( type + " connection failure", e );
 							}
 
 							close();
