@@ -19,6 +19,7 @@
 package com.biglybt.ui.common.table.impl;
 
 import java.util.*;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 
 import com.biglybt.core.config.impl.ConfigurationManager;
@@ -129,6 +130,8 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 	/** Queue removed datasources and add them on refresh */
 	private IdentityHashMap<DATASOURCETYPE, String> dataSourcesToRemove = new IdentityHashMap<>(4);
 
+	private AtomicInteger	datsaSourceQueueProcessingCount = new AtomicInteger();
+	
 	// class used to keep filter stuff in a nice readable parcel
 	public static class filter<DATASOURCETYPE>
 	{
@@ -645,23 +648,22 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 					}
 				}
 			}
-			
-			Set<DATASOURCETYPE> existing = new HashSet<>(
-					getDataSources());
+
+			Collection<DATASOURCETYPE> existing = getDataSources();
 			List<DATASOURCETYPE> listRemoves = new ArrayList<>();
 			List<DATASOURCETYPE> listAdds = new ArrayList<>();
 
-			for (int i = 0; i < unfilteredArray.length; i++) {
-				boolean bHave = existing.contains(unfilteredArray[i]);
-				boolean isOurs = filter.checker.filterCheck(unfilteredArray[i],
-						filter.text, filter.regex);
+			for (DATASOURCETYPE dst : unfilteredArray) {
+				boolean bHave = existing.contains(dst);
+				boolean isOurs = filter.checker.filterCheck(dst, filter.text,
+						filter.regex);
 				if (!isOurs) {
 					if (bHave) {
-						listRemoves.add(unfilteredArray[i]);
+						listRemoves.add(dst);
 					}
 				} else {
 					if (!bHave) {
-						listAdds.add(unfilteredArray[i]);
+						listAdds.add(dst);
 					}
 				}
 			}
@@ -714,51 +716,61 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 	}
 
 	private void _processDataSourceQueue() {
-		Object[] dataSourcesAdd = null;
-		Object[] dataSourcesRemove = null;
+		boolean hasAdd;
+		boolean hasRemove;
+		
+		try{
+			datsaSourceQueueProcessingCount.incrementAndGet();
 
-		synchronized (rows_sync) {
-			if (dataSourcesToAdd.size() > 0) {
-				boolean removed_something = false;
-				for ( DATASOURCETYPE ds: dataSourcesToRemove.keySet()){
+			Object[] dataSourcesAdd = null;
+			Object[] dataSourcesRemove = null;
 
-					if ( dataSourcesToAdd.remove( ds ) != null ){
-
-						removed_something = true;
+			synchronized (rows_sync) {
+				if (dataSourcesToAdd.size() > 0) {
+					boolean removed_something = false;
+					for ( DATASOURCETYPE ds: dataSourcesToRemove.keySet()){
+	
+						if ( dataSourcesToAdd.remove( ds ) != null ){
+	
+							removed_something = true;
+						}
 					}
+	
+					if ( removed_something&& DEBUGADDREMOVE){
+						debug("Saved time by not adding a row that was removed");
+					}
+	
+					dataSourcesAdd = dataSourcesToAdd.keySet().toArray();
+	
+					dataSourcesToAdd.clear();
 				}
-
-				if ( removed_something&& DEBUGADDREMOVE){
-					debug("Saved time by not adding a row that was removed");
+	
+				if (dataSourcesToRemove.size() > 0) {
+					dataSourcesRemove = dataSourcesToRemove.keySet().toArray();
+					if (DEBUGADDREMOVE && dataSourcesRemove.length > 1) {
+						debug("Streamlining removing " + dataSourcesRemove.length + " rows");
+					}
+					dataSourcesToRemove.clear();
 				}
-
-				dataSourcesAdd = dataSourcesToAdd.keySet().toArray();
-
-				dataSourcesToAdd.clear();
 			}
-
-			if (dataSourcesToRemove.size() > 0) {
-				dataSourcesRemove = dataSourcesToRemove.keySet().toArray();
-				if (DEBUGADDREMOVE && dataSourcesRemove.length > 1) {
-					debug("Streamlining removing " + dataSourcesRemove.length + " rows");
+	
+			hasAdd = dataSourcesAdd != null && dataSourcesAdd.length > 0;
+			if (hasAdd) {
+				reallyAddDataSources(dataSourcesAdd);
+				if (DEBUGADDREMOVE && dataSourcesAdd.length > 1) {
+					debug("Streamlined adding " + dataSourcesAdd.length + " rows");
 				}
-				dataSourcesToRemove.clear();
 			}
-		}
-
-		boolean hasAdd = dataSourcesAdd != null && dataSourcesAdd.length > 0;
-		if (hasAdd) {
-			reallyAddDataSources(dataSourcesAdd);
-			if (DEBUGADDREMOVE && dataSourcesAdd.length > 1) {
-				debug("Streamlined adding " + dataSourcesAdd.length + " rows");
+	
+			hasRemove = dataSourcesRemove != null && dataSourcesRemove.length > 0;
+			if (hasRemove) {
+				reallyRemoveDataSources(dataSourcesRemove);
 			}
+		}finally{
+			
+			datsaSourceQueueProcessingCount.decrementAndGet();
 		}
-
-		boolean hasRemove = dataSourcesRemove != null && dataSourcesRemove.length > 0;
-		if (hasRemove) {
-			reallyRemoveDataSources(dataSourcesRemove);
-		}
-
+		
 		if (hasAdd || hasRemove) {
 			tableMutated();
 		}
@@ -832,7 +844,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 		addDataSources(dataSources, false);
 	}
 
-	public void addDataSources(final DATASOURCETYPE dataSources[],
+	private void addDataSources(final DATASOURCETYPE dataSources[],
 			boolean skipFilterCheck) {
 
 		if (dataSources == null) {
@@ -910,8 +922,13 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 	{
 		synchronized (rows_sync){
 			
-			return( !dataSourcesToAdd.isEmpty() || !dataSourcesToRemove.isEmpty());
+			if ( !dataSourcesToAdd.isEmpty() || !dataSourcesToRemove.isEmpty()){
+				
+				return( true );
+			}
 		}
+		
+		return( datsaSourceQueueProcessingCount.get() > 0 );
 	}
 
 	// @see TableView#dataSourceExists(java.lang.Object)
@@ -1152,21 +1169,21 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 
 	// @see TableView#getDataSources()
 	@Override
-	public ArrayList<DATASOURCETYPE> getDataSources() {
+	public HashSet<DATASOURCETYPE> getDataSources() {
 		synchronized (rows_sync) {
-			return new ArrayList<>(mapDataSourceToRow.keySet());
+			return new HashSet<>(mapDataSourceToRow.keySet());
 		}
 	}
 
 	// @see TableView#getDataSources()
 	@Override
-	public ArrayList<DATASOURCETYPE> getDataSources(boolean include_filtered) {
+	public HashSet<DATASOURCETYPE> getDataSources(boolean include_filtered) {
 		synchronized (rows_sync) {
 			if ( include_filtered ){
-				return new ArrayList<>(listUnfilteredDataSources.keySet());
+				return new HashSet<>(listUnfilteredDataSources.keySet());
 
 			}else{
-				return new ArrayList<>(mapDataSourceToRow.keySet());
+				return new HashSet<>(mapDataSourceToRow.keySet());
 			}
 		}
 	}
@@ -1235,8 +1252,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 		}
 
 		synchronized (rows_sync) {
-			for (int i = 0; i < dataSources.length; i++) {
-				DATASOURCETYPE dataSource = dataSources[i];
+			for (DATASOURCETYPE dataSource : dataSources) {
 				dataSourcesToAdd.remove(dataSource); // override any pending addition
 				dataSourcesToRemove.put(dataSource, "");
 			}
@@ -1269,7 +1285,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 		}
 	}
 
-	public void reallyAddDataSources(final Object dataSources[]) {
+	private void reallyAddDataSources(final Object dataSources[]) {
 		// Note: We assume filterCheck has already run, and the list of dataSources
 		//       all passed the filter
 
@@ -1401,7 +1417,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 	}
 
 	@SuppressWarnings("null")
-	public void reallyRemoveDataSources(final Object[] dataSources) {
+	private void reallyRemoveDataSources(final Object[] dataSources) {
 		final long lStart = SystemTime.getCurrentTime();
 
 		int rows_removed = 0;
@@ -1542,7 +1558,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 		boolean	orderChanged = false;
 
 		synchronized (rows_sync) {
-			
+						
 			if (bForceDataRefresh && !sortColumns.isEmpty()) {
 				for (TableColumnCore sortColumn : sortColumns) {
 					String sColumnID = sortColumn.getName();
@@ -2278,7 +2294,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 						//same
 					}
 
-					TableColumnManager.getInstance().setDefaultSortColumnNames(tableID, sortColumnNames, true );
+					TableColumnManager.getInstance().setSortColumnNames(tableID, sortColumnNames );
 				} else {
 					TableColumnCore sortColumn = sortColumns.get(0);
 					sortColumn.setSortAscending(!sortColumn.isSortAscending());
@@ -2286,7 +2302,7 @@ public abstract class TableViewImpl<DATASOURCETYPE>
 			} else {
 				sortColumns.clear();
 				sortColumns.addAll(Arrays.asList(newSortColumns));
-				TableColumnManager.getInstance().setDefaultSortColumnNames(tableID, sortColumnNames, true );
+				TableColumnManager.getInstance().setSortColumnNames(tableID, sortColumnNames );
 			}
 			if (columnsChanged) {
 				for (TableRowCore row : sortedRows) {
