@@ -347,6 +347,17 @@ public class ConfigSectionFile
 		bpUseResume.addEnabledOnSelection(pgResumeGroup);
 		save_peers.addEnabledOnSelection(savePeersMax);
 
+		// disable interim state save 
+		BooleanParameterImpl bDisableSveInterim = new BooleanParameterImpl(BCFG_DISABLE_SAVE_INTERIM_DOWNLOAD_STATE,
+				"ConfigView.label.disableinterimstatesave");
+		add(bDisableSveInterim, Parameter.MODE_ADVANCED );
+		
+			// skip complete download file existance checks 
+		
+		BooleanParameterImpl bSkipCompDLFileChecks = new BooleanParameterImpl(BCFG_SKIP_COMP_DL_FILE_CHECKS,
+				"ConfigView.label.skipCompDLFileChecks");
+		add(bSkipCompDLFileChecks, Parameter.MODE_ADVANCED );
+
 		List<Parameter> listExt = new ArrayList<>();
 
 		StringParameterImpl priorityExtensions = new StringParameterImpl(
@@ -363,7 +374,8 @@ public class ConfigSectionFile
 		pgPriorityExt.setNumberOfColumns(2);
 
 		add(new StringParameterImpl(SCFG_FILE_AUTO_SEQUENTIAL_EXTS,
-				"ConfigView.label.sequential.exts"), Parameter.MODE_INTERMEDIATE);
+				"ConfigView.label.sequential.exts"), Parameter.MODE_INTERMEDIATE,
+				listExt);
 
 		// quick view
 
