@@ -27,6 +27,7 @@ import org.eclipse.swt.widgets.Display;
 
 import com.biglybt.core.*;
 import com.biglybt.core.CoreOperationTask.ProgressCallback;
+import com.biglybt.core.CoreOperationTask.ProgressCallbackAdapter;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.global.GlobalManager;
@@ -741,44 +742,14 @@ public class Initializer
 					AESemaphore stop_sem = new AESemaphore( "stop" );
 					
 					ProgressCallback	prog = 
-						new ProgressCallback()
+						new ProgressCallbackAdapter()
 						{
-							private volatile int 	percent = 0;
-							private volatile String	subtask = "";
-							
 							@Override
 							public int 
 							getStyle()
 							{
 								return( STYLE_NO_CLOSE | STYLE_MODAL );
 							}
-							public int
-							getProgress()
-							{
-								return( percent );
-							}
-							
-							@Override
-							public void 
-							setProgress(
-								int _percent )
-							{
-								percent = _percent;
-							}
-							
-							public String
-							getSubTaskName()
-							{
-								return( subtask );
-							}
-							
-							@Override
-							public void 
-							setSubTaskName(
-								String name )
-							{
-								subtask	= name;
-							}
 							
 							@Override
 							public int 
@@ -792,54 +763,59 @@ public class Initializer
 							{
 								return( ProgressCallback.ST_SUBTASKS );
 							}
-							
-							public void
-							setTaskState(
-								int		state )
-							{
-							}
 						};
 						
-						if ( Utils.isSWTThread()){
+						boolean show_progress = COConfigurationManager.getBooleanParameter( "Tidy Close With Progress" );
+						
+						if ( show_progress ){
 							
-							Debug.out( "Can't run closedown progress window as already on SWT thread" );
-							
-						}else{
-
-							Utils.execSWTThread(()->{
-								FileUtil.runAsTask(
-									CoreOperation.OP_PROGRESS,
-									new CoreOperationTask()
-									{
-										@Override
-										public String 
-										getName()
-										{
-											return( MessageText.getString( isForRestart?"label.restarting.app":"label.closing.app" ));
-										}
+							if ( Utils.isSWTThread()){
+								
+								Debug.out( "Can't run closedown progress window as already on SWT thread" );
+								
+							}else{
 	
-										@Override
-										public void
-										run(
-											CoreOperation operation)
+								Utils.execSWTThread(()->{
+									FileUtil.runAsTask(
+										CoreOperation.OP_PROGRESS,
+										new CoreOperationTask()
 										{
-											try{
-												
-												stop_sem.reserve();
-												
-											}catch( Throwable e ){
-	
-												throw( new RuntimeException( e ));
+											@Override
+											public String 
+											getName()
+											{
+												return( MessageText.getString( isForRestart?"label.restarting.app":"label.closing.app" ));
 											}
-										}
-	
-										@Override
-										public ProgressCallback 
-										getProgressCallback()
-										{
-											return( prog );
-										}
-									});});
+		
+											@Override
+											public DownloadManager
+											getDownload()
+											{
+												return( null );
+											}
+											
+											@Override
+											public void
+											run(
+												CoreOperation operation)
+											{
+												try{												
+													stop_sem.reserve();
+													
+												}catch( Throwable e ){
+		
+													throw( new RuntimeException( e ));
+												}
+											}
+		
+											@Override
+											public ProgressCallback 
+											getProgressCallback()
+											{
+												return( prog );
+											}
+										});});
+							}
 					}
 					
 					try{
