@@ -51,6 +51,8 @@ import com.biglybt.plugin.I2PHelpers;
 import com.biglybt.plugin.net.buddy.BuddyPluginUtils;
 import com.biglybt.plugin.net.buddy.BuddyPluginBeta;
 import com.biglybt.plugin.net.buddy.BuddyPluginBeta.ChatInstance;
+import com.biglybt.plugin.net.buddy.BuddyPluginUI;
+import com.biglybt.plugin.net.buddy.swt.BuddyUIUtils;
 
 
 /**
@@ -173,6 +175,8 @@ public class MenuBuildUtils {
 		public void notifyFillListeners(MenuItem menu_item);
 
 		public void buildSubmenu(MenuItem parent);
+		
+		public void buildComplete( Menu menu );
 	}
 
 	/**
@@ -221,6 +225,10 @@ public class MenuBuildUtils {
 				}
 			}
 		}
+		
+		@Override
+		public void buildComplete(Menu menu){
+		}
 	}
 
 	/**
@@ -354,6 +362,8 @@ public class MenuBuildUtils {
 			menuItem.setEnabled(enable_items && az_menuitem.isEnabled());
 
 		}
+		
+		controller.buildComplete( parent );
 	}
 
 	/**
@@ -455,8 +465,18 @@ public class MenuBuildUtils {
 
 			j++;
 
-			edge_lens[i] 	= Math.min( c1.length,Math.max( edge_lens[i], j ));
-			edge_lens[i+1] 	= Math.min( c2.length, j );
+			int e1 = edge_lens[i] 	= Math.min( c1.length,Math.max( edge_lens[i], j ));
+			
+			if ( e1 > 0 && e1 < c1.length && Character.isHighSurrogate( c1[e1-1] )){
+				edge_lens[i]++;
+			}
+			
+			int e2 = edge_lens[i+1] 	= Math.min( c2.length, j );
+			
+			if ( e2 > 0 && e2 < c2.length && Character.isHighSurrogate( c2[e2-1] )){
+				edge_lens[i+1]++;
+			}
+
 		}
 
 		int	bucket_pos 	= 0;
@@ -775,17 +795,27 @@ public class MenuBuildUtils {
 	public static void
 	addChatMenu(
 		Menu			menu,
-		String			menu_resource_key,
-		final String	chat_key )
+		String			chat_resource,
+		String			chat_key )
 	{
+		final Menu top_menu = new Menu(menu.getShell(), SWT.DROP_DOWN);
+
+		top_menu.setEnabled( chat_key != null );
+			
+		final org.eclipse.swt.widgets.MenuItem top_item = new org.eclipse.swt.widgets.MenuItem(menu, SWT.CASCADE);
+
+		Messages.setLanguageText( top_item, "label.chat" );
+
+		top_item.setMenu(top_menu);
+
 		final Menu chat_menu = new Menu(menu.getShell(), SWT.DROP_DOWN);
 
-		final org.eclipse.swt.widgets.MenuItem chat_item = new org.eclipse.swt.widgets.MenuItem(menu, SWT.CASCADE);
+		final org.eclipse.swt.widgets.MenuItem chat_item = new org.eclipse.swt.widgets.MenuItem(top_menu, SWT.CASCADE);
 
-		Messages.setLanguageText( chat_item, menu_resource_key );
-
-		chat_item.setMenu(chat_menu);
+		Messages.setLanguageText( chat_item, chat_resource );
 
+		chat_item.setMenu( chat_menu );
+		
 		chat_menu.addMenuListener(
 			new MenuAdapter()
 			{
@@ -895,9 +925,26 @@ public class MenuBuildUtils {
 	public interface
 	ChatKeyResolver
 	{
+		public String
+		getResourceKey();
+		
 		public String
 		getChatKey(
-			Object		object );
+			Object		target );
+		
+		public default boolean
+		canShareMessage()
+		{
+			return( false );
+		}
+		
+		public default void
+		shareMessage(
+			Object				target,
+			ChatInstance		chat )
+			
+		{	
+		}
 	}
 
 	public static MenuItem
@@ -920,8 +967,42 @@ public class MenuBuildUtils {
 				{
 					menu.removeAllChildItems();
 
+					if ( chat_key_resolver.canShareMessage()){
+							
+						MenuItem chat_share = menu_manager.addMenuItem(chat_item, "menu.share.download");
+						
+						addChatSelectionMenu( 
+							menu_manager, 
+							chat_share, 
+							null, 
+							new ChatSelectionListener(){
+								public void
+								chatSelected(
+									Object	target,
+									String	chat )
+								{
+								}
+								
+								public void
+								chatAvailable(
+									Object			target,
+									ChatInstance	chat )
+								{
+									chat_key_resolver.shareMessage(target, chat );
+								}
+							});
+						
+						MenuItem mi = menu_manager.addMenuItem( chat_item, "sep" );
+						
+						mi.setStyle( MenuItem.STYLE_SEPARATOR );
+					}
+					
+					MenuItem discuss_menu = menu_manager.addMenuItem(chat_item, chat_key_resolver.getResourceKey());
+
+					discuss_menu.setStyle( MenuItem.STYLE_MENU );
+					
 					{
-						MenuItem chat_pub = menu_manager.addMenuItem(chat_item,  "label.public");
+						MenuItem chat_pub = menu_manager.addMenuItem(discuss_menu,  "label.public");
 
 						chat_pub.addMultiListener(
 								new MenuItemListener() {
@@ -977,7 +1058,7 @@ public class MenuBuildUtils {
 
 					if ( BuddyPluginUtils.isBetaChatAnonAvailable()){
 
-						MenuItem chat_priv = menu_manager.addMenuItem(chat_item,  "label.anon");
+						MenuItem chat_priv = menu_manager.addMenuItem(discuss_menu,  "label.anon");
 
 						chat_priv.addMultiListener(
 							new MenuItemListener()
@@ -1068,7 +1149,172 @@ public class MenuBuildUtils {
 	{
 		public void
 		chatSelected(
-			String	chat );
+			Object		target,
+			String		chat );
+		
+		public void
+		chatAvailable(
+			Object			target,
+			ChatInstance	chat );
+	}
+	
+	private static class
+	ChatSelectionDelegator
+		implements ChatSelectionListener
+	{
+		private ChatSelectionListener	delegate;
+				
+		private TimerEventPeriodic	timer;
+		
+		protected
+		ChatSelectionDelegator(
+			ChatSelectionListener	_delegate )
+		{
+			delegate = _delegate;
+		}
+		
+		public void
+		chatSelected(
+			Object		target,
+			String		name )
+		{
+			delegate.chatSelected(target, name);
+		}
+		
+		public void
+		chatAvailable(
+			Object			target,
+			ChatInstance	chat )
+		{
+			synchronized( ChatSelectionDelegator.this ){
+				
+				if ( timer != null ){
+					
+					return;
+				}
+				
+				long start = SystemTime.getMonotonousTime();
+				
+				timer = SimpleTimer.addPeriodicEvent( "availcheck", 1000, (ev)->{
+					
+					if ( chat.isAvailable()){
+						
+						if ( !chat.isDestroyed()){
+						
+							delegate.chatAvailable(target, chat);
+						}
+						
+						synchronized( ChatSelectionDelegator.this ){
+							
+							timer.cancel();
+						}
+					}
+					
+					if ( SystemTime.getMonotonousTime() - start > 3*60*1000 ){
+
+						Debug.out( "Gave up waiting for " + chat.getNetAndKey() + " to become available" );
+						
+						synchronized( ChatSelectionDelegator.this ){
+							
+							timer.cancel();
+						}
+					}
+				});
+			}
+		}
+	}
+	
+	public static void
+	addChatSelectionMenu(
+		final MenuManager		menu_manager,
+		final MenuItem			chat_item,
+		String					existing_chat,
+		ChatSelectionListener	_listener )
+	{
+		ChatSelectionListener listener = new ChatSelectionDelegator( _listener );
+		
+		chat_item.setStyle( MenuItem.STYLE_MENU );
+
+		chat_item.addFillListener(
+			new MenuItemFillListener()
+			{
+
+				@Override
+				public void
+				menuWillBeShown(
+					MenuItem 	menu,
+					Object 		data)
+				{
+					menu.removeAllChildItems();
+
+					MenuItem create_menu = menu_manager.addMenuItem( chat_item, "chat.view.create.chat" );
+
+					create_menu.setStyle( MenuItem.STYLE_MENU );
+					
+					BuddyUIUtils.createChat( 
+						menu_manager,
+						create_menu,
+						true,
+						new BuddyUIUtils.ChatCreationListener(){
+							
+							@Override
+							public void chatCreated(Object target, String name){
+								listener.chatSelected( target, name);
+							}
+							
+							@Override
+							public void chatAvailable(Object target,ChatInstance chat){
+								listener.chatAvailable( target,chat );
+							}
+						});
+					
+					MenuItem mi = menu_manager.addMenuItem( chat_item, "sep" );
+					
+					mi.setStyle( MenuItem.STYLE_SEPARATOR );
+					
+					List<ChatInstance> chats = BuddyPluginUtils.getChats();			
+					
+					for ( int i=0;i<2;i++){
+						
+						MenuItem chat_menu = menu_manager.addMenuItem( chat_item, i==0?"label.public":"label.anon" );
+
+						chat_menu.setStyle( MenuItem.STYLE_MENU );
+		
+						String net = i==0?AENetworkClassifier.AT_PUBLIC:AENetworkClassifier.AT_I2P;
+
+						for ( ChatInstance chat: chats ){
+							
+							if ( chat.getNetwork() == net ){
+								
+								mi = menu_manager.addMenuItem( chat_menu, "!" + chat.getKey() + "!" );
+																
+								mi.addMultiListener(
+									new MenuItemListener()
+									{
+										@Override
+										public void
+										selected(
+											MenuItem			menu,
+											Object 				target )
+										{
+											listener.chatSelected( target, chat.getNetAndKey());
+											
+											listener.chatAvailable( target, chat );
+											
+											try{
+												BuddyPluginUtils.getBetaPlugin().showChat( chat );
+												
+											}catch( Throwable e ){
+												
+												Debug.out( e );
+											}
+										}
+									});
+							}
+						}
+					}
+				}
+			});
 	}
 	
 	public static void
@@ -1076,8 +1322,10 @@ public class MenuBuildUtils {
 		Menu					menu,
 		String					resource_key,
 		String					existing_chat,
-		ChatSelectionListener	listener )
-{
+		ChatSelectionListener	_listener )
+	{
+		ChatSelectionListener listener = new ChatSelectionDelegator( _listener );
+
 		final Menu chat_menu = new Menu(menu.getShell(), SWT.DROP_DOWN);
 
 		final org.eclipse.swt.widgets.MenuItem chat_item = new org.eclipse.swt.widgets.MenuItem(menu, SWT.CASCADE);
@@ -1110,6 +1358,21 @@ public class MenuBuildUtils {
 						return;
 					}
 					
+					BuddyUIUtils.createChat( 
+						chat_menu,
+						new BuddyUIUtils.ChatCreationListener(){
+							@Override
+							public void chatCreated(Object target, String name){
+								listener.chatSelected( target, name );
+							}
+							@Override
+							public void chatAvailable(Object target, ChatInstance chat){
+								listener.chatAvailable( target,chat );
+							}
+						});
+					
+					new org.eclipse.swt.widgets.MenuItem( chat_menu, SWT.SEPARATOR );
+					
 					org.eclipse.swt.widgets.MenuItem mi_none = new org.eclipse.swt.widgets.MenuItem(chat_menu, SWT.RADIO);
 					
 					Messages.setLanguageText(mi_none, "label.none");
@@ -1120,7 +1383,7 @@ public class MenuBuildUtils {
 						@Override
 						public void handleEvent(Event event){
 							
-							listener.chatSelected( "" );
+							listener.chatSelected( null, "" );
 						}});
 					
 					for ( int i=0;i<2;i++){
@@ -1164,7 +1427,9 @@ public class MenuBuildUtils {
 												@Override
 												public void handleEvent(Event event){
 													
-													listener.chatSelected( chat.getNetAndKey());
+													listener.chatSelected( null, chat.getNetAndKey());
+													
+													listener.chatAvailable( null, chat);
 												}});
 										}
 									}
@@ -1174,4 +1439,25 @@ public class MenuBuildUtils {
 				}
 			});
 	}
+	
+		/**
+		 * Adds a separator if the current last item isn't one already
+		 * @param menu
+		 */
+	
+	public static void
+	addSeparater(
+		Menu		menu )
+	{
+		if ( menu.isDisposed() || menu.getItemCount() == 0 ){
+			return;
+		}
+		
+		org.eclipse.swt.widgets.MenuItem[] items = menu.getItems();
+		
+		if ( items[items.length-1].getStyle() != SWT.SEPARATOR ){
+			
+			new org.eclipse.swt.widgets.MenuItem( menu, SWT.SEPARATOR );
+		}
+	}
 }
