@@ -37,9 +37,10 @@ import com.biglybt.util.StringCompareUtils;
  */
 public class Messages {
 
-  private static final String RESOURCE_KEY = "com.biglybt.ui.swt.Messages:res";
-  
-  private static final Pattern HIG_ELLIP_EXP = Pattern.compile("([\\.]{3})"); // rec. hig style on some platforms
+	private static final String RESOURCE_KEY 		= "com.biglybt.ui.swt.Messages:res";
+	private static final String RESOURCE_TT_KEY 	= "com.biglybt.ui.swt.Messages:ttres";
+
+	private static final Pattern HIG_ELLIP_EXP = Pattern.compile("([\\.]{3})"); // rec. hig style on some platforms
 	private static Listener hoverListener;
 
 	static {
@@ -50,103 +51,103 @@ public class Messages {
 			}
 		};
 	}
-  /**
-   *
-   */
-  private Messages() {
-  }
+	/**
+	 *
+	 */
+	private Messages() {
+	}
 
-  public static void updateLanguageForControl(Widget widget) {
-    if (widget == null || widget.isDisposed())
-      return;
+	public static void updateLanguageForControl(Widget widget) {
+		if (widget == null || widget.isDisposed())
+			return;
 
-    updateLanguageFromData(widget);	// OK, so we loose parameters on language change...
-  	widget.removeListener(SWT.MouseHover, hoverListener);
-  	widget.addListener(SWT.MouseHover, hoverListener);
+		updateLanguageFromData(widget);	// OK, so we loose parameters on language change...
+		widget.removeListener(SWT.MouseHover, hoverListener);
+		widget.addListener(SWT.MouseHover, hoverListener);
 
-    if (widget instanceof CTabFolder) {
-      CTabFolder folder = (CTabFolder) widget;
-      CTabItem[] items = folder.getItems();
-      for (int i = 0; i < items.length; i++) {
-        updateLanguageForControl(items[i]);
-        updateLanguageForControl(items[i].getControl());
-      }
-    } else if (widget instanceof TabFolder) {
-      TabFolder folder = (TabFolder) widget;
-      TabItem[] items = folder.getItems();
-      for (int i = 0; i < items.length; i++) {
-        updateLanguageForControl(items[i]);
-        updateLanguageForControl(items[i].getControl());
-      }
-    }
-    else if(widget instanceof CoolBar) {
-      CoolItem[] items = ((CoolBar)widget).getItems();
-      for(int i = 0 ; i < items.length ; i++) {
-        Control control = items[i].getControl();
-        updateLanguageForControl(control);
-      }
-    }
-    else if(widget instanceof ToolBar) {
-      ToolItem[] items = ((ToolBar)widget).getItems();
-      for(int i = 0 ; i < items.length ; i++) {
-        updateLanguageForControl(items[i]);
-      }
-    }
-    else if (widget instanceof Composite) {
-      Composite group = (Composite) widget;
-      Control[] controls = group.getChildren();
-      for (int i = 0; i < controls.length; i++) {
-        updateLanguageForControl(controls[i]);
-      }
-      if (widget instanceof Table) {
-        Table table = (Table) widget;
-        TableColumn[] columns = table.getColumns();
-        for (int i = 0; i < columns.length; i++) {
-          updateLanguageFromData(columns[i]);
-        }
-        updateLanguageForControl(table.getMenu());
+		if (widget instanceof CTabFolder) {
+			CTabFolder folder = (CTabFolder) widget;
+			CTabItem[] items = folder.getItems();
+			for (int i = 0; i < items.length; i++) {
+				updateLanguageForControl(items[i]);
+				updateLanguageForControl(items[i].getControl());
+			}
+		} else if (widget instanceof TabFolder) {
+			TabFolder folder = (TabFolder) widget;
+			TabItem[] items = folder.getItems();
+			for (int i = 0; i < items.length; i++) {
+				updateLanguageForControl(items[i]);
+				updateLanguageForControl(items[i].getControl());
+			}
+		}
+		else if(widget instanceof CoolBar) {
+			CoolItem[] items = ((CoolBar)widget).getItems();
+			for(int i = 0 ; i < items.length ; i++) {
+				Control control = items[i].getControl();
+				updateLanguageForControl(control);
+			}
+		}
+		else if(widget instanceof ToolBar) {
+			ToolItem[] items = ((ToolBar)widget).getItems();
+			for(int i = 0 ; i < items.length ; i++) {
+				updateLanguageForControl(items[i]);
+			}
+		}
+		else if (widget instanceof Composite) {
+			Composite group = (Composite) widget;
+			Control[] controls = group.getChildren();
+			for (int i = 0; i < controls.length; i++) {
+				updateLanguageForControl(controls[i]);
+			}
+			if (widget instanceof Table) {
+				Table table = (Table) widget;
+				TableColumn[] columns = table.getColumns();
+				for (int i = 0; i < columns.length; i++) {
+					updateLanguageFromData(columns[i]);
+				}
+				updateLanguageForControl(table.getMenu());
 
-        // XXX We could (should?) send this event for all widget types
-        // XXX Would it better to have a custom event type?
-        Event event = new Event();
-        event.type = SWT.Settings;
-        event.widget = widget;
-        widget.notifyListeners(SWT.Settings, event);
-      }
-      else if (widget instanceof Tree) {
-        Tree tree = (Tree) widget;
-        TreeItem[] treeitems = tree.getItems();
-        for (int i = 0; i < treeitems.length; i++) {
-          updateLanguageForControl(treeitems[i]);
-        }
-      }
+				// XXX We could (should?) send this event for all widget types
+				// XXX Would it better to have a custom event type?
+				Event event = new Event();
+				event.type = SWT.Settings;
+				event.widget = widget;
+				widget.notifyListeners(SWT.Settings, event);
+			}
+			else if (widget instanceof Tree) {
+				Tree tree = (Tree) widget;
+				TreeItem[] treeitems = tree.getItems();
+				for (int i = 0; i < treeitems.length; i++) {
+					updateLanguageForControl(treeitems[i]);
+				}
+			}
 
-      group.layout();
-    }
-    else if (widget instanceof MenuItem) {
-      MenuItem menuItem = (MenuItem) widget;
-      updateLanguageForControl(menuItem.getMenu());
-    }
-    else if (widget instanceof Menu) {
-      Menu menu = (Menu) widget;
-      if (menu.getStyle() == SWT.POP_UP)
-        System.out.println("POP_UP");
+			group.layout();
+		}
+		else if (widget instanceof MenuItem) {
+			MenuItem menuItem = (MenuItem) widget;
+			updateLanguageForControl(menuItem.getMenu());
+		}
+		else if (widget instanceof Menu) {
+			Menu menu = (Menu) widget;
+			if (menu.getStyle() == SWT.POP_UP)
+				System.out.println("POP_UP");
 
-      MenuItem[] items = menu.getItems();
-      for (int i = 0; i < items.length; i++) {
-        updateLanguageForControl(items[i]);
-      }
-    }
-    else if (widget instanceof TreeItem) {
-      TreeItem treeitem = (TreeItem) widget;
-      TreeItem[] treeitems = treeitem.getItems();
-      for (int i = 0; i < treeitems.length; i++) {
-        updateLanguageFromData(treeitems[i]);
-        updateLanguageForControl(treeitems[i]);
-      }
-    }
+			MenuItem[] items = menu.getItems();
+			for (int i = 0; i < items.length; i++) {
+				updateLanguageForControl(items[i]);
+			}
+		}
+		else if (widget instanceof TreeItem) {
+			TreeItem treeitem = (TreeItem) widget;
+			TreeItem[] treeitems = treeitem.getItems();
+			for (int i = 0; i < treeitems.length; i++) {
+				updateLanguageFromData(treeitems[i]);
+				updateLanguageForControl(treeitems[i]);
+			}
+		}
 
-  }
+	}
 
 	// Needed only because old plugins use this signature
 	public static void setLanguageText(Widget widget, String key) {
@@ -181,59 +182,13 @@ public class Messages {
 
 	public static void setLanguageText(Widget widget, String key,
 			boolean setTooltipOnly, String... params) {
-  	widget.setData(RESOURCE_KEY,key);
-  	if(!setTooltipOnly)
-      updateLanguageFromData(widget, params);
-  	widget.removeListener(SWT.MouseHover, hoverListener);
-  	widget.addListener(SWT.MouseHover, hoverListener);
-  }
-
-	private static void updateToolTipFromData(Widget widget, boolean showKey) {
-		String key = (String) widget.getData(RESOURCE_KEY);
-		if (key == null) {
-			return;
-		}
-		if (widget instanceof Control) {
-			if (showKey) {
-				Utils.setTT((Control) widget,key);
-				return;
-			}
-			if (!key.endsWith(".tooltip")) {
-				key += ".tooltip";
-			}
-			String toolTip = MessageText.getString(key);
-			if (!toolTip.equals('!' + key + '!')) {
-				Utils.setTT((Control) widget, toolTip);
-			}
-		} else if (widget instanceof ToolItem) {
-			if (!key.endsWith(".tooltip")) {
-				key += ".tooltip";
-			}
-			String toolTip = MessageText.getString(key);
-			if (!toolTip.equals('!' + key + '!')) {
-				Utils.setTT((ToolItem) widget,toolTip.replaceAll("Meta\\+",
-						Constants.isOSX ? "Cmd+" : "Ctrl+"));
-			}
-		} else if (widget instanceof TableColumn) {
-			if (!key.endsWith(".info")) {
-				key += ".info";
-			}
-			String toolTip = MessageText.getString(key, (String) null);
-			if (toolTip == null) {
-				toolTip = MessageText.getString(key.substring(0, key.length() - 5),
-						(String) null);
-			}
-			if (toolTip != null) {
-				try {
-					Utils.setTT((TableColumn) widget,toolTip);
-				} catch (NoSuchMethodError e) {
-					// Pre SWT 3.2
-				}
-			}
-		}
+		widget.setData(RESOURCE_KEY,key);
+		if(!setTooltipOnly)
+			updateLanguageFromData(widget, params);
+		widget.removeListener(SWT.MouseHover, hoverListener);
+		widget.addListener(SWT.MouseHover, hoverListener);
 	}
 
-
 	private static void updateLanguageFromData(Widget widget, String... params) {
 		if (widget == null || widget.isDisposed()) {
 			return;
@@ -306,21 +261,80 @@ public class Messages {
 		}
 	}
 
-  public static void setLanguageTooltip(Widget widget, String key) {
-  	if (widget == null || widget.isDisposed()) {
-  		return;
-  	}
+	private static void updateToolTipFromData(Widget widget, boolean showKey) {
+		
+		String tt_key = (String)widget.getData(RESOURCE_TT_KEY);
+		
+		if ( tt_key != null ){
+			
+			updateTooltipLanguageFromData( widget );
+			
+			return;
+		}
+		
+		String key = (String) widget.getData(RESOURCE_KEY);
+		if (key == null) {
+			return;
+		}
+		if (widget instanceof Control) {
+			if (showKey) {
+				Utils.setTT((Control) widget,key);
+				return;
+			}
+			if (!key.endsWith(".tooltip")) {
+				key += ".tooltip";
+			}
+			String toolTip = MessageText.getString(key);
+			if (!toolTip.equals('!' + key + '!')) {
+				Utils.setTT((Control) widget, toolTip);
+			}
+		} else if (widget instanceof ToolItem) {
+			if (!key.endsWith(".tooltip")) {
+				key += ".tooltip";
+			}
+			String toolTip = MessageText.getString(key);
+			if (!toolTip.equals('!' + key + '!')) {
+				Utils.setTT((ToolItem) widget,toolTip.replaceAll("Meta\\+",
+						Constants.isOSX ? "Cmd+" : "Ctrl+"));
+			}
+		} else if (widget instanceof TableColumn) {
+			if (!key.endsWith(".info")) {
+				key += ".info";
+			}
+			String toolTip = MessageText.getString(key, (String) null);
+			if (toolTip == null) {
+				toolTip = MessageText.getString(key.substring(0, key.length() - 5),
+						(String) null);
+			}
+			if (toolTip != null) {
+				try {
+					Utils.setTT((TableColumn) widget,toolTip);
+				} catch (NoSuchMethodError e) {
+					// Pre SWT 3.2
+				}
+			}
+		}
+	}
 
-  	widget.setData(RESOURCE_KEY,key);
-    updateTooltipLanguageFromData(widget);
-  }
 
-  public static void updateTooltipLanguageFromData(Widget widget) {
-  	if (widget == null || widget.isDisposed()) {
-  		return;
-  	}
-		if (widget.getData(RESOURCE_KEY) != null) {
-			String sToolTip = MessageText.getString((String) widget.getData(RESOURCE_KEY));
+	public static void setLanguageTooltip(Widget widget, String key) {
+		if (widget == null || widget.isDisposed()) {
+			return;
+		}
+
+		widget.setData(RESOURCE_TT_KEY,key);
+		updateTooltipLanguageFromData(widget);
+	}
+
+	private static void updateTooltipLanguageFromData(Widget widget) {
+		if (widget == null || widget.isDisposed()) {
+			return;
+		}
+		
+		String tt_key = (String)widget.getData(RESOURCE_TT_KEY);
+		
+		if ( tt_key != null ){
+			String sToolTip = MessageText.getString( tt_key );
 			if (widget instanceof CLabel)
 				Utils.setTT((CLabel) widget,sToolTip);
 			else if (widget instanceof Label)
