@@ -131,12 +131,14 @@ public class UIConfigDefaultsSWT
 		}
 
 		def.addParameter("suppress_file_download_dialog", false);
+		def.addParameter("Suppress File Move Dialog", false);
 		def.addParameter("Suppress Sharing Dialog", false);
 		def.addParameter("auto_remove_inactive_items", false);
 		def.addParameter("show_torrents_menu", true);
 		def.addParameter("mainwindow.search.history.enabled", true);
 		def.addParameter("Disable All Tooltips", false);
-
+		def.addParameter("Tidy Close With Progress", true );
+		
 		def.addParameter("Ignore Icon Exts", "" );
 		
 		if ( Constants.isWindows ){
@@ -210,7 +212,9 @@ public class UIConfigDefaultsSWT
 		def.addParameter("Table.useTree", false);
 		def.addParameter("Table.tooltip.disable", false);
 		def.addParameter("Table.sort.intuitive", false);
-
+		def.addParameter("Dark Table Colors", false );
+		def.addParameter("Dark Misc Colors", false );
+		
 		if ("az2".equalsIgnoreCase(COConfigurationManager.getStringParameter("ui", "az3"))) {
 			def.addParameter("v3.Show Welcome", false);
 
@@ -303,6 +307,7 @@ public class UIConfigDefaultsSWT
 		}
 		
 		def.addParameter( "XferStats.show.samples", true );
+		def.addParameter( "XferStats.local.show.rates", true );
 		def.addParameter( "browser.external.non.pub", true );
 		def.addParameter( "browser.external.test.url", Constants.URL_CLIENT_HOME );
 
@@ -314,5 +319,6 @@ public class UIConfigDefaultsSWT
 		def.addParameter( "Library.TagInTabBar", 1 );
 		
 		def.addParameter( "Peers View Show Local Peer", false);
+		def.addParameter( "Pieces View Show Uploading", false);
 	}
 }
