@@ -73,7 +73,7 @@ public abstract class BaseMDI
 	private Map<String, MdiEntryCreationListener2> mapIdToCreationListener2 = new LightHashMap<>();
 
 	// Sync changes to entry maps on mapIdEntry
-	private final Map<String, BaseMdiEntry> mapIdToEntry = new LinkedHashMap<>(8);
+	private final Map<String, BaseMdiEntry> mapIdToEntry = new HashMap<>(8);
 
 	private final List<MdiListener> listeners = new ArrayList<>();
 
@@ -207,7 +207,7 @@ public abstract class BaseMDI
 		return( closeEntryByID( id, false ));
 	}
 
-	private BaseMdiEntry closeEntryByID(String id, boolean userInitiated) {
+	protected BaseMdiEntry closeEntryByID(String id, boolean userInitiated) {
 		// We'll get here from closeEntry (BaseMDI), skinObjectDestroyed (TabbedMDI)
 		// with a 99% chance the display is disposed
 		if (Utils.isDisplayDisposed()) {
@@ -297,19 +297,6 @@ public abstract class BaseMDI
 			return mapIdToEntry.get(id);
 		}
 	}
-	
-	protected int indexOf(String id) {
-		int i = 0;
-		synchronized(mapIdToEntry){
-			for (String key : mapIdToEntry.keySet()) {
-				if (key.equals(id)) {
-					return i;
-				}
-				i++;
-			}
-		}
-		return -1;
-	}
 
 	/**
 	 * @param skinView
@@ -395,6 +382,40 @@ public abstract class BaseMDI
 		
 	}
 
+	private boolean
+	canCreateEntryByCreationListener(String id ){
+	
+		MdiEntryCreationListener mdiEntryCreationListener = null;
+		
+		for (String key : mapIdToCreationListener.keySet()) {
+			if (Pattern.matches(key, id)) {
+				mdiEntryCreationListener = mapIdToCreationListener.get(key);
+				break;
+			}
+		}
+		
+		if (mdiEntryCreationListener != null) {
+			
+			return( true );
+		}
+
+		MdiEntryCreationListener2 mdiEntryCreationListener2 = null;
+		
+		for (String key : mapIdToCreationListener2.keySet()) {
+			if (Pattern.matches(key, id)) {
+				mdiEntryCreationListener2 = mapIdToCreationListener2.get(key);
+				break;
+			}
+		}
+		
+		if (mdiEntryCreationListener2 != null) {
+			
+			return( true );
+		}
+
+		return( false );
+	}
+	
 	protected MdiEntry
 	createEntryByCreationListener(String id, Map<?, ?> autoOpenInfo)
 	{
@@ -472,12 +493,37 @@ public abstract class BaseMDI
 	public boolean showEntryByID(String id) {
 		return loadEntryByID(id, true, false, null);
 	}
+	
+	public boolean
+	canShowEntryByID( String id ){
+	
+		if ( id == null ){
+			
+			return( false );
+		}
+						
+		if ( canCreateEntryByCreationListener( id )){
+			
+			return( true );
+		}
+		
+		ViewManagerSWT vi = ViewManagerSWT.getInstance();
+		
+		UISWTViewBuilderCore builder = vi.getBuilder(viewID, id);
+		
+		if (builder == null) {
+		
+			builder = vi.getBuilder(getDataSourceType(), id);
+		}	
+		
+		return( builder != null );
+	}
 
 	@Override
 	public boolean showEntryByID(String id, Object datasource) {
 		return loadEntryByID(id, true, false, datasource);
 	}
-
+	
 	@Override
 	public Object skinObjectInitialShow(SWTSkinObject skinObject, Object params) {
 		final UIManager ui_manager = PluginInitializer.getDefaultInterface().getUIManager();
@@ -552,7 +598,7 @@ public abstract class BaseMDI
 	public void updateUI() {
 		MdiEntry currentEntry = getCurrentEntry();
 		if (currentEntry != null) {
-			currentEntry.updateUI();
+			currentEntry.updateUI( false );
 		}
 	}
 
@@ -623,7 +669,6 @@ public abstract class BaseMDI
 		return false;
 	}
 
-
 	protected abstract void setEntryLoadedOnce(String id);
 
 	protected abstract boolean wasEntryLoadedOnce(String id);
