@@ -20,6 +20,7 @@
 package com.biglybt.ui.swt.progress;
 
 import com.biglybt.core.Core;
+import com.biglybt.core.CoreFactory;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -44,6 +45,7 @@ import com.biglybt.core.CoreOperation;
 import com.biglybt.core.CoreOperationListener;
 import com.biglybt.core.CoreOperationTask;
 import com.biglybt.core.CoreOperationTask.ProgressCallback;
+import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.ui.swt.imageloader.ImageLoader;
 
 public class
@@ -58,24 +60,59 @@ ProgressWindow
 			{
 				@Override
 				public boolean
-				operationCreated(
+				operationExecuteRequest(
 					CoreOperation operation )
 				{
-					if ( 	( 	operation.getOperationType() == CoreOperation.OP_FILE_MOVE ||
-								operation.getOperationType() == CoreOperation.OP_DOWNLOAD_EXPORT ||
-								operation.getOperationType() == CoreOperation.OP_PROGRESS )&&
+					int type = operation.getOperationType();
+					
+					if ( 	( 	type == CoreOperation.OP_FILE_MOVE ||
+								type == CoreOperation.OP_DOWNLOAD_EXPORT ||
+								type == CoreOperation.OP_PROGRESS ) &&
 							Utils.isThisThreadSWT()){
 
-						if ( operation.getTask() != null ){
+						CoreOperationTask task = operation.getTask();
+						
+						if ( task != null ){
 
-							new ProgressWindow( operation );
+							if ( 	type ==  CoreOperation.OP_FILE_MOVE && 
+									COConfigurationManager.getBooleanParameter("Suppress File Move Dialog" )){
 
+								AEThread2.createAndStartDaemon( "Core Operation", ()->{
+									
+									try{
+										task.run( operation );
+										
+									}finally{
+										
+										core.removeOperation( operation );
+									}
+								});
+								
+							}else{
+								
+								new ProgressWindow( operation );
+							}
+							
 							return( true );
 						}
 					}
 
 					return( false );
 				}
+				
+				@Override
+				public void 
+				operationAdded(
+					CoreOperation operation )
+				{
+				}
+				
+				@Override
+				public void 
+				operationRemoved(
+					CoreOperation operation )
+				{
+				}
 			});
 	}
 
@@ -91,7 +128,7 @@ ProgressWindow
 	
 	private boolean	task_paused;
 	
-	protected
+	private
 	ProgressWindow(
 		final CoreOperation operation )
 	{
@@ -207,7 +244,12 @@ ProgressWindow
 					error[0] = new RuntimeException( e );
 
 				}finally{
-
+					
+						// report complete now as the SWT dispatch loop below might not exit until other windows
+						// has closed their loops...
+					
+					CoreFactory.getSingleton().removeOperation( operation );
+					
 					Utils.execSWTThread(
 							new Runnable()
 							{
@@ -497,80 +539,155 @@ ProgressWindow
 
 				// buttons
 		
+				boolean 	has_pause_resume 	= ( states & ( ProgressCallback.ST_PAUSE | ProgressCallback.ST_RESUME  )) != 0;
+				boolean 	has_cancel			= ( states & ProgressCallback.ST_CANCEL ) != 0;
+				
+					// must have something...
+				
+				if ( !has_pause_resume ){
+					has_cancel = true;
+				}
+				
+				int	 num_buttons = 0;
+				
+				if ( has_pause_resume  )num_buttons+=2;
+				if ( has_cancel  )num_buttons++;
+				
 				Composite comp = new Composite(shell,SWT.NULL);
 				gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.HORIZONTAL_ALIGN_END | GridData.HORIZONTAL_ALIGN_FILL);
 				gridData.grabExcessHorizontalSpace = true;
 				gridData.horizontalSpan = 2;
 				comp.setLayoutData(gridData);
 				GridLayout layoutButtons = new GridLayout();
-				layoutButtons.numColumns = 3;
+				layoutButtons.numColumns = num_buttons;
 				comp.setLayout(layoutButtons);
 				comp.setBackground( Colors.white );
 		
 				List<Button> buttons = new ArrayList<>();
 		
-				Button bPause = new Button(comp,SWT.PUSH);
-				bPause.setText(MessageText.getString("v3.MainWindow.button.pause"));
-				gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.HORIZONTAL_ALIGN_END | GridData.HORIZONTAL_ALIGN_FILL);
-				gridData.grabExcessHorizontalSpace = true;
-				//gridData.widthHint = 70;
-				bPause.setLayoutData(gridData);
-				buttons.add( bPause );
-		
-				Button bResume = new Button(comp,SWT.PUSH);
-				bResume.setText(MessageText.getString("v3.MainWindow.button.resume"));
-				gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.HORIZONTAL_ALIGN_END | GridData.HORIZONTAL_ALIGN_FILL);
-				gridData.grabExcessHorizontalSpace = false;
-				//gridData.widthHint = 70;
-				bResume.setLayoutData(gridData);
-				buttons.add( bResume );
-
+				Button bPause;
+				Button bResume;
+				Button bCancel;
+				
+				if ( has_pause_resume ){
+					bPause = new Button(comp,SWT.PUSH);
+					bPause.setText(MessageText.getString("v3.MainWindow.button.pause"));
+					gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.HORIZONTAL_ALIGN_END | GridData.HORIZONTAL_ALIGN_FILL);
+					gridData.grabExcessHorizontalSpace = true;
+					//gridData.widthHint = 70;
+					bPause.setLayoutData(gridData);
+					buttons.add( bPause );
+					
+					bResume = new Button(comp,SWT.PUSH);
+					bResume.setText(MessageText.getString("v3.MainWindow.button.resume"));
+					gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.HORIZONTAL_ALIGN_END | GridData.HORIZONTAL_ALIGN_FILL);
+					gridData.grabExcessHorizontalSpace = false;
+					//gridData.widthHint = 70;
+					bResume.setLayoutData(gridData);
+					buttons.add( bResume );
+				}else{
+					bPause	= null;
+					bResume = null;
+				}
+				
+				if ( has_cancel ){
+					bCancel = new Button(comp,SWT.PUSH);
+					bCancel.setText(MessageText.getString("UpdateWindow.cancel"));
+					gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
+					gridData.grabExcessHorizontalSpace = false;
+					//gridData.widthHint = 70;
+					bCancel.setLayoutData(gridData);
+					buttons.add( bCancel );
+				}else{
+					bCancel = null;
+				}
 				
-				Button bCancel = new Button(comp,SWT.PUSH);
-				bCancel.setText(MessageText.getString("UpdateWindow.cancel"));
-				gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
-				gridData.grabExcessHorizontalSpace = false;
-				//gridData.widthHint = 70;
-				bCancel.setLayoutData(gridData);
-				buttons.add( bCancel );
-
 				Utils.makeButtonsEqualWidth( buttons );
 				
-				bResume.setEnabled( false );
+				if ( has_pause_resume ){
+					
+						// if we have resume then we have pause
+					
+					bResume.setEnabled( false );
+					
+					bPause.addListener(SWT.Selection,new Listener() {
+						@Override
+						public void handleEvent(Event e) {
+							task_paused	= true;
+							bPause.setEnabled( false );
+							bResume.setEnabled( true );
+							shell.setDefaultButton( bResume );
+							progress.setTaskState( ProgressCallback.ST_PAUSE );
+						}
+					});
+					
+					bResume.addListener(SWT.Selection,new Listener() {
+						@Override
+						public void handleEvent(Event e) {
+							task_paused	= false;
+							bPause.setEnabled( true );
+							bResume.setEnabled( false );
+							shell.setDefaultButton( bPause );
+							progress.setTaskState( ProgressCallback.ST_RESUME );
+						}
+					});
+				}
 				
-				bPause.addListener(SWT.Selection,new Listener() {
-					@Override
-					public void handleEvent(Event e) {
-						task_paused	= true;
-						bPause.setEnabled( false );
-						bResume.setEnabled( true );
-						shell.setDefaultButton( bResume );
-						progress.setTaskState( ProgressCallback.ST_PAUSE );
-					}
-				});
+				if ( bCancel != null ){
+					
+					bCancel.addListener(SWT.Selection,new Listener() {
+						@Override
+						public void handleEvent(Event e) {
+							if ( has_pause_resume ){
+								bPause.setEnabled( false );
+								bResume.setEnabled( false );
+							}
+							bCancel.setEnabled( false );
+							progress.setTaskState( ProgressCallback.ST_CANCEL );
+						}
+					});
+				}
 				
-				bResume.addListener(SWT.Selection,new Listener() {
-					@Override
-					public void handleEvent(Event e) {
-						task_paused	= false;
-						bPause.setEnabled( true );
-						bResume.setEnabled( false );
-						shell.setDefaultButton( bPause );
-						progress.setTaskState( ProgressCallback.ST_RESUME );
-					}
-				});
-		
-				bCancel.addListener(SWT.Selection,new Listener() {
-					@Override
-					public void handleEvent(Event e) {
-						bPause.setEnabled( false );
-						bResume.setEnabled( false );
-						bCancel.setEnabled( false );
-						progress.setTaskState( ProgressCallback.ST_CANCEL );
-					}
-				});
+				Utils.execSWTThreadLater(
+					250,
+					new Runnable()
+					{
+						@Override
+						public void run(){
+							if ( comp.isDisposed()){
+								return;
+							}
+							
+							int state = progress.getTaskState();
+							
+							if ( state == ProgressCallback.ST_CANCEL ){
+								
+								shell.dispose();
+								
+							}else{
+								
+								if ( has_pause_resume ){
+									
+									if ( state == ProgressCallback.ST_PAUSE ){
+										
+										bPause.setEnabled( false );
+										
+										bResume.setEnabled( true );
+										
+									}else{
+										
+										bPause.setEnabled( true );
+										
+										bResume.setEnabled( false );
+									}
+								}
+								
+								Utils.execSWTThreadLater( 250, this );
+							}
+						}
+					});
 				
-				shell.setDefaultButton( bPause );
+				shell.setDefaultButton( has_pause_resume?bPause:bCancel );
 				
 				shell.addDisposeListener(
 					new DisposeListener(){
