@@ -44,6 +44,7 @@ import com.biglybt.ui.swt.skin.SWTSkin;
 import com.biglybt.ui.swt.skin.SWTSkinFactory;
 import com.biglybt.ui.swt.skin.SWTSkinObject;
 import com.biglybt.ui.swt.skin.SWTSkinUtils;
+import com.biglybt.ui.swt.views.QuickLinksView;
 import com.biglybt.ui.swt.views.skin.SkinViewManager;
 import com.biglybt.ui.swt.views.skin.sidebar.SideBar;
 
@@ -275,6 +276,8 @@ public class MainMenuV3
 	private void buildSimpleViewMenu(final Menu viewMenu, int accelerator) {
 		try {
 
+			SWTSkin skin = SWTSkinFactory.getInstance();
+
 			MenuFactory.addMenuItem(viewMenu, SWT.CHECK, PREFIX_V3 + ".view.sidebar",
 					new Listener() {
 						@Override
@@ -288,17 +291,15 @@ public class MainMenuV3
 
 			if (COConfigurationManager.getIntParameter("User Mode") > 1) {
 
-				SWTSkin skin = SWTSkinFactory.getInstance();
-
 				SWTSkinObject plugin_bar = skin.getSkinObject(SkinConstants.VIEWID_PLUGINBAR);
 
 				if ( plugin_bar != null ){
 
 					MenuItem mi =
-						MainMenuV3.createViewMenuItem(skin, viewMenu,
+						MainMenuV3.createPluginBarMenuItem(skin, viewMenu,
 							"v3.MainWindow.menu.view." + SkinConstants.VIEWID_PLUGINBAR,
 							SkinConstants.VIEWID_PLUGINBAR + ".visible",
-							SkinConstants.VIEWID_PLUGINBAR, true, -1);
+							SkinConstants.VIEWID_PLUGINBAR );
 
 					if ( accelerator != -1 && mi.getAccelerator() == accelerator ){
 
@@ -315,8 +316,14 @@ public class MainMenuV3
 					}
 				}
 			}
-
+			
 			MenuFactory.addViewToolbarMenuItem(viewMenu);
+
+			MenuItem mi =
+					MainMenuV3.createQuickLinksMenuItem(skin, viewMenu,
+						"v3.MainWindow.menu.view." + SkinConstants.VIEWID_QUICK_LINKS,
+						SkinConstants.VIEWID_QUICK_LINKS + ".visible",
+						SkinConstants.VIEWID_QUICK_LINKS );
 			
 			/////////
 
@@ -603,16 +610,15 @@ public class MainMenuV3
 	/**
 	 * @param viewMenu
 	 */
-	public static MenuItem createViewMenuItem(final SWTSkin skin, Menu viewMenu,
-			final String textID, final String configID, final String viewID,
-			final boolean fast, int menuIndex) {
+	public static MenuItem createPluginBarMenuItem(final SWTSkin skin, Menu viewMenu,
+			final String textID, final String configID, final String viewID){
 		MenuItem item;
 
 		if (!ConfigurationDefaults.getInstance().doesParameterDefaultExist(configID)) {
 			COConfigurationManager.setBooleanDefault(configID, true);
 		}
 
-		item = MenuFactory.addMenuItem(viewMenu, SWT.CHECK, menuIndex, textID,
+		item = MenuFactory.addMenuItem(viewMenu, SWT.CHECK, textID,
 				new Listener() {
 					@Override
 					public void handleEvent(Event event) {
@@ -620,13 +626,12 @@ public class MainMenuV3
 						if (skinObject != null) {
 							boolean newVisibility = !skinObject.isVisible();
 
-							SWTSkinUtils.setVisibility(skin, configID, viewID, newVisibility,
-									true, fast);
+							SWTSkinUtils.setVisibility(skin, configID, viewID, newVisibility, true);
 						}
 					}
 				});
 		SWTSkinUtils.setVisibility(skin, configID, viewID,
-				COConfigurationManager.getBooleanParameter(configID), false, true);
+				COConfigurationManager.getBooleanParameter(configID), false);
 
 		final MenuItem itemViewPluginBar = item;
 		final ParameterListener listener = new ParameterListener() {
@@ -647,6 +652,47 @@ public class MainMenuV3
 		return item;
 	}
 
+	public static MenuItem createQuickLinksMenuItem(final SWTSkin skin, Menu viewMenu,
+			final String textID, final String configID, final String viewID) {
+		
+		if (!ConfigurationDefaults.getInstance().doesParameterDefaultExist(configID)) {
+			COConfigurationManager.setBooleanDefault(configID, true);
+		}
+
+		MenuItem item = MenuFactory.addMenuItem(viewMenu, SWT.CHECK, textID,
+				new Listener() {
+					@Override
+					public void handleEvent(Event event) {
+						SWTSkinObject skinObject = skin.getSkinObject(viewID);
+						if (skinObject != null) {
+							boolean newVisibility = !skinObject.isVisible();
+							
+							QuickLinksView.setVisible( newVisibility );
+						}
+					}
+				});
+	
+		item.setEnabled(COConfigurationManager.getBooleanParameter( "IconBar.enabled" ));
+		
+		final ParameterListener listener = new ParameterListener() {
+			@Override
+			public void parameterChanged(String parameterName) {
+				item.setSelection(COConfigurationManager.getBooleanParameter(parameterName));
+			}
+		};
+
+		COConfigurationManager.addAndFireParameterListener(configID, listener);
+		item.addDisposeListener(new DisposeListener() {
+			@Override
+			public void widgetDisposed(DisposeEvent e) {
+				COConfigurationManager.removeParameterListener(configID, listener);
+			}
+		});
+
+		return item;
+	}
+	
+	/*
 	// backward compat..
 	public static void setVisibility(SWTSkin skin, String configID,
 			String viewID, boolean visible) {
@@ -658,5 +704,5 @@ public class MainMenuV3
 			String viewID, boolean visible, boolean save) {
 		SWTSkinUtils.setVisibility(skin, configID, viewID, visible, save, false);
 	}
-
+	*/
 }
