@@ -1009,6 +1009,13 @@ public class UIFunctionsImpl
 		}
 	}
 
+	@Override
+	public boolean
+	isUIThread()
+	{
+		return( Utils.isSWTThread());
+	}
+	
 	@Override
 	public boolean
 	isProgramInstalled(
