@@ -122,6 +122,7 @@ public class FilesView
     new FileCRC32Item(),
     new FileMD5Item(),
     new FileSHA1Item(),
+    new TorrentV2RootHashItem(),
     new FileAvailabilityItem(),
     new AlertsItem(  TableManager.TABLE_TORRENT_FILES ),
     new FileReadSpeedItem(),
@@ -552,9 +553,29 @@ public class FilesView
 	filterCheck(
 		DiskManagerFileInfo ds, String filter, boolean regex )
 	{
-		if ( hide_dnd_files && ds.isSkipped()){
+		if ( hide_dnd_files ){
+			
+			if ( ds.isSkipped()){
 
-			return( false );
+				return( false );
+				
+			}else if ( ds instanceof FilesViewNodeInner ){
+				
+					// see if all kids skipped
+				
+				if (((FilesViewNodeInner)ds).getSkippedState() == 0 ){
+					
+					return( false );
+				}
+			}else{
+				
+				TOTorrentFile tf = ds.getTorrentFile();
+				
+				if ( tf != null && tf.isPadFile()){
+					
+					return( false );
+				}
+			}
 		}
 
 		if ( filter == null || filter.length() == 0 ){
@@ -577,10 +598,26 @@ public class FilesView
 			return( true );
 		}
 		
-		try {
+		try {			
 			File file = ds.getFile(true);
 
-			String name = filter.contains( File.separator )?file.getAbsolutePath():file.getName();
+			String name;
+			
+			if ( filter.startsWith( File.separator )){
+				
+				filter = filter.substring( 1 );
+				
+				if ( filter.isEmpty()){
+					
+					return( true );
+				}
+				
+				name = file.getAbsolutePath();
+				
+			}else{
+				
+				name = file.getName();
+			}
 
 			String s = regex ? filter : "\\Q" + filter.replaceAll("[|;]", "\\\\E|\\\\Q") + "\\E";
 
@@ -668,15 +705,21 @@ public class FilesView
 			
 			List<FilesViewNodeInner>	nodes = Arrays.asList( node );
 			
-			if ( node.isExpanded()){
-								
-				doTreeAction(nodes, 1, false );
+			try{
+				tv.setRedrawEnabled( false );
 				
-			}else{
+				if ( node.isExpanded()){
 									
-				doTreeAction(nodes, 0, false );
+					doTreeAction(nodes, 1, false );
+					
+				}else{
+										
+					doTreeAction(nodes, 0, false );
+				}
+			}finally{
+				
+				tv.setRedrawEnabled( true );
 			}
-			
 			return;
 			
 		}else if ( fileInfo.getIndex() == -1 ){
@@ -1956,6 +1999,11 @@ public class FilesView
 		{
 			return( false );
 		}
+		
+		@Override
+		public boolean exists(){
+			return( false );
+		}
 
 		@Override
 		public String getLastError(){
@@ -1985,7 +2033,7 @@ public class FilesView
 
 		@Override
 		public boolean
-		setStorageType(int type )
+		setStorageType(int type, boolean force )
 		{
 			return( false );
 		}
@@ -2359,9 +2407,9 @@ public class FilesView
 
 		@Override
 		public boolean
-		setStorageType(int type )
+		setStorageType(int type, boolean force )
 		{
-			return( delegate.setStorageType(type));
+			return( delegate.setStorageType(type, force));
 		}
 
 		@Override
@@ -2469,6 +2517,11 @@ public class FilesView
 			}
 		}
 
+		@Override
+		public boolean exists(){
+			return( delegate.exists());
+		}
+		  
 		@Override
 		public int
 		getIndex()
