@@ -705,7 +705,18 @@ public class GeneralView
     gridData.horizontalSpan = 3;
     lblComment.setLayoutData(gridData);
 
-
+    ClipboardCopy.addCopyToClipMenu( 
+    	lblComment,
+    	new ClipboardCopy.copyToClipProvider(){
+			
+			@Override
+			public String getText(){
+				String comment = (String)lblComment.getData( "comment" );
+				
+				return( comment==null?"":comment );
+			}
+		});
+    
     piecesImage.addListener(SWT.Paint, new Listener() {
       @Override
       public void handleEvent(Event e) {
@@ -826,7 +837,13 @@ public class GeneralView
 
     	piecesDoneAndSum = pm.getPiecePicker().getNbPiecesDone() + "/" + piecesDoneAndSum;
 
-    	distributedCopies = new DecimalFormat("0.000").format(pm.getPiecePicker().getMinAvailability()-pm.getNbSeeds()-(pm.isSeeding()&&stats.getDownloadCompleted(false)==1000?1:0));
+    	float dc = pm.getPiecePicker().getMinAvailability()-pm.getNbSeeds()-(pm.isSeeding()&&stats.getDownloadCompleted(false)==1000?1:0);
+    	
+    	if ( dc < 0 ){
+    		dc = 0;
+    	}
+    	
+    	distributedCopies = new DecimalFormat("0.000").format( dc );
     }
 
 
