@@ -482,8 +482,22 @@ public class MyTorrentsView
     	
     }else if ( filterParent != null ){
 			
-		Object x = filterParent.getData( "ViewUtils:ViewTitleExtraInfo" );
-
+    	Composite comp = filterParent;
+    	
+    	Object x = null;
+    	
+    	while( comp != null ){
+    		
+    		x = comp.getData( "ViewUtils:ViewTitleExtraInfo" );
+    		
+    		if ( x != null ){
+    			
+    			break;
+    		}
+    		
+    		comp = comp.getParent();
+    	}
+    	
 		if ( x instanceof ViewUtils.ViewTitleExtraInfo ){
 			
 			vtxi = (ViewUtils.ViewTitleExtraInfo)x;
@@ -2750,7 +2764,9 @@ public class MyTorrentsView
 
 						for ( Tag t: pending_tag_changes ){
 				
-							boolean should_be_visible	= (filterOnly?t.getFlag( Tag.FL_IS_FILTER ):t.isVisible())&& !hiddenTags.contains( t );
+							boolean manual = t.getTagType().getTagType() == TagType.TT_DOWNLOAD_MANUAL;
+							
+							boolean should_be_visible	= (manual&&filterOnly?t.getFlag( Tag.FL_IS_FILTER ):t.isVisible())&& !hiddenTags.contains( t );
 							boolean is_visible			= allTags.contains( t );
 
 							if ( should_be_visible != is_visible ){
@@ -3327,7 +3343,10 @@ public class MyTorrentsView
 				int pos = 0;
 				for (int i = 0; i < files.length; i++) {
 					DiskManagerFileInfo fileInfo = files[i];
-					if (fileInfo.isSkipped()
+					if ( fileInfo.getTorrentFile().isPadFile()){
+						continue;
+					}
+					if (	fileInfo.isSkipped()
 							&& (fileInfo.getStorageType() == DiskManagerFileInfo.ST_COMPACT || fileInfo.getStorageType() == DiskManagerFileInfo.ST_REORDER_COMPACT)) {
 						continue;
 					}
