@@ -27,6 +27,7 @@ import com.biglybt.core.networkmanager.admin.NetworkAdmin;
 import com.biglybt.core.peer.PEPeer;
 import com.biglybt.core.peer.PEPeerManager;
 import com.biglybt.core.speedmanager.SpeedLimitHandler;
+import com.biglybt.core.tag.TagGroup;
 import com.biglybt.core.util.*;
 import com.biglybt.pifimpl.local.PluginCoreUtils;
 import com.biglybt.plugin.net.buddy.BuddyPlugin;
@@ -75,6 +76,7 @@ PeersViewBase
 				new ProtocolItem(table_id),
 				new PiecesItem(table_id),
 				new PercentItem(table_id),
+				new PercentHaveWeNeedItem(table_id),
 				new DownSpeedItem(table_id),
 				new UpSpeedItem(table_id),
 				new PeerSourceItem(table_id),
@@ -329,7 +331,6 @@ PeersViewBase
 
 			tcManager.setDefaultColumnNames( TableManager.TABLE_ALL_PEERS, basicItems );
 
-
 			tv = TableViewFactory.createTableViewSWT(PLUGIN_DS_TYPE,
 					TableManager.TABLE_ALL_PEERS, getPropertiesPrefix(), basicItems,
 					"connected_time", SWT.MULTI | SWT.FULL_SELECTION | SWT.VIRTUAL);
@@ -348,6 +349,16 @@ PeersViewBase
 		return tv;
 	}	
 	
+	@Override
+	public void
+	tableViewTabInitComplete()
+	{
+		if ( tv.getParentDataSource() instanceof TagGroup ){
+		
+			tv.setEnabled( false );
+		}
+	}
+	
 	private static void registerPluginViews() {
 		ViewManagerSWT vm = ViewManagerSWT.getInstance();
 		if (vm.areCoreViewsRegistered(PLUGIN_DS_TYPE)) {
@@ -379,8 +390,10 @@ PeersViewBase
 		Map<String, Object> data) 
 	{
 		switch (eventType) {
-			case EVENT_TABLELIFECYCLE_INITIALIZED:
+			case EVENT_TABLELIFECYCLE_INITIALIZED:{
 				shell = this.tv.getComposite().getShell();
+				break;
+			}
 		}
 	}
 	
@@ -430,6 +443,8 @@ PeersViewBase
 					}
 				});
 		
+		swarm_view.setAlwaysShowDownloadName( true );
+		
 		swarm_view.initialize( parent );
 		
 		swarm_view.getComposite().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
@@ -532,13 +547,13 @@ PeersViewBase
 				}
 				break;
 			}
-			case UISWTViewEvent.TYPE_FOCUSGAINED:{
+			case UISWTViewEvent.TYPE_SHOWN:{
 				if ( swarm_view != null ) {
 					swarm_view.setFocused( true );
 				}
 				break;
 			}				
-			case UISWTViewEvent.TYPE_FOCUSLOST:{
+			case UISWTViewEvent.TYPE_HIDDEN:{
 				if ( swarm_view != null ) {
 					swarm_view.setFocused( false );
 				}
