@@ -1600,9 +1600,24 @@ public class SB_Transfers
 
 							String prev_id = getTagPosition( mdi, parent_id, tag_type, tag_group );
 							
-							MdiEntry entry = mdi.createEntryFromSkinRef(
+							MdiEntry entry;
+							
+							if ( tag.getTaggableTypes() == Taggable.TT_DOWNLOAD ){
+							
+								entry = mdi.createEntryFromSkinRef(
 									parent_id, group_id, "library", tag_group, viewTitleInfo, tag.getGroupContainer(), false, prev_id );
 							
+							}else{
+								
+								UISWTViewBuilderCore builder = new UISWTViewBuilderCore( group_id, null,
+										PeersGeneralView.class);
+								builder.setParentEntryID(parent_id);
+								builder.setPreferredAfterID(prev_id).setInitialDatasource( tag.getGroupContainer() );
+								entry = mdi.createEntry(builder, false);
+
+								entry.setViewTitleInfo( viewTitleInfo );
+							}
+							
 							setTagIcon( tag, entry, true );
 							
 							entry.setUserData( TAG_TAG_OR_GROUP_KEY, tag.getGroupContainer());
@@ -1805,25 +1820,45 @@ public class SB_Transfers
 						if (dms.isEmpty()) {
 							return;
 						}
+						
 						boolean doAdd = false;
-						for (DownloadManager dm : dms) {
-							if (!tag.hasTaggable(dm)) {
-								doAdd = true;
-								break;
+
+						int mods = Utils.getDragDetectModifiers();
+						
+						if (( mods & SWT.ALT ) != 0 ){
+							
+								/// alt+drop -> addl shift+alt+drop -> remove otherwise existing togglish logic
+							
+							doAdd = ( mods & SWT.SHIFT ) == 0 ;
+							
+						}else{
+							
+							for (DownloadManager dm : dms) {
+								if (!tag.hasTaggable(dm)) {
+									doAdd = true;
+									break;
+								}
 							}
 						}
 
 						boolean[] auto = tag.isTagAuto();
-						if (auto.length < 2 || (doAdd && auto[0])
-							|| (!doAdd && auto[0] && auto[1])) {
+						
+						if ( 	auto.length < 2 || 
+								(doAdd && auto[0]) || 
+								(!doAdd && auto[0] && auto[1])){
+							
 							return;
 						}
 
 						for (DownloadManager dm : dms) {
 							if ( doAdd ){
-								tag.addTaggable( dm );
+								if ( !tag.hasTaggable( dm )){
+									tag.addTaggable( dm );
+								}
 							}else{
-								tag.removeTaggable( dm );
+								if ( tag.hasTaggable( dm )){
+									tag.removeTaggable( dm );
+								}
 							}
 						}
 					}
