@@ -64,7 +64,7 @@ import com.biglybt.util.MapUtils;
 public class SideBarEntrySWT
 	extends BaseMdiEntry
 {
-	private static final boolean DARK_MODE = Utils.isDarkAppearance();
+	private static final boolean DARK_MODE = Utils.isDarkAppearanceNative();
 	
 	private static final boolean PAINT_BG = !Constants.isUnix;
 
@@ -583,9 +583,15 @@ public class SideBarEntrySWT
 
 		triggerOpenListeners();
 
-		swtItem.getParent().select(swtItem);
-		swtItem.getParent().showItem(swtItem);
+		Tree tree = swtItem.getParent();
+		
+		tree.select(swtItem);
+		tree.showItem(swtItem);
 
+		if ( Constants.isOSX ){
+			tree.redraw();	// issue up to at least 4934r6 whereby tree is not being repainted after being scrolled via 'showItem'
+		}
+		
 		super.show();
 	}
 
@@ -801,6 +807,17 @@ public class SideBarEntrySWT
 					maxIndicatorWidth = width;
 				}
 				gc.setForeground(text_color);
+				
+				
+				// I REALLY can't explain this, but on OSX if the textIndicator is '0' then it is getting rendered in
+				// BLACK even when the text_color is set to WHITE. I've spend ages looking into it and got nowhere. Interestingly
+				// if I prefix it with a zero-width unicode space the problem is solved.
+				// Feel free to try and figure it out if you have time on your hands...
+				
+				if ( Constants.isOSX ){
+					textIndicator = "\u200b" + textIndicator;
+				}
+				
 				GCStringPrinter.printString(gc, textIndicator, new Rectangle(startX,
 						startY + textOffsetY, width, height), true, false, SWT.CENTER);
 			}
