@@ -27,7 +27,6 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.dnd.*;
 import org.eclipse.swt.events.*;
 import org.eclipse.swt.graphics.*;
-import org.eclipse.swt.internal.DPIUtil;
 import org.eclipse.swt.layout.*;
 import org.eclipse.swt.widgets.*;
 
@@ -87,7 +86,7 @@ public class TableViewPainted
 	private static final boolean DEBUG_WITH_SHELL = false;
 
 	public static final boolean DIRECT_DRAW = (Constants.isOSX
-			|| Constants.isUnix) && DPIUtil.getDeviceZoom() != 100;
+			|| Constants.isUnix) && Utils.getDeviceZoom() != 100;
 
 	private static final boolean DEBUG_REDRAW_CLIP = false;
 
@@ -163,7 +162,8 @@ public class TableViewPainted
 	private int totalHeight = 0;
 
 	private boolean redrawTableScheduled;
-
+	private int		redrawTableDisabled;
+	
 	private ScrollBar hBar;
 
 	private ScrollBar vBar;
@@ -358,6 +358,9 @@ public class TableViewPainted
 						filter.widget.setText( "" );
 					}
 				}
+				if ( getComposite().isDisposed()){
+					return;
+				}
 				if (getComposite() != event.widget) {
 					if ( getComposite().isVisible()){
 							// we only want super to get the opportunity to consume the event if it is visible
@@ -1391,7 +1394,7 @@ public class TableViewPainted
 			defaultRowHeight = minRowHeight;
 		}
 
-		cTable.setBackground(Colors.getSystemColor(parent.getDisplay(), SWT.COLOR_LIST_BACKGROUND));
+		cTable.setBackground(TablePaintedUtils.getColour(parent.getDisplay(), SWT.COLOR_LIST_BACKGROUND));
 
 		clientArea = cTable.getClientArea();
 
@@ -1748,14 +1751,13 @@ public class TableViewPainted
 							gc.setBackground(color);
 						}
 						if (color == null) {
-							gc.setBackground(gc.getDevice().getSystemColor(
-									SWT.COLOR_LIST_BACKGROUND));
+							gc.setBackground(TablePaintedUtils.getColour(gc, SWT.COLOR_LIST_BACKGROUND));
 						}
 						gc.fillRectangle(drawBounds.x, yDirty, drawBounds.width, rowHeight);
 						yDirty += rowHeight;
 					}
 				} else {
-					gc.setBackground(gc.getDevice().getSystemColor(cTable.isEnabled() ?
+					gc.setBackground(TablePaintedUtils.getColour(gc,cTable.isEnabled() ?
 							SWT.COLOR_LIST_BACKGROUND : SWT.COLOR_WIDGET_BACKGROUND));
 					gc.fillRectangle(drawBounds.x, yDirty, drawBounds.width, h);
 				}
@@ -1765,7 +1767,12 @@ public class TableViewPainted
 			Utils.setClipping(gc, drawBounds);
 			TableColumnCore[] visibleColumns = getVisibleColumns();
 			int x = DIRECT_DRAW ? -clientArea.x : 0;
-			gc.setAlpha(20);
+			if ( TablePaintedUtils.isDark()){
+				gc.setAlpha(120);
+				gc.setForeground( TablePaintedUtils.getColour(gc,SWT.COLOR_WIDGET_BACKGROUND ));
+			}else{
+				gc.setAlpha(20);
+			}
 			for (TableColumnCore column : visibleColumns) {
 				x += column.getWidth();
 	
@@ -1781,7 +1788,7 @@ public class TableViewPainted
 
 	private Color getColorLine() {
 		if (colorLine == null) {
-			colorLine = Colors.getSystemColor(cTable.getDisplay(), SWT.COLOR_LIST_BACKGROUND);
+			colorLine = TablePaintedUtils.getColour(cTable.getDisplay(), SWT.COLOR_LIST_BACKGROUND);
 			HSLColor hslColor = new HSLColor();
 			hslColor.initHSLbyRGB(colorLine.getRed(), colorLine.getGreen(),
 					colorLine.getBlue());
@@ -3212,11 +3219,39 @@ public class TableViewPainted
 		});
 	}
 
-	public void redrawTable() {
+	@Override
+	public void 
+	setRedrawEnabled(
+		boolean enabled)
+	{
 		synchronized (TableViewPainted.this) {
-			if (redrawTableScheduled) {
+			
+			if ( enabled ){
+				
+				redrawTableDisabled--;
+				
+			}else{
+				
+				redrawTableDisabled++;
+			}
+		}
+		
+		if ( enabled ){
+			
+			redrawTable();
+		}
+	}
+	
+	public void 
+	redrawTable()
+	{
+		synchronized (TableViewPainted.this) {
+			
+			if ( redrawTableScheduled || redrawTableDisabled > 0 ){
+				
 				return;
 			}
+			
 			redrawTableScheduled = true;
 		}
 
