@@ -24,6 +24,7 @@ package com.biglybt.ui.swt.views.tableitems.files;
 
 import java.io.File;
 import java.io.IOException;
+import java.util.Collections;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.Color;
@@ -35,6 +36,7 @@ import org.eclipse.swt.graphics.Rectangle;
 import com.biglybt.core.config.COConfigurationManager;
 import com.biglybt.core.config.ParameterListener;
 import com.biglybt.core.disk.DiskManagerFileInfo;
+import com.biglybt.core.download.DownloadManager;
 import com.biglybt.core.internat.MessageText;
 import com.biglybt.core.util.Constants;
 import com.biglybt.core.util.Debug;
@@ -49,6 +51,7 @@ import com.biglybt.ui.swt.imageloader.ImageLoader;
 import com.biglybt.ui.swt.shells.GCStringPrinter;
 import com.biglybt.ui.swt.shells.MessageBoxShell;
 import com.biglybt.ui.swt.views.FilesView;
+import com.biglybt.ui.swt.views.FilesViewMenuUtil;
 import com.biglybt.ui.swt.views.table.CoreTableColumnSWT;
 import com.biglybt.ui.swt.views.table.TableCellSWT;
 import com.biglybt.ui.swt.views.table.TableCellSWTPaintListener;
@@ -73,6 +76,8 @@ public class NameItem extends CoreTableColumnSWT implements
 {
 	private static final String ID_CHECKHITAREA		= "checkHitArea";
 
+	private static final Object	KEY_OLD_CELL_TT	= new Object();
+	
 	private static boolean NEVER_SHOW_TWISTY;
 	
 	static{
@@ -304,7 +309,16 @@ public class NameItem extends CoreTableColumnSWT implements
 			
 			if ( is_leaf ){
 				if (fileInfo.isSkipped()){
-					check_key = "check_no";
+					
+					int	st = fileInfo.getStorageType();
+					
+					if ( 	st == DiskManagerFileInfo.ST_COMPACT ||
+							st == DiskManagerFileInfo.ST_REORDER_COMPACT ){
+						
+						check_key = "check_no";
+					}else{
+						check_key = "check_no_grey";
+					}
 				}else{
 					if ( fileInfo.getLength() == fileInfo.getDownloaded()){
 						check_key = "check_ro_yes";
@@ -549,13 +563,35 @@ public class NameItem extends CoreTableColumnSWT implements
 				Rectangle hitArea = (Rectangle) data;
 				boolean inCheck = hitArea.contains(event.x, event.y);
 
+				String our_tt = MessageText.getString( "filesview.name.check.tt" );
+				
+				Object set_tt = null;
+				
 				if ( inCheck ){
+					
+					final DiskManagerFileInfo fileInfo = (DiskManagerFileInfo)event.cell.getDataSource();
+
+					boolean realFile;
+					
+					if ( fileInfo instanceof FilesView.FilesViewTreeNode ){
+						
+						realFile = ((FilesView.FilesViewTreeNode)fileInfo).isLeaf();
+						
+					}else{
+						
+						realFile = true;
+					}
+					
+					if ( realFile ){
+						
+						set_tt = our_tt;
+					}
+					
 					inArea = true;
 					if (event.eventType == TableCellMouseEvent.EVENT_MOUSEDOWN) {
 						if (row instanceof TableRowCore) {
 							TableRowCore rowCore = (TableRowCore) row;
 							if ( rowCore != null ){
-								final DiskManagerFileInfo fileInfo = (DiskManagerFileInfo)event.cell.getDataSource();
 							
 								if ( fileInfo instanceof FilesView.FilesViewTreeNode ){
 									
@@ -581,11 +617,50 @@ public class NameItem extends CoreTableColumnSWT implements
 									}
 								}
 								
-								ManagerUtils.setFileSkipped( fileInfo, !fileInfo.isSkipped());
+								if ( ( event.keyboardState & SWT.SHIFT ) != 0 && !fileInfo.isSkipped()){
+									
+									FilesViewMenuUtil.changePriority(
+											FilesViewMenuUtil.PRIORITY_DELETE,
+											Collections.singletonList(fileInfo), false );
+								}else{
+								
+									ManagerUtils.setFileSkipped( fileInfo, !fileInfo.isSkipped());
+								}
 							}
 						}
 					}
 				}
+				
+				if ( event.cell instanceof TableCellCore ){
+					
+					TableCellCore cellCore = (TableCellCore)event.cell;
+										
+					Object existingTT = cellCore.getToolTip();
+					
+					if ( existingTT != set_tt ){
+						
+						if ( existingTT instanceof String && set_tt != null && set_tt.equals( existingTT )){
+							
+							// no change
+							
+						}else{
+							
+								// we want to store any existing TTs we get that aren't ours
+							
+							if ( existingTT != null && !existingTT.equals( our_tt )){
+								
+								cellCore.setData( KEY_OLD_CELL_TT, existingTT );
+							}
+							
+							if ( set_tt == null ){
+								
+								set_tt = cellCore.getData(KEY_OLD_CELL_TT );
+							}
+							
+							cellCore.setToolTip( set_tt );
+						}
+					}
+				}
 			}
 			
 			if (event.eventType == TableCellMouseEvent.EVENT_MOUSEMOVE) {
@@ -593,7 +668,7 @@ public class NameItem extends CoreTableColumnSWT implements
 			}
 		}
 	}
-
+	  
 	@Override
 	public boolean inplaceValueSet(TableCell cell, String value, boolean finalEdit) {
 		if (value.equalsIgnoreCase(cell.getText()) || "".equals(value) || "".equals(cell.getText()))
@@ -623,10 +698,19 @@ public class NameItem extends CoreTableColumnSWT implements
 		FileUtil.runAsTask(new CoreOperationTask()
 		{
 			@Override
-			public String getName(){
-				// TODO Auto-generated method stub
+			public String 
+			getName()
+			{
 				return fileInfo.getFile(true).getName();
 			}
+			
+			@Override
+			public DownloadManager
+			getDownload()
+			{
+				return( fileInfo.getDownloadManager());
+			}
+			
 			@Override
 			public void run(CoreOperation operation) {
 				result[0] = fileInfo.setLink(target);
